<?php
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
require_once 'SearchSettings.php';
require_once SearchSettings::ZEND_PATH;
require_once 'TextParser.php';
require_once 'Zend/Feed.php';

class FeedGenerator {
	
	protected $indexPath;
	protected $searchString;
	protected $findElementsOfType;
	protected $maxResultsPerType;
	protected $feedType;
	protected $tags;
	protected $title;
	protected $desc;
	protected $link;
	protected $copyright;
	protected $language;
	protected $docBase;
	protected $docFolder;
	protected $searchResults;
	protected $hitsCount;
	protected $phpPageName;
	protected $dispatherPage;
	protected $isCJKAnalyzer;
	protected $umaTypeArray = array();

	public function __construct($dispatherPage, $searchString, $findElementsOfType, $maxResultsPerType, $feedType, $tags, $title, $desc, $link, $copyright, $language)
	{
		$this->dispatherPage = $dispatherPage;
		$this->searchString = $searchString;
		$this->findElementsOfType = $findElementsOfType; 
		$this->maxResultsPerType = $maxResultsPerType; 
		$this->feedType = $feedType=='atom'?'atom':'rss';
		$this->tags = $tags; 
		$this->title = $title==null?'':$title; 
		$this->desc = $desc==null?'':$desc; 
		$this->link = $link==null?'http://w3.ibm.com':$link; 
		$this->copyright = $copyright==null?'':$copyright; 
		$this->language = $language==null?'en_us':$language;
		$this->_initialize();
		$this->_doSearch();
	}
	
	private function _initialize()
	{
		$this->indexPath = realpath('./index/');
		$version = file($this->indexPath."/version.txt");
		if (preg_match('/^CJKAnalyzer$/u', $version[1]))
		{
			$this->isCJKAnalyzer = true;
		}
		else
		{
			$this->isCJKAnalyzer = false;
		}
		$this->docFolder = explode('*',$version[0]);
		$this->docFolder = $this->docFolder[0];
		$this->phpPageName = 'http://'.$_SERVER['SERVER_NAME'].':'.$_SERVER["SERVER_PORT"].$_SERVER['PHP_SELF'];
		
		$this->docBase = substr($this->phpPageName, 0, strrpos($this->phpPageName, '/'));
		$this->docBase = substr($this->docBase, 0, strrpos($this->docBase, '/'));
	}
	
	private function _doSearch()
	{
		if ($this->_isInvalidSearchString($this->searchString))
		{
			return;
		}
		
		Zend_Search_Lucene_Analysis_Analyzer::setDefault(new TextAnalyzer($this->isCJKAnalyzer));
		Zend_Search_Lucene::setDefaultSearchField('contents');
		$index = Zend_Search_Lucene::open($this->indexPath);
		$group = array();

		if ($this->searchString!=null)
		{
			$strQuery = array(
								'queryString' => $this->searchString,
								'sign' => true
						);
			array_push($group, $strQuery);
		}

		if ($this->tags!=null)
		{
			$tagQuery = array(
								'queryString' => $this->getTagsCondition(),
								'sign' => true
						);
			array_push($group, $tagQuery);
		}
		
		if ($this->findElementsOfType!=null)
		{
			$typeQuery = array(
								'queryString' => 'uma.type:'.$this->findElementsOfType,
								'sign' => true
						);
			array_push($group, $typeQuery);
		}
		
		$query = TextParser::parseByGroup($group, 'UTF-8');
		$hits = $index->find($query);
		$this->hitsCount = count($hits);
		$resultCount = 0;
		for ($i = 0; $i < $this->hitsCount; $i++)
		{
			$hit = $hits[$i];
			$document = $hit->getDocument();
			$url =  $document->getFieldValue('url');
			$fixedUrl = $this->docBase.substr($url, strpos($url, $this->docFolder)+ strlen($this->docFolder));
			$title = $document->getFieldValue('title');
			$summary = $document->getFieldValue('summary');;
			$umaType = $document->getFieldValue('uma.type');
			if (isset($this->umaTypeArray[$umaType]))
			{
				$this->umaTypeArray[$umaType]++;
			}
			else
			{
				$this->umaTypeArray[$umaType] = 1;
			}
			
			if (($this->maxResultsPerType!=null)&&($this->umaTypeArray[$umaType]>$this->maxResultsPerType))
			{
				continue;
			}
			
			$this->searchResults[$resultCount]['url'] = $this->urlEscape($fixedUrl);
			$this->searchResults[$resultCount]['title'] = $title;
			$this->searchResults[$resultCount]['summary'] = $summary;
			$this->searchResults[$resultCount]['uma.type'] = $umaType;
			$resultCount++;
		}
	}
	
	private function getTagsCondition()
	{
		$condition = '';
		$tagArray = explode(',',$this->tags);
		foreach ($tagArray as $tag)
		{
			$condition .= ' tags:'.$tag;
		}
		return $condition;
	}
	
	public function getResults()
	{
		try {
			$entries = array();
			
			for($i = 0; $i < count($this->searchResults); $i++)
			{
				$entry = array(
					'title'			=>	$this->searchResults[$i]['title'],
					'link'			=>	$this->searchResults[$i]['url'],
					'description'  	=>	$this->searchResults[$i]['summary'],
					'category'		=>	array(
											array(
												// required
												'term' => $this->searchResults[$i]['uma.type'])
										)
				);
				
				array_push($entries, $entry);
			}
			
			$rss = array(
				'title'			=>	$this->title,
				'description'	=>	$this->desc,
				'link'        	=>	$this->link,
				'charset'		=>	'utf-8',
				'language'		=>	$this->language,
				'copyright'		=>	$this->copyright,
				'entries'		=>	$entries
			);

			$feed = Zend_Feed::importArray($rss, $this->feedType);
			$result = $feed->saveXML();
		}  catch(Exception $e) {
			$result = SearchSettings::SEARCH_FAILED;
		}
		return $result;
	}
	
	protected function urlEscape($url)
	{
		$result = str_replace('\\', '/', $url);
		return $result;
	}
	
	private function _isInvalidSearchString($str)
	{
		if (($str==null)||(trim($str)=='')||(iconv_strlen($str, 'UTF-8')==1))
		{
			return true;
		}
		return false;
	}
}

?>