package com.rational.rpw.compositetreeview;

import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.metal.MetalTreeUI;
import javax.swing.tree.TreePath;

/**
 * MetalTreeUI that handles drag and drop of multiple 
 * selection (this was originally posted by keithjohnson
 * under bug ID 4165577 in the Java bug parade).
 */
public class MultiDragMetalTreeUI extends MetalTreeUI
{
    public static ComponentUI createUI(JComponent x) 
    {
        return new MultiDragMetalTreeUI();
    }
    
    protected MouseListener createMouseListener() 
    {
        return new MouseHandler();
    }

    public class MouseHandler extends MouseAdapter  
    {
        private TreePath alreadySelectedPath = null;
        
        // Handles mouse press.  If path is already 
        // selected, saves path and applies selection/deselection 
        // when mouse is released (if released above saved path).        
        public void mousePressed(MouseEvent e) 
        {
            if (getTree() != null && getTree().isEnabled()) 
            {
                getTree().requestFocus();
                TreePath path = getClosestPathForLocation(getTree(), 
                                                          e.getX(),
                                                          e.getY());

                if (path != null) 
                {
                    Rectangle bounds = getPathBounds(getTree(), path);
                    if (e.getY() > (bounds.y + bounds.height)) 
                    {
                        return;
                    }

                    if (SwingUtilities.isLeftMouseButton(e))
                        checkForClick(path, e.getX(), e.getY());

                    int x = e.getX();

                    if (x > bounds.x) 
                    {
                        if (x <= (bounds.x + bounds.width) && 
                            !startEdit(path, e)) 
                        {
                            if (getTree().isPathSelected(path)) 
                            {
                                alreadySelectedPath = path;
                            } 
                            else 
                            {
                                alreadySelectedPath = null;
                                selectPath(path, e);
                            }
                        }
                    }
                }
            }
        }
        
        // Handles mouse release.  If released above 
        // previously selected path, applies selection/deselection.         
        public void mouseReleased(MouseEvent e) 
        {
            if (getTree() != null && getTree().isEnabled()) 
            {
                TreePath path = getClosestPathForLocation(getTree(), e.getX(), e.getY());
              
                if (path != null) 
                {
                    Rectangle bounds = getPathBounds(getTree(), path);

                    if (e.getY() > (bounds.y + bounds.height)) 
                    {
                        return;
                    }
                    
                    int x = e.getX();

                    if (x > bounds.x && 
                        x <= (bounds.x + bounds.width) &&
                        alreadySelectedPath == path) {
                            selectPath(path, e);
                    }
                }
            }
            alreadySelectedPath = null;            
        }  
    } 

    protected JTree getTree()
    {
        return tree;
    }

    protected void checkForClick(TreePath path, int mouseX, int mouseY)
    {
        checkForClickInExpandControl(path, mouseX, mouseY);
    }

    protected boolean startEdit(TreePath path, MouseEvent e)
    {
        return startEditing(path,e);
    }

    protected void selectPath(TreePath path, MouseEvent e)
    {
        selectPathForEvent(path, e);
    }
    
	protected void selectPathForEvent(TreePath path,
                                  		MouseEvent event)
	{
		super.selectPathForEvent( path, event );
		// add to selection if right mouse click and single node only
		// this overrides the default behaviour where right-mouse click
		// does not select the node
		if( SwingUtilities.isRightMouseButton( event ) &&
		    getTree().getSelectionCount() < 2 )
		{				
            TreePath treePath = getClosestPathForLocation(getTree(), 
                                                          event.getX(),
                                                          event.getY());
            getTree().setSelectionPath( treePath );
		}
	}
}