package com.rational.rpw.compositetreeview;

import javax.swing.JPopupMenu;
import javax.swing.tree.MutableTreeNode;
import java.util.Vector;

/**
 * This abstract class allows the tree to retrieve the
 * popup menu for a specific node.  It is expected that 
 * subclasses will provide the concrete menus and 
 * menu handling logic.
 */
public abstract class TreePopupMenuAdapter
{
    // change listener to notify of node changes
    protected TreeChangeListener _treeChangeListener = null;
    
    /**
     * Constructor.
     */
    public TreePopupMenuAdapter()
    {
    }
    
    /**
     * Sets the change listener.
     */
    public void setTreeChangeListener( TreeChangeListener listener )
    {
        _treeChangeListener = listener;    
    }
    
    /**
     * Checks whether a menu is available for this node.
     * @param   node        The node for which the menu is available or not.
     * @return  boolean     <code>true</code> if so, <code>false</code> if not.
     */
    public abstract boolean isMenuAvailable( MutableTreeNode node );
    
    /**
     * Checks whether a menu is available for multiple nodes.
     * @param   node        The node for which the menu is available or not.
     * @return  boolean     <code>true</code> if so, <code>false</code> if not.
     */
    public abstract boolean isMenuAvailable( Vector nodes );
    
    /**
     * Gets the popup menu for the node.
     * @param   node        The node for the popup menu.
     * @return  JPopupMenu  The popup menu for the node.
     */
    public abstract JPopupMenu getMenu( MutableTreeNode node );
    
    /**
     * Gets the popup menu for the multiple nodes.
     * @param   node        The node for the popup menu.
     * @return  JPopupMenu  The popup menu for the node.
     */
    public abstract JPopupMenu getMenu( Vector nodes );
    
}