package com.rational.rpw.dnd;

import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import java.io.*;

import com.rational.rpw.dnd.datatransfer.*;

/**
 * This class implements the DragGestureListener interface.
 * It acts as a listener that will start the drag operation.
 * It maintains a reference to an IDragComponent which will be
 * queried for the transferable and acceptable actions.
 */
public class DragGestureAdapter implements DragGestureListener, Serializable
{
    private IDragComponent _dragComponent = null;
    private Cursor _cursor = DragSource.DefaultMoveDrop;

    /**
     * Constructor.
     * @param   component   The implementer of IDragComponent.
     */
    public DragGestureAdapter( IDragComponent component )
    {
        _dragComponent = component;
    }

    /**
     * Constructor.
     * @param   component   The implementer of IDragComponent.
     * @param   cursor      The cursor to use for the drag operation.
     */
    public DragGestureAdapter( IDragComponent component,
                               Cursor cursor )
    {
	_dragComponent = component;
	_cursor = cursor;
    }

    /**
     * Start the drag if the operation is ok.
     * @param e the event object
     */
    public void dragGestureRecognized( DragGestureEvent e )
    {
	    int dragAction = _dragComponent.getDragAction();
	    int eventDragAction = e.getDragAction();
	    // make sure copy or move are allowed
	    if( dragAction == DnDConstants.ACTION_MOVE && eventDragAction == DnDConstants.ACTION_COPY_OR_MOVE )
	    {
	    }
	    else if( dragAction == DnDConstants.ACTION_COPY && eventDragAction == DnDConstants.ACTION_COPY_OR_MOVE )
	    {
	    }
	    else if( dragAction == DnDConstants.ACTION_COPY_OR_MOVE && eventDragAction == DnDConstants.ACTION_COPY )
	    {
	    }
	    else if( dragAction == DnDConstants.ACTION_COPY_OR_MOVE && eventDragAction == DnDConstants.ACTION_MOVE )
	    {
	    }
	    else if( (eventDragAction & dragAction) == 0 )
	    {
	        return;
	    }

	    if( !_dragComponent.isStartDragOk( e ) )
	        return;

	    try
	    {
	        Transferable transferable =
	            _dragComponent.getTransferable( e.getDragOrigin() );

                // store the transferable object
                TransferableStore.setTransferableObject( transferable );

                // start the drag
	        DragSourceListener dsListener = _dragComponent.getDragSourceListener();
	        e.startDrag( _cursor,
		             transferable,
		             dsListener );

	    }
	    catch( InvalidDnDOperationException idoe )
	    {
	        idoe.printStackTrace();
	    }
    }


}
