package com.rational.rpw.dnd;

import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import java.io.*;

import com.rational.rpw.dnd.datatransfer.*;

/**
 * This class implements the DragSourceListener interface
 * and tracks the state of the drag and drop operation.
 * It maintains a reference to an IDragComponent object.
 */

public class DragSourceAdapter implements DragSourceListener, Serializable
{
    // reference to IDragComponent
    protected IDragComponent _dragComponent = null;

    /**
     * Constructor.
     */
    public DragSourceAdapter( IDragComponent c ) 
    {
	    _dragComponent = c;
    }

    // DropSourceListener interface methods
    /**
     * invoked when drag and drop operation is complete
     */
    public void dragDropEnd( DragSourceDropEvent e ) 
    {
	    if( e.getDropSuccess() ) 
	    {
            // move the component if requested by the drop target
        	int dragAction = _dragComponent.getDragAction();
	        if( ( e.getDropAction() & DnDConstants.ACTION_MOVE ) != 0 &&
	            ( dragAction & DnDConstants.ACTION_MOVE) != 0 ) 
	        {
	            _dragComponent.move();
    	    }
	    }
    }

    /**
     * called as the cursor's hotspot enters a platform-dependent drop site
     */
    public void dragEnter( DragSourceDragEvent e ) 
    {
	    setDragOverFeedback( e );
    }

    /**
     * called as the cursor's hotspot moves over a platform-dependent drop site
     */
    public void dragOver( DragSourceDragEvent e ) 
    {
	    setDragOverFeedback( e );
    }

    /**
     * called as the cursor's hotspot exits a platform-dependent drop site
     */
    public void dragExit( DragSourceEvent e ) 
    {
	    DragSourceContext context = e.getDragSourceContext();
	    context.setCursor(null); 	    
	    context.setCursor(DragSource.DefaultCopyNoDrop); 
    }

    /**
     * called when the user has modified the drop gesture   
     */
    public void dropActionChanged( DragSourceDragEvent e ) 
    {
	    setDragOverFeedback( e );
    }
    
    /*
     * Set the drag over feedback.
     */
    private void setDragOverFeedback( DragSourceDragEvent e ) 
    {
	    DragSourceContext context = e.getDragSourceContext();
	    int dropOp = e.getDropAction();
	    int targetAct = e.getTargetActions();
	    int ra = dropOp & targetAct;
	    Cursor c = null;

	    if( ra == DnDConstants.ACTION_NONE ) 
	    { 
	        // no drop possible
	        if( (dropOp & DnDConstants.ACTION_MOVE) == DnDConstants.ACTION_MOVE )
		        c = DragSource.DefaultMoveNoDrop;
	        else
		        c = DragSource.DefaultCopyNoDrop;
	    } 
	    else 
	    { 
	        // drop possible
	        if( (ra & DnDConstants.ACTION_MOVE) == DnDConstants.ACTION_MOVE ) 
	        {
		        c = DragSource.DefaultMoveDrop;
	        } 
	        else 
	        {
		        c = DragSource.DefaultCopyDrop;
	        }
	    }

        context.setCursor( null ); 
	    context.setCursor( c ); 
    }
    
}
