package com.rational.rpw.dnd;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

import com.rational.rpw.dnd.datatransfer.*;

/**
 * This interface can be implemented to be able to act as
 * a drag source with a DragSourceAdapter.
 */
public interface IDragComponent 
{
    /**
     * Called by the DragSourceAdapter in dragDropEnd if the operation
     * is DnDConstants.ACTION_MOVE 
     */
    public void move();

    /**
     * What are the drag actions supported by this component. Could be
     * DnDConstants.ACTION_MOVE, DnDConstants.ACTION_COPY or
     * DnDConstants.ACTION_COPY_OR_MOVE.  Called by the
     * DragSourceAdapter in dragDropEnd and the DragGestureAdapter in
     * dragGestureRecognized 
     */
    public int getDragAction();

    /**
     * Called by the DragGestureAdapter in dragGestureRecognized
     */
    public Transferable getTransferable( Point point );

    /**
     * Called by the DragGestureAdapter in dragGestureRecognized. This
     * is the listener that is registered with the drag source in the
     * startDrag method.
     *@returns usually the DragSourceAdapter 
     */
    public DragSourceListener getDragSourceListener();

    /**
     * Is the location within the component draggable?  Called by the
     * DragGestureAdapter in dragGestureRecognized. If false is
     * returned then there will be no drag initiated.  
     */
    public boolean isStartDragOk( DragGestureEvent dragGestureEsvent );
}
