package com.rational.rpw.dnd.datatransfer;

import javax.swing.*;
import java.awt.datatransfer.*;
import java.util.List;
import java.util.Arrays;
import java.util.Vector;
import java.io.IOException;
import java.io.File;

/**
 * This class is a wrapper for a list of files for transfer.
 * It provides the data flavor using java.io.File.
 */
public class FileListTransferable implements Transferable
{
    // the permitted data flavor
    public static DataFlavor _fileListFlavor = null;
    static
    {
	    try
	    {
	        //_fileListFlavor = new DataFlavor( DataFlavor.javaJVMLocalObjectMimeType +
                //                                    "; class=java.io.File",
		//	                                  "Local File List");
                _fileListFlavor = DataFlavor.javaFileListFlavor;
    	}
    	catch( Exception e )
    	{
	        e.printStackTrace();
	    }
    }

    // the permitted list of flavors
    public static final DataFlavor[] _flavors = { FileListTransferable._fileListFlavor };
    private static final List _flavorList = Arrays.asList( _flavors );

    // the list of files which are wrapped by this class
    private Vector _fileList = null;
    
    /**
     * Constructor.
     */
    public FileListTransferable( ProcessFile file )
    {
        _fileList = new Vector();
        _fileList.add( file );
    }

    /**
     * Constructor.
     */
    public FileListTransferable( Vector fileList )
    {
        _fileList = fileList;
    }

    /**
     * Retrieves the permitted flavors.
     * @return  an array of DataFlavor objects
     */
    public synchronized DataFlavor[] getTransferDataFlavors()
    {
	    return _flavors;
    }

    /**
     * Checks whether the given flavor is permitted.
     * @return  boolean     <code>true</code> if so, <code>false</code> if not
     */
    public boolean isDataFlavorSupported( DataFlavor flavor )
    {
	    return ( _flavorList.contains( flavor ) );
    }

    /**
     * Retrieves the transferable data.
     * @return  Object  The transferable data (Vector).
     */
    public synchronized Object getTransferData( DataFlavor flavor )
	    throws UnsupportedFlavorException, IOException
	{
    	if( flavor.equals( FileListTransferable._fileListFlavor ) )
    	{
	        return _fileList;
	    }
	    else
	    {
	        throw new UnsupportedFlavorException (flavor);
	    }
    }

    public String toString()
    {
	    return "FileListTransferable";
    }
}

