package com.rational.rpw.dnd.datatransfer;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.datatransfer.*;
import java.util.List;
import java.util.Arrays;
import java.util.Vector;
import java.io.IOException;

/**
 * This class is a wrapper for a list of tree nodes for trasnfer.  
 * It provides the data flavor using DefaultMutableTreeNode.
 */
public class TreeTransferable implements Transferable
{
    // the permitted data flavor
    public static DataFlavor _treeNodeFlavor = null;
    static 
    {
	    try 
	    {
	        _treeNodeFlavor = new DataFlavor( DataFlavor.javaJVMLocalObjectMimeType + 
			                                  "; class=javax.swing.tree.DefaultMutableTreeNode",
			                                  "Local Tree Node View");
    	}
    	catch( Exception e )
    	{
	        e.printStackTrace();
	    }
    }
  
    // the permitted list of flavors
    public static final DataFlavor[] _flavors = { TreeTransferable._treeNodeFlavor };  
    private static final List _flavorList = Arrays.asList( _flavors );

    // the tree nodes which are wrapped by this class
    private Vector _treeNode = null;

    /**
     * Constructor.
     */
    public TreeTransferable( TreeNode treeNode ) 
    {
        _treeNode = new Vector();
        _treeNode.add( treeNode );        
    }

    /**
     * Constructor.
     */
    public TreeTransferable( Vector treeNode ) 
    {
        _treeNode = treeNode;
    }

    /**
     * Retrieves the permitted flavors.
     * @return  an array of DataFlavor objects
     */
    public synchronized DataFlavor[] getTransferDataFlavors() 
    {
	    return _flavors;
    }
    
    /**
     * Checks whether the given flavor is permitted.
     * @return  boolean     <code>true</code> if so, <code>false</code> if not
     */
    public boolean isDataFlavorSupported( DataFlavor flavor ) 
    {
	    return ( _flavorList.contains( flavor ) );
    }

    /**
     * Retrieves the transferable data.
     * @return  Object  The transferable data (Vector).
     */
    public synchronized Object getTransferData( DataFlavor flavor )
	    throws UnsupportedFlavorException, IOException 
	{
    	if( flavor.equals( TreeTransferable._treeNodeFlavor ) ) 
    	{
	        return _treeNode;
	    } 
	    else 
	    {
	        throw new UnsupportedFlavorException (flavor);
	    }
    }

    public String toString() 
    {
	    return "TreeTransferable";
    }

}

