package com.rational.rpw.environment;

import java.awt.Font;

/**
 * This class knows how to generate a common font, given a style,
 * for use in RPW applications and applets.
 */
public class FontFactory
{
	private static final String COMMON_NAME = "SansSerif";
	private static final int COMMON_SIZE = 12;
	
	/**
	 * Creates a common font that is plain.
	 */
	public static Font getCommonPlainFont()
	{
		return( new Font( COMMON_NAME, Font.PLAIN, COMMON_SIZE ) );	
	}
	
	/**
	 * Creates a common font that is bold.
	 */
	public static Font getCommonBoldFont()
	{
		return( new Font( COMMON_NAME, Font.BOLD, COMMON_SIZE ) );	
	}
	
	/**
	 * Creates a common font that is italic.
	 */
	public static Font getCommonItalicFont()
	{
		return( new Font( COMMON_NAME, Font.ITALIC, COMMON_SIZE ) );	
	}
	
	/**
	 * Creates a common font of the given style, of type
	 * Font.PLAIN, Font.ITALIC, or Font.BOLD.
	 */
	public static Font getCommonFont( int styleType )
	{
		return( new Font( COMMON_NAME, styleType, COMMON_SIZE ) );	
	}
	
	/**
	 * Creates a common font of the given style, of type
	 * Font.PLAIN, Font.ITALIC, or Font.BOLD, and of the
	 * given size.
	 */
	public static Font getCommonFont( int styleType,
	                                    int size )
	{
		return( new Font( COMMON_NAME, styleType, size ) );	
	}
	
}
