/**
 * Copyright 2000 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */

package com.rational.rpw.environment;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.*;

import com.rational.rpw.utilities.CommonFunctions;

/**
 * 
 * This class stores RPW configuratio settings.  Use the getHandle method to a reference to the
 * class.
 * 
 */


public class RPWConfiguration extends ConfigurationFile
{
	private static RPWConfiguration theConfiguration = null;
	private static Map theResourceTable = new HashMap(30);
	
	private RPWConfiguration()
	{
		super(StringConstants.RPW_CONFIGURATION_FILE, theResourceTable);
	}
	
	/**
	 * returns a handle to the configuration
	 */
	static public RPWConfiguration getHandle()
	{
		if (theConfiguration == null)
		{
			theConfiguration = new RPWConfiguration();
		}
		
		return theConfiguration;
	}
	
	/**
	 * Must implement this abstract method
	 * @return Resource value if it exists, "" otherwise
	 */
	public String getResourceString(String pKey)
	{
		Object theObj = theResourceTable.get(pKey);
		if (theObj == null)
		{
			return "";
		}
		
		return (String)theObj;
	}

	
	//--------------------------------------------------------------------------------------------
	//
	//		CLIENT INTERFACE -- These routines are responsible for getting and settings the values
	//		of the RPW Configuration file.
	//
	//		All of the type conversions are handled by these routines rather than by the clients.
	//
	//		For Example: 
	//				Priority is returned as an Integer. 
	//				Generation Flags are returned as Booleans.
	//				etc...
	//
	//---------------------------------------------------------------------------------------------
	
	//----------------------------------------------------------------------------
	// Return the Element modifier
	public String getElementModifier()
	{
		String tempValue = (String) theResourceTable.get(StringConstants.RPWCONFIGURATION_MODIFIER_SYMBOL_KEY);
		return tempValue;
	}
		
	// Return the flag that indicates if we modifing the presentation name
	public boolean changePresentationName()
	{
		String tempValue = (String) theResourceTable.get(StringConstants.RPWCONFIGURATION_REPLACE_NAME_KEY);
		if ( tempValue.equals("1") ) {
			return true;
		}
		else {
			return false;
		}
	}
	
	/** This method will return a hashtable containing all file names which are to be excluded from
	 * hyperlink processing
	*/
	public Hashtable getExcludedFilesFromProcessingLinks() {
		
		Hashtable theReturnHash = new Hashtable( 30 );
		StringTokenizer theParser = new StringTokenizer( ((String)theResourceTable.get(StringConstants.RPWCONFIGURATION_HYPERLINK_EXCLUSION_KEY)),
														 "," );
		while ( theParser.hasMoreTokens() ) {
			//format paths
			String thePath = CommonFunctions.formatPath(theParser.nextToken());
			theReturnHash.put(thePath, "yes");
		}
		return theReturnHash;
	}
	
	/** This method will return a String value indicating type of encoding the content_library is using
	*/
	public String getContentLibraryEncoding () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_CONTENT_LIBRARY_ENCODING_KEY );
		return theValue;
	}
	
	/** This method will return a String value indicating type of encoding the content_library is using
	*/
	public String getPublishWebsiteEncoding () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_PUBLISH_WEBSITE_ENCODING_KEY );
		return theValue;
	}
												   
	/** This method will return a String value of the role subtree filename
	*/
	public String getRoleSubtreeFileName () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_ROLE_SUBTREE_FILENAME_KEY );
		return theValue;
	}	
	
	/** This method will return a String value of the artifact subtree filename
	*/
	public String getArtifactSubtreeFileName () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_ARTIFACT_SUBTREE_FILENAME_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the discipline subtree filename
	*/
	public String getDisciplineSubtreeFileName () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_DISCIPLINE_SUBTREE_FILENAME_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the tool subtree filename
	*/
	public String getToolSubtreeFileName () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_TOOL_SUBTREE_FILENAME_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the tool subtree filename
	*/
	public String getLifecycleSubtreeFileName () {
		//return "rpw_lifecycle_subtree.dat";	// TBD: copy from above operation.
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_LIFECYCLE_SUBTREE_FILENAME_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the full class name of the keywordindex applet
	*/
	public String getKeyWordIndexClassPath () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_KEYWORDINDEX_APPLET_CLASS_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the full class name of the sitemap applet
	*/
	public String getSitemapClassPath () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SITEMAP_APPLET_CLASS_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the full class name of the search db applet
	*/
	public String getSearchDBClassPath () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SEARCH_DATABASE_CLASS_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the full class name of the search db helper applet
	*/
	public String getSearchDBHelperClassPath () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SEARCH_DATABASE_HELPER_CLASS_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the full class name of the search db response file
	*/
	public String getSearchDBResponseFile () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SEARCH_DATABASE_RESPONSE_FILE_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the search applet's applet subdirectory in the installation directory
	*/
	public String getSearchAppletDirectory () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SEARCH_APPLET_SUBDIR_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the full class name of the keywordindex applet
	*/
	public String getKeyWordIndexAppletConfigurationFile () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_KEYWORDINDEX_APPLET_CONFIGURATION_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the file name of the static tree file
	*/
	public String getStaticTreeFileName () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_STATIC_TREE_FILE_KEY );
		return theValue;
	}
	
	/** This method will return a String value of delimiter that is used in the tree files
	*/
	public String getTreeFileDelimiter () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_TREE_FILE_DELIMITER_KEY );
		return theValue;
	}
	
	/** This method will return a String value of sitemap applet's execution file
	*/
	public String getSiteMapExecutionFile () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SITEMAP_EXECUTION_FILE_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the sitemap applet's applet subdirectory in the installation directory
	*/
	public String getSiteMapAppletDirectory () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SITEMAP_APPLET_SUBDIR_KEY );
		return theValue;
	}
	
	/** This method will return a String value of the sitemap applet's static file directory
	*/
	public String getSiteMapStaticFileDir () {
		
		String theValue = (String)theResourceTable.get( StringConstants.RPWCONFIGURATION_SITEMAP_STATIC_FILE_DIRECTORY_KEY );
		return theValue;
	}
	
	/**
	 * @return the configuration value for the parameter specified. "" if it does not exist.
	 * @param pConfiguration the Configuration string to retrieve
	 */
	public String getConfigurationSetting(String pConfiguration) {
		String theValue = (String)theResourceTable.get(pConfiguration);
		if (theValue == null) {
			return "";
		}
		return theValue;
	}
	
	
	public void dumpConfigurationToConsole()
	{
		Iterator theKeys = theResourceTable.keySet().iterator();
		while(theKeys.hasNext())
		{
			String theKey = (String)theKeys.next();
			String theValue = getResourceString(theKey);
			System.out.println("Key=>" + theKey + ", Value=>" + theValue);
		}
	}
	
	//test driver
	static public void main(String args[])
	{
		
		System.out.println("Starting the Test Driver for RPWConfiguration");
		System.out.println("=============================================");
		
		RPWConfiguration theConfiguration = RPWConfiguration.getHandle();
		
		System.out.println("CL Encoding:" + theConfiguration.getContentLibraryEncoding());
		System.out.println("Publish Encoding:" + theConfiguration.getPublishWebsiteEncoding());
		
		System.out.println("\n\nDumping the RPWConfiguration file\n\n");
		theConfiguration.dumpConfigurationToConsole();
		
		System.out.println("Finished the Test Driver for RPWConfiguration");
		System.out.println("=============================================");
		
		System.exit(0);
		
	}
	
}
