package com.rational.rpw.environment;

/**
 * This class will dynamically load the classes to be used to regenerate 
 * the sitemap, keywordindex and search database once a new website has
 * been generated. The classes to be loaded must be located in a directory
 * which is referenced by the classpath. If not the class will not be found
 * and an error will be returned.
 */

import java.lang.reflect.*;

public class RUPClassLoader
{
	private java.lang.Class RUPClass;
	private Object obj;
	
	//----------------------------------------------------------------------------
	// Constructor -- load the specified classes.
	public RUPClassLoader(String className)
		throws RUPLoadingException
	{
		try {
			RUPClass = Class.forName(className);
		} catch ( ClassNotFoundException cnfe ) {
			// Class cannot be loaded, throw new RUPLoadingException
			throw new RUPLoadingException("Class Not Found", cnfe.getMessage());
		}
		
		// Create a new instance of the class which was just loaded.
		try {
			//Constructor
			//theConstructor = RUPClass.getConstructor( new Class[0] );
			//obj = theConstructor.newInstance( new Object[0] );
			obj = RUPClass.newInstance();
		} catch (InstantiationException ie) {
			throw new RUPLoadingException("Object Creation Exception", ie.getMessage());
		} catch (IllegalAccessException iae) {
			throw new RUPLoadingException("Illegal Access Exception", iae.getMessage());
		}/* catch (NoSuchMethodException nme) {
			  throw new RUPLoadingException("No Such Method Exception", nme.getMessage());
		} catch (InvocationTargetException ite) {
			  throw new RUPLoadingException("Invocation Target Exception", ite.getMessage());
		}*/
	}

	//------------------------------------------------------------------------------
	// Invoke the specified method of the loaded class to kick off the regeneration.
	public Object startProcessing(String methodName, Object[] methodArguments)
		throws RUPLoadingException
	{
		Method[] allMethods;
				
		// Check that the class has been loaded properly.
		if ( RUPClass == null ) 
		{
			throw new RUPLoadingException("Class Load Error", "Class was not loaded before method invocation");	
		}
		
		// Use the Reflection API to get a handle to the requested method of the object.
		Class targetClass = obj.getClass();
		
		// Get a list of all the methods that are in the loaded class.
		try {
			allMethods = targetClass.getMethods();
		} catch (SecurityException se) {
			throw new RUPLoadingException("Security Violation", se.getMessage());
		}
		
		// Search for the requested method in the array of methods which were returned.
		for ( int i = 0; i < allMethods.length; i++ )
		{
			if (methodName.equals(allMethods[i].getName()))
			{
				// Invoke this method with the supplied arguments.
				try {
					Object returnValue = allMethods[i].invoke(obj, methodArguments);
					return returnValue;
				} catch (IllegalAccessException iae) {
					throw new RUPLoadingException("Illegal Access Exception", iae.getMessage());
				} catch (IllegalArgumentException iae) {
					throw new RUPLoadingException("Illegal Argument Exception",  iae.getMessage());  
				} catch (InvocationTargetException ite) {
					throw new RUPLoadingException("Invocation Target Exception", ite.getMessage());  
				}
			}
		}
		// If we get this far then the method was not found so something weird is going on.
		throw new RUPLoadingException("Method not found", "Could not find start method in loaded class");
	}
}
