package com.rational.rpw.environment;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.JFrame;

import com.rational.rpw.html.RPWHTMLFileException;
import com.rational.rpw.html.RPWMetaData;
import com.rational.rpw.rpwapplication.RPWAlertDlg;

/**
 * @author pdenno
 *
 * Copyright 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */

public class UnixRegistry implements IRegistryService
{
	private String INSTALL_DIR_ID		= "INSTALL_DIRECTORY";
	private String theInstallDir 		= null;
	private String CONFIG_FILE			= "builder.conf";
    private HashMap _RegistryValues	= new HashMap();
    private RPWMetaData theResourceFile;
      
	/**
	 * Constructor. 
	 */
	public UnixRegistry()
	{
		/**
         * Constructor - create an instance of this class and populate it with 
         * all the information from the configuration file.
         * 
         * The install dir is read from the system properties and then the hashmap
         * is populated with the data from the config file in that folder.
         * 
         * If the install dir cannot be read or the config file is not present 
         * then when the client asks for any system dependent information an 
         * exception will be raised as none of the information will be available. 
         */
        theInstallDir = System.getProperty(INSTALL_DIR_ID);
        if ( theInstallDir != null && 
        	 !theInstallDir.endsWith(File.separator) )
        {
       		theInstallDir += File.separator;	
        }
        return;
    }
    
    /**
     * Method to read the configuration file and populate the resource bundle.
     */	
	private void load()	
	{
		if ( theInstallDir == null )
		{
			return;	
		}
		
		theResourceFile = new RPWMetaData(theInstallDir + CONFIG_FILE);
	 	 
	 	try {
	 		Iterator theValues = theResourceFile.getMetaNames().iterator();
	 	 	while( theValues.hasNext() ) {
	 	 		String theName = (String)theValues.next();
	 	 		String theValue = theResourceFile.getValue(theName);
	 	 		_RegistryValues.put(theName, theValue);
	 	 	}
	 	} catch (RPWHTMLFileException rhe) {
	 	 	System.out.println("Registry Loading Exception: " + rhe.getMessage());
	 	}
	}
	
	/**
     * This method does the lookup in the hash table for the input key.
     * @return the value of the system service identified.
     * @exception EnvironmentException if the identified service cannot be found.
     */
    private String getValue(String pKey)
    	throws EnvironmentException 
    {    	    	
    	if ( theResourceFile == null )
    	{
    		load();	
    	}
    	
    	Object theValue = _RegistryValues.get(pKey);    	
     	if ( theValue == null )
     	{
     		throw new EnvironmentException("Could not get system dependent inormation for key " + pKey);	
     	}
     	return (String) theValue;
    }  
		
	/**
	 * Retrieve the location of the default repository.
	 */
	public String getRepositoryDirectory()
		throws EnvironmentException
	{
		String REPOSITORY_DIRECTORY	= "REPOSITORY_DIR";
		return theInstallDir + getValue(REPOSITORY_DIRECTORY);
	}
	
	/**
	 * Retrieve the location of the process library root.
	 */
	public String getProcessLibrary()
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the application which is registered to handle files of the input type.
	 */
	public String getViewer(String fileEnding)
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the application which is registered to handle files of the input type.
	 */
	public String getEditor(String fileEnding)
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the directory where we store application specific information for builder.
	 */
	public String getBuilderApplicationDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the directory where we store application specific information for Modeller.
	 */
	public String getModellerApplicationDirectory()
		throws EnvironmentException
	{
		return "";	
	}	
	
	/**
	 * Get the directory where we store application specific information for the 
	 * standalone Organizer.
	 */
	public String getStandaloneOrganizerApplicationDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the directory where we store application specific information for the 
	 * extended help application. 
	 */
	public String getExtendedHelpApplicationDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the install location for Builder.
	 */
	public String getBuilderInstallDirectory()
		throws EnvironmentException
	{
		if ( theInstallDir == null )
		{
			throw new EnvironmentException("Install Directory could not be determined");	
		} 
		return theInstallDir;	
	}
	
	/**
	 * Get the install location for Modeller.
	 */
	public String getModellerInstallDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	/**
	 * Get the install location for standalone organizer.
	 */
	public String getStandaloneOrganizerInstallDirectory()
		throws EnvironmentException
	{
		return "";	
	}	
	
	/**
	 * Get the install location for the extended help application.
	 */
	public String getExtendedHelpInstallDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	
	public String getTempDirectory()
		throws EnvironmentException
	{
		String tempDirectory = theInstallDir + "tmp";
      		
      	File tempFile = new File(tempDirectory);
      		
      	/**
      	 * Attempt to create the temp directory if it does not already exist.
      	 */
      	if ( !tempFile.exists() )
      	{
      		boolean result = tempFile.mkdirs();
      		
      		if ( !result )
      		{
      			throw new EnvironmentException("Failed to create the temp directory.");	
      		}	
      	}
      		
      	return tempDirectory;	
	}
	
	public String getRUPInstallDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	public String getRUPEclipseInstallDirectory()
		throws EnvironmentException
	{
		return "";	
	}
	
	public String getRUPDotNetInstallDirectory()
		throws EnvironmentException
	{
		return "";	
	}	
}
