package com.rational.rpw.environment;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

import javax.swing.JFrame;

import com.rational.rpw.rpwapplication.RPWAlertDlg;

/**
 * This class provides the point of entry for all the preference handling
 * in the various applications. Clients call the static method getInstance()
 * to gain access to it. 
 * 
 * The class uses the property mechanism of java to store the data and this 
 * data is flushed to the file upon application exit.
 */

public class UserPreferences
{
	/**
	 * Reference to the singleton instance of this class.
	 */
	private static UserPreferences thePreference					= new UserPreferences();
	
	private HashMap preferences = new HashMap();
	private String preferenceFileName = "preference.properties"; //$NON-NLS-1$
	
	private String publishFolder;
	private String theChosenViews = "";
	private String entrySeparator = ";";
	
	// Determines mode of publishing for diagrams.
	public static final int USE_EXSTING_DIAGRAMS					= 1;
	public static final int GENERATE_DIAGRAMS					= 2;
	public static final int GENERATE_TABLES						= 3;
	
	public static final int ALL_ARTIFACTS						= 4;
	public static final int MANDATORY_ARTIFACTS					= 5;
	public static final int NO_ARTIFACTS							= 6;
	
	public static final String OPEN_LAYOUT						= "OPEN_LAYOUT";
	public static final String NEW_LAYOUT						= "NEW_LAYOUT";
	public boolean SHOW_ERROR_LOG									= false;
		
	// These variables and flags are for the detection of process views
	private static String theConfigurationDirectory = null;
	private static String theConfigurationName = null;
		
	/**
	 * These are all the keys used in the UserPreference resource file.
	 */
	public final static String GENERATE_SEARCH				= "GENERATE_SEARCH";
	public final static String GENERATE_KEYWORD				= "GENERATE_KEYWORD";
	public final static String DISCIPLINE_DIAGRAMS			= "DISCIPLINE_DIAGRAMS";
	public final static String WORKFLOWDETAIL_DIAGRAMS 		= "WORKFLOWDETAIL_DIAGRAMS";
	public final static String ITERATION_WORKFLOW_DIAGRAMS 	= "ITERATION_WORKFLOW_DIAGRAMS";
	public final static String ROLE_DIAGRAMS					= "ROLE_DIAGRAMS";
	public final static String VIEWER_APPLICATION 			= "VIEWER_APPLICATION";
	public final static String HTML_VIEWER					= "HTML_VIEWER";
	public final static String GIF_VIEWER					= "GIF_VIEWER";
	public final static String JPEG_VIEWER					= "JPEG_VIEWER";
	public final static String ERROR_LOG_DISPLAY				= "ERROR_LOG_DISPLAY";
	public final static String PROCESS_RELATIVE_LINKS 		= "PROCESS_RELATIVE_LINKS";
	public final static String PROCESS_EXTERNAL_LINKS 		= "PROCESS_EXTERNAL_LINKS";
	public final static String USER_DEFINED_REPOSITORY		= "USER_DEFINED_REPOSITORY"; 
	public final static String SHOW_TYPES_OF_ARTIFACTS		= "SHOW_ARTIFACTS";
	public final static String KNOWN_LAYOUT_LOCATIONS		= "KNOWN_LAYOUT_LOCATIONS";
	public final static String FILENAME_DISPLAY				= "FILENAME_DISPLAY_TOGGLE";
	public final static String INITIAL_DISPLAY_FLAG			= "INITIAL_DISPLAY_FLAG";
	public final static String ORGANIZER_INITIAL_DEFAULT		= "ORGANIZER_INITIAL_DISPLAY_FLAG";
				
	/**
	 * Static method to return the object which manages the preference for 
	 * the current application context.
	 */
	public static UserPreferences getInstance()
	{
		if (thePreference == null)
		{
			synchronized (UserPreferences.class)
			{
				if (thePreference == null)
				{
					thePreference = new UserPreferences();
				}
		  }
		}
		return thePreference;	
	}
	
	/**
	 * Singleton constructor - load the preferences from the file upon startup.
	 */
	private UserPreferences()
	{
		load();
		return;		
	}
	
	public void reLoadpreferences()
	{
		load();		
	}
	
	/**
	 * Load the preferences file for this application from the install directory.
	 */
	private void load()
	{
		String applicationDirectory = ""; //$NON-NLS-1$
		
		/**
		 * Get the application directory.
		 */
		try {
			RegistryService theService = RegistryService.getInstance();
			applicationDirectory = theService.getApplicationDirectory();
		} catch ( EnvironmentException ee ) {
			// Just ignore.	
		}
			
		File theFile = new File(applicationDirectory + preferenceFileName);
		String theString;
			
		try {
			FileInputStream theReader = new FileInputStream(theFile.getAbsolutePath());
			int available = theReader.available();
			byte[] theByteArray = new byte[available];
			theReader.read(theByteArray);
			theString = new String(theByteArray);
		} catch ( FileNotFoundException fnfe ) {
			// File cannot be found and can therefore not be loaded.
			return;
		} catch ( IOException ioe ) {
			// File could not be read and can therefore not be loaded.
			return;
		}
				
		// Tokenize the string and insert the values into the hashtable.
		StringTokenizer st = new StringTokenizer(theString, System.getProperty("line.separator"), false); //$NON-NLS-1$
			
		// Break the line on the delimiter and insert into the hashtable.		
		int index;
		while ( st.hasMoreTokens() )
		{
			String theLine = st.nextToken();
			index = theLine.indexOf("="); //$NON-NLS-1$
			if ( index == -1 )
			{
				continue;
			}
			else
			{
				String key = theLine.substring(0, index).toUpperCase();
				String value = theLine.substring(index + 1, theLine.length());
				StringBuffer tempValue	= new StringBuffer();
				if ( value.indexOf(StringConstants.tempPathDelimiter) != -1 )
				{
					// Tokenize the string and then reassamble with the double-sep.
					StringTokenizer temp_st = new StringTokenizer(value, StringConstants.tempPathDelimiter, false);
					tempValue.append(temp_st.nextToken());
					while ( temp_st.hasMoreTokens() )
					{
						tempValue.append(File.separator);
						tempValue.append(temp_st.nextToken());
					}
					value = tempValue.toString();
				}
				preferences.put(key, value);
			}
		}
	}
	
	/**
	 * Write the preference data out to disk.
	 */
	public void commit()
	{
		String applicationDirectory = ""; //$NON-NLS-1$
		
		/**
		 * Get the application directory.
		 */
		try {
			RegistryService theService = RegistryService.getInstance();
			applicationDirectory = theService.getApplicationDirectory();
		} catch ( EnvironmentException ee ) {
			// Just ignore.	
		}
		
		File theFile = new File(applicationDirectory + preferenceFileName);
		StringBuffer message	= new StringBuffer();
		String sep				= System.getProperty("file.separator"); //$NON-NLS-1$
						
		// Get all the elements from the Hashtable.
		Set keySet = preferences.keySet();
		Iterator theIter = keySet.iterator();
		while ( theIter.hasNext() )
		{
			String key				= (String) theIter.next();
			String value			= (String) preferences.get(key);
			StringBuffer tempValue	= new StringBuffer();
				
			// Adding paths causes a problem because the "\" path seperators on windows
			// are interpreted as escape sequences. The same problem exists when reading
			// the data also. Therefore in order to solve this we will replace those
			// characters with another seperator.
			if ( value.indexOf(sep) != -1 )
			{
				// Tokenize the string and then reassamble with the double-sep.
				StringTokenizer st = new StringTokenizer(value, sep, false);
				tempValue.append(st.nextToken());
				while ( st.hasMoreTokens() )
				{
					tempValue.append(StringConstants.tempPathDelimiter);
					tempValue.append(st.nextToken());
				}
				value = tempValue.toString();
			}
				
			message.append(key);
			message.append("="); //$NON-NLS-1$
			message.append(value);
			message.append(System.getProperty("line.separator")); //$NON-NLS-1$
		}
		
		// Ensure that all the parents exist before trying to write out the file.
		boolean result = ensurePathExists(theFile);
			
		if ( !result )
		{
			return;	
		}
			
		// Truncate the existing file and write our data out.
		try {
			FileWriter fw;
			fw = new FileWriter(theFile);
			fw.write(message.toString());
			fw.close();
		} catch ( IOException ioe ) {
			// Error, should alert the user that the Preferences could not be saved.
			String theMessage = Translations.getString("ENVIRONMENT_5"); //$NON-NLS-1$
			Object[] args = {theFile.getAbsolutePath(), ioe.getMessage()};
			RPWAlertDlg.showErrorMessage(new JFrame(), 
										Translations.getString("ENVIRONMENT_6"), //$NON-NLS-1$
										MessageFormat.format(theMessage, args));
		}		
	}
	
	private boolean ensurePathExists(File fileName)
	{
		// Break the path up on it's dividers.
		StringTokenizer st = new StringTokenizer(fileName.getAbsolutePath(), File.separator, false);
			
		int total = st.countTokens();
			
		// Iterate over all the path components and make sure they all exist, and if 
		// not then create them.
		String targetName = "";
		
		for ( int i = 0; i < total - 1; i++ )
		{
			targetName += st.nextToken() + File.separator;
			File name = new File(targetName);
								
			if ( !name.exists() )
			{
				boolean result = name.mkdir();	
				if ( !result )
				{
					return false;	
				}
			}
		}
		return true; 	  	
	}	
	
	/**
	 * Sets the value in the preference hashtable to be the input value.
	 */
	private void putInt(String key, int value)
	{
		preferences.put(key, (new Integer(value)).toString());	
	}
	
	/**
	 * Sets the value in the preference hashtable to be the input value.
	 */
	private void putString(String key, String value)
	{
		preferences.put(key, value);	
	}
	
	/**
	 * Sets the value in the preference hashtable to be the input value.
	 */
	private void putBoolean(String key, boolean value)
	{
		preferences.put(key, (new Boolean(value)).toString());	
	}
	
	/**
	 * Gets the value in the preference hashtable with the input key.
	 * If the key does not exist then the default value is returned.
	 */
	private int getInt(String key, int defaultValue)
	{
		if ( preferences.containsKey(key) )
		{
			String value = (String) preferences.get(key);
			try {
				return Integer.parseInt(value);
			} catch ( NumberFormatException npe ) {
				// Number is invalid so return the default.	
			}	
		}
		return defaultValue;
	}
	
	/**
	 * Gets the value in the preference hashtable with the input key.
	 * If the key does not exist then the default value is returned.
	 */
	private String getString(String key, String defaultValue)
	{
		if ( preferences.containsKey(key) )
		{
			return (String) preferences.get(key);
		}
		return defaultValue;
	}
	
	/**
	 * Gets the value in the preference hashtable with the input key.
	 * If the key does not exist then the default value is returned.
	 */
	private boolean getBoolean(String key, boolean defaultValue)
	{
		if ( preferences.containsKey(key) )
		{
			String value = (String) preferences.get(key);
			return (Boolean.valueOf(value)).booleanValue();
		}
		return defaultValue;
	}
	
	/**
	 * Sets the policy that users have set regarding what type of artifacts
	 * are to be shown.
	 */
	public void setArtifactShowingPolicy(int policy)
	{
		this.putInt(UserPreferences.SHOW_TYPES_OF_ARTIFACTS, policy);	
	}
	
	/**
	 * Gets the policy that users have set regarding what type of artifacts 
	 * are to be shown.
	 */
	public int getArtifactShowingPolicy()
	{
		return this.getInt(UserPreferences.SHOW_TYPES_OF_ARTIFACTS, UserPreferences.MANDATORY_ARTIFACTS);	
	}
	
	/**
	 * @return this method figures out what type of file is being viewed and 
	 * returns the registered application depending on the file extension.
	 */
	public String getViewingApplication(String fileExtension)
	{
		if ( fileExtension.toUpperCase().equals(".HTM") || fileExtension.toUpperCase().equals(".HTML") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			return getString(".HTML_VIEWER", StringConstants.useSystemDefault); //$NON-NLS-1$	
		}
		else if ( fileExtension.toUpperCase().equals(".GIF") ) //$NON-NLS-1$
		{
			return getString(".GIF_VIEWER", StringConstants.useSystemDefault); //$NON-NLS-1$	
		}
		else if ( fileExtension.toUpperCase().equals(".JPEG") || fileExtension.toUpperCase().equals(".JPG") ) //$NON-NLS-1$ //$NON-NLS-2 //$NON-NLS-2$
		{
			return getString(".JPEG_VIEWER", StringConstants.useSystemDefault); //$NON-NLS-1$	
		}
		else if ( fileExtension.toUpperCase().equals(".DOC") ) //$NON-NLS-1$
		{
			return getString(".DOC_VIEWER", StringConstants.useSystemDefault);	//$NON-NLS-1$
		}
		else
		{
			// return the default application.
			return getString("DEFAULT_VIEWER", StringConstants.useSystemDefault);	//$NON-NLS-1$
		}
	}
	
	/**
	 * Set the application that is to be used in viewing files with the input 
	 * extension.
	 */
	public void setViewingApplication(String fileExtension, String application)
	{
		if ( fileExtension.toUpperCase().equals(".HTM") || fileExtension.toUpperCase().equals(".HTML") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			putString(".HTML_VIEWER", application);	 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".GIF") ) //$NON-NLS-1$
		{
			putString(".GIF_VIEWER", application);	 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".JPEG") || fileExtension.toUpperCase().equals(".JPG") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			putString(".JPEG_VIEWER", application);	 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".DOC") ) //$NON-NLS-1$
		{
			putString(".DOC_VIEWER", application);	 //$NON-NLS-1$
		}
		else
		{
			// return the default application.
			putString("DEFAULT_VIEWER", application);	 //$NON-NLS-1$
		}			
	}
	
	/**
	 * Gets the application which is registered for editing files of the input type.
	 */
	public String getEditingApplication(String fileExtension)
	{
		if ( fileExtension.toUpperCase().equals(".HTM") || fileExtension.toUpperCase().equals(".HTML") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			return getString(".HTML_EDITOR", StringConstants.useSystemDefault);	 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".GIF") ) //$NON-NLS-1$
		{
			return getString(".GIF_EDITOR", StringConstants.useSystemDefault);		 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".JPEG") || fileExtension.toUpperCase().equals(".JPG") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			return getString(".JPEG_EDITOR", StringConstants.useSystemDefault);		 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".DOC") ) //$NON-NLS-1$
		{
			return getString(".DOC_EDITOR", StringConstants.useSystemDefault);		 //$NON-NLS-1$
		}
		else
		{
			return getString("DEFAULT_EDITOR", StringConstants.useSystemDefault); 		 //$NON-NLS-1$
		}	
	}
	
	/**
	 * Sets the application which is to be used when editing files of the input
	 * type to be the input application.
	 */
	public void setEditingApplication(String fileExtension, String application)
	{
		if ( fileExtension.toUpperCase().equals(".HTM") || fileExtension.toUpperCase().equals(".HTML") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			putString(".HTML_EDITOR", application);	 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".GIF") ) //$NON-NLS-1$
		{
			putString(".GIF_EDITOR", application);		 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".JPEG") || fileExtension.toUpperCase().equals(".JPG") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			putString(".JPEG_EDITOR", application);		 //$NON-NLS-1$
		}
		else if ( fileExtension.toUpperCase().equals(".DOC") ) //$NON-NLS-1$
		{
			putString(".DOC_EDITOR", application);		 //$NON-NLS-1$
		}
		else
		{
			putString("DEFAULT_EDITOR", application); 		 //$NON-NLS-1$
		}	
	}
	
	/**
	 * Get the graphic policy assocaited with the input type.
	 */
	public int getGraphicPolicy(String graphicType)
	{
		return getInt(graphicType, UserPreferences.GENERATE_TABLES);	
	}
	
	/**
	 * Set the graphic policy associated with the input type to the input value.
	 */
	public void setGraphicPolicy(String graphicType, int policy)
	{
		putInt(graphicType, policy);
	}
	
	/**
	 * Set the policy for processing relative links during publishing.
	 */
	public void setRelativeLink(boolean link) 
	{
		putBoolean(UserPreferences.PROCESS_RELATIVE_LINKS, link);
	}

	/**
	 * @return the policy for processing relative links during publishing.
	 */
	public boolean getRelativeLink() 
	{
		return getBoolean(UserPreferences.PROCESS_RELATIVE_LINKS, false);		
	}

	/**
	 * Set the policy for processing external links during publishing.
	 */
	public void setExternalLink(boolean link) 
	{
		putBoolean(UserPreferences.PROCESS_EXTERNAL_LINKS, link);
	}

	/**
	 * @return get the policy for processing external links during publishing.
	 */
	public boolean getExternalLink() 
	{
		return getBoolean(UserPreferences.PROCESS_EXTERNAL_LINKS, false);		
	}
	
	/**
	 * @return the flag which indicates if the error log should be displayed
	 * during publishing.
	 */
	public boolean getErrorLogDisplayFlag()
	{
		return getBoolean(UserPreferences.ERROR_LOG_DISPLAY, false);	
	}

	/**
	 * Set the flag which indicates if the user wants the error log displayed
	 * during publishing.
	 */
	public void setErrorLogDisplayFlag(boolean newValue)
	{
		putBoolean(UserPreferences.ERROR_LOG_DISPLAY, newValue);	
	}

	/**
	 * NOTE: The publish folder is persisted on a per configuration basis 
	 * and hence are stored with the configuration. This information is
	 * inserted into the UserPreferences at loadtime.
	 */
	public void setPublishFolder(String folderName)
	{
		publishFolder = folderName;
	}

	/**
	 * NOTE: The publish folder is persisted on a per configuration basis 
	 * and hence are stored with the configuration. This information is
	 * inserted into the UserPreferences at loadtime.
	 */
	public String getPublishFolder()
	{
		return publishFolder;		
	}
	
	/**
	 * NOTE: The set of choosen process views is also stored on a per 
	 * configuration basis and are only inserted into this class at 
	 * load time.
	 */
	public void setChosenProcessViews( String chosenViews )
	{
		theChosenViews = chosenViews;
	}
	
	/**
	 * Gets the flag which indicates if the user wants to see the initial 
	 * prompting dialog.
	 */
	public boolean getInitialDisplayFlag()
	{
		return getBoolean(UserPreferences.INITIAL_DISPLAY_FLAG, false);	
	}
	
	/**
	 * Sets the flag which indicates if the user wants to see the initial 
	 * prompting dialog.
	 */
	public void setInitialDisplayFlag(boolean value)
	{
		putBoolean(UserPreferences.INITIAL_DISPLAY_FLAG, value);	
	}

	/**
	 * NOTE: The set of choosen process views is also stored on a per 
	 * configuration basis and are only inserted into this class at 
	 * load time.
	 */
	public String getChosenProcessViews()
	{
		return theChosenViews;	
	}
	
	/**
	 * Set the policy for generating the search DB after publishing.
	 */
	public void setGenerateSearchDB(boolean generate) 
	{
		putBoolean(UserPreferences.GENERATE_SEARCH, generate);
	}

	/**
	 * @return the policy for regenerating the search database, post publishing.
	 */
	public boolean getGenerateSearchDB() 
	{
		return getBoolean(UserPreferences.GENERATE_SEARCH, true);		
	}

	/**
	 * Sets the policy for generating the keyword index after publishing. 
	 */
	public void setGenerateKeywordIndex(boolean generate) 
	{
		putBoolean(UserPreferences.GENERATE_KEYWORD, generate); 
	}

	/**
	 * @return the policy for regenerating the keyword index, post publishing.
	 */
	public boolean getGenerateKeywordIndex() 
	{
		return getBoolean(UserPreferences.GENERATE_KEYWORD, true);		
	}
	
	/**
	 * Sets the policy for how the names/presentation names toggle is to
	 * be displayed. 
	 */
	public void setFileBrowserNamePolicy(boolean value)
	{
		putBoolean(UserPreferences.FILENAME_DISPLAY, value);	
	}
	
	/**
	 * @return the policy for how the names/presentation names toggle is
	 * to be displayed.
	 */
	public boolean getFileBrowserNamePolicy()
	{
		return getBoolean(UserPreferences.FILENAME_DISPLAY, false);	
	}
	
	/**
	 * Set the default selection in the startup screen when Standalone Organizer
	 * is started.
	 */
	public void setInitialStartupDefaultForStandAloneOrganizer(String value)
	{
		putString(UserPreferences.ORGANIZER_INITIAL_DEFAULT, value);	
	}
	
	/**
	 * @return the initial default for the standalone organizer when first 
	 * started.
	 */
	public String getInitialStartupDefaultForStandAloneOrganizer()
	{
		return getString(UserPreferences.ORGANIZER_INITIAL_DEFAULT, UserPreferences.NEW_LAYOUT);	
	}
	
	/**
	 * Adds the repository pointed to by the input location to the list 
	 * of repositories which we are maintaining.  
	 */
	public void addRepository(String newLocation)
	{
		if ( newLocation.endsWith(File.separator) )
		{
			newLocation = newLocation.substring(0, newLocation.length() - 1);	
		}
		
		String userRepository = getString(UserPreferences.USER_DEFINED_REPOSITORY, "");
		
		if ( userRepository.indexOf(newLocation) != -1 )
		{
			return;	
		}
		else
		{
			userRepository += entrySeparator + newLocation;	
			putString(UserPreferences.USER_DEFINED_REPOSITORY, userRepository);
		}
	}
	
	/**
	 * Remove the path to the designated repository. This method is called when 
	 * the path no longer represents a valid repository.
	 */
	public void removeRepository(String path)
	{
		String userRepository = getString(UserPreferences.USER_DEFINED_REPOSITORY, "");
		
		int index = userRepository.indexOf(path); 
		if ( index != -1 )
		{
			String firstPart = userRepository.substring(0, index);
			String middlePart = userRepository.substring(index, index + path.length());
			String endPart = userRepository.substring(index + path.length());
			userRepository = firstPart + endPart;	
			putString(UserPreferences.USER_DEFINED_REPOSITORY, userRepository);
		}	
	}
	
	/**
	 * @return an enumeration over all the repositories which this object
	 * knows about.
	 */
	public Enumeration getAllRepositories()
	{
		String userRepository = getString(UserPreferences.USER_DEFINED_REPOSITORY, "");
		return new StringTokenizer(userRepository, entrySeparator, false);
	}
	
	/**
	 * Add the location of the new layout to the list of locations which 
	 * this program knows about.
	 */
	public void addLayoutLocation(String newLocation)
	{
		if ( newLocation.endsWith(File.separator) )
		{
			newLocation = newLocation.substring(0, newLocation.length() - 1);	
		}
		
		newLocation += entrySeparator; 
		String layoutString = getString(UserPreferences.KNOWN_LAYOUT_LOCATIONS, "");		
				
		/**
		 * Make the comparison case-insensitive so that we don't add duplicates
		 * when a file has been renamed. 
		 */
		String capitalizedLayoutString = layoutString.toUpperCase();
		String capitalizedNewLocation = newLocation.toUpperCase();
		
		if ( capitalizedLayoutString.indexOf(capitalizedNewLocation) != -1 )
		{
			return;	
		}
		else
		{
			if ( layoutString != "" )
			{
				layoutString += newLocation;	
			}
			else
			{
				layoutString = newLocation;	
			}
			putString(UserPreferences.KNOWN_LAYOUT_LOCATIONS, layoutString);
		}
	}
	
	/**
	 * @return an enumeration over all the layouts which this program knows about.
	 */
	public Enumeration getUserLayouts()
	{
		String layoutString = getString(UserPreferences.KNOWN_LAYOUT_LOCATIONS, "");	
		return new StringTokenizer(layoutString, entrySeparator, false);
	}
	
	/**
	 * This method looks for the input layout in the string and moves it to 
	 * the front of the string to indicate that it was the layout which was 
	 * most recently used.
	 */
	public void setMostRecentlyUsedLayout(String layoutPath)
	{
		String layoutString = getString(UserPreferences.KNOWN_LAYOUT_LOCATIONS, "");
		int index = layoutString.indexOf(layoutPath + entrySeparator);
		if ( index != -1 )
		{
			String firstPart = layoutString.substring(0, index);
			String secondPart = layoutString.substring(index + layoutPath.length() + 1);
			String newLayoutString = layoutPath + entrySeparator + firstPart + secondPart;	
			putString(UserPreferences.KNOWN_LAYOUT_LOCATIONS, newLayoutString);
		}			
	}
	
	/**
	 * This is a maintenance method which we will use to remove layout 
	 * references in the file which are no longer valid on disk.
	 */
	public void removeLayoutReference(String layoutPath)
	{
		String layoutString = getString(UserPreferences.KNOWN_LAYOUT_LOCATIONS, "");
		int index = layoutString.indexOf(layoutPath + entrySeparator);
		if ( index != -1 )
		{
			String firstPart = layoutString.substring(0, index);
			String secondPart = layoutString.substring(index + layoutPath.length() + 1);
			String newLayoutString = firstPart + secondPart;	
			putString(UserPreferences.KNOWN_LAYOUT_LOCATIONS, newLayoutString);
		}		
	}
}
