package com.rational.rpw.processtools;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.search.Hits;

//import com.rational.rpw.environment.PlatformSpecificDispenser;
//import com.rational.rpw.environment.UserPreferences;
import com.rational.rpw.processtools.contextrupmapper.ContextRupMapper;
import com.rational.rpw.processview.Bookmark;
import com.rational.rpw.processview.Context;
import com.rational.rpw.processview.TopicFile;
import com.rational.rpw.processview.TopicFileParser;
import com.rational.rpw.processviewer.ViewerUtilities;
import com.rational.rpw.search.SearchRup;
import com.rational.rpw.search.indexutil.IndexLoader;


/**
 * This class gathers the extended help content. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ContextRupPresenter
{
	public static final String SELECT_OTHER_SITES = Translations.getString("PROCESSTOOLS_54"); //$NON-NLS-1$
	
	private String _documentReadRoot = null;
	private String _appletDirectory = null;
	private URL _documentBase = null;
	private URL _applicationDirectoryUrl = null;
	
	private Bookmark _viewableBookmark = null;
	private String _indexPath = null;
	private String _productName = null;
	
	/**
	 * Default constructor.
	 */
	public ContextRupPresenter( URL applicationDirectoryUrl,
	                             String documentReadRoot,
	                             URL documentBase,
	                             String appletDirectory )
	{
        super();
        
    	// Create the environment for extended help.
    	//PlatformSpecificDispenser.createExtendedHelpEnvironment();
    	// NOTE: At a later date we should actually parse out the install directory from the URL.
    	//UserPreferences.createExtendedHelpPreferences(documentBase.toString());
 
        // initialize directory locations
        _applicationDirectoryUrl = applicationDirectoryUrl;
        _documentReadRoot = documentReadRoot;
        _documentBase = documentBase;
        
		String userRupFolder = System.getProperty( "user.home" ) + //$NON-NLS-1$
			                   File.separator +
			                   "rup";  //$NON-NLS-1$
			                   
		// download and extract index jar if neccessary
		String indexFolder =   userRupFolder + File.separator + "index"; //$NON-NLS-1$
		_indexPath = indexFolder;
		String indexFile = indexFolder  + File.separator + "index.jar";					                         //$NON-NLS-1$
		String dBase = documentBase.toString();		
		if (versionOutOfDate(indexFolder, dBase))
		{
			IndexLoader loader = new IndexLoader( dBase,
   												  indexFolder,
   												  indexFile);
			_productName = loader.go();
			if( null != _productName )
			{
				// successful loading, just close
				loader.setVisible( false );
				loader = null;
			}
		}
        
   	}
	
	/**
	 * Retrieves the bookmark with the context built in.
	 */
	public Bookmark getContextBookmark()
		throws java.security.AccessControlException,
		        IOException,
		        ClassNotFoundException
	{
		if( null == _viewableBookmark )
		{
			// create a bookmark only if it has not been created
			createBookmark();
		}
		return( _viewableBookmark );
	}
		
	/**
	 * Creates the bookmark.
	 */
	private void createBookmark()
		throws java.security.AccessControlException,
		        IOException,
		        ClassNotFoundException
	{
		// set up context
		TopicFile contextTopicFile = TopicFileParser.parseTopicFile( _applicationDirectoryUrl,
		                                                             "xhelp.context" ); //$NON-NLS-1$

		// string together the context into words
		Context context = new Context();
		context.setTool( contextTopicFile.getTool() );
		Iterator topics = contextTopicFile.getTopics();
		while( topics.hasNext() )
		{
			context.addTopic( (String)topics.next() );			
		}
		ContextRupMapper mapper = new ContextRupMapper( _documentReadRoot + "ext_help" + ViewerUtilities.UNC_SEPARATOR, //$NON-NLS-1$
		                                                context );		
		String mappedString = mapper.getMappedString();
		//System.out.println( "MAPPED STRING = " + mappedString );
		
		Hits hitReturns = null;
		String currentSearchString = formatSimpleSearchString( mappedString, 
		                                                       " OR ",  //$NON-NLS-1$
		                                                       true );
		//System.out.println( "SEARCH STRING = " + currentSearchString );
		// search using the string in the contents
		hitReturns = SearchRup.searchIndex( _indexPath,
		                                    currentSearchString,
		                                    "contents" ); //$NON-NLS-1$
		parseHits( hitReturns, context.getTool() );
		
	}
	
	/**
	 * Parse the hit results into viewable bookmark.
	 */
	private void parseHits( Hits hitReturns,
	                          String toolName )
		throws IOException, ClassNotFoundException
	{
		_viewableBookmark = new Bookmark( Translations.getString("PROCESSTOOLS_55") + toolName ); //$NON-NLS-1$
		_viewableBookmark.setClosedIconName( "bookc.gif" ); //$NON-NLS-1$
		_viewableBookmark.setOpenIconName( "booko.gif" ); //$NON-NLS-1$
		
		int topTenCounter = 0;
		Bookmark moreFolder = new Bookmark( Translations.getString("PROCESSTOOLS_56") ); //$NON-NLS-1$
		for( int i = 0; i < hitReturns.length(); i++ )
		{
			Document doc = hitReturns.doc( i );
			
			Field url = doc.getField( "url" ); //$NON-NLS-1$
			String finalUrl = url.stringValue();
			finalUrl = _documentReadRoot.substring( 0, _documentReadRoot.length() - 1 ) + 
						finalUrl.substring( finalUrl.indexOf( _productName ) + _productName.length() );
			finalUrl = ViewerUtilities.convertFilenameToUNC( finalUrl );
			
			Field title = doc.getField( "title" ); //$NON-NLS-1$
			
			Bookmark topic = new Bookmark( title.stringValue() );
			topic.setFileName( finalUrl );
			topic.setClosedIconName( "bookc.gif" ); //$NON-NLS-1$
			topic.setOpenIconName( "booko.gif" ); //$NON-NLS-1$
			topic.setFromContentLibrary( false );
		
			if( topTenCounter < 15 )
			{	
				_viewableBookmark.add( topic );
			}
			else
			{
				moreFolder.add( topic );
			}
			
			topTenCounter++;
		}
		
		// add more folder if neccessary
		if( moreFolder.getChildCount() > 0 )
		{
			_viewableBookmark.add( moreFolder );
		}	
		
	    // now add node to end of bookmark, for list of other sites
	    Bookmark siteBookmark = new Bookmark( SELECT_OTHER_SITES );
	    siteBookmark.setFileName( SELECT_OTHER_SITES );
		siteBookmark.setClosedIconName( "selectother.gif" ); //$NON-NLS-1$
		siteBookmark.setOpenIconName( "selectother.gif" ); //$NON-NLS-1$
	    
	    Vector otherSites = ContextSiteRegistryLibrary.readContextSiteRegistry().getSiteList();
	    // add only if more than one (if it's just one, no point)
	    if( otherSites.size() > 1 )
	    {
	    	_viewableBookmark.add( siteBookmark );
	    }
		
	}

	protected boolean versionOutOfDate(String indexFolder, String httpBase)
	{
		boolean result = true;
		
		String base = httpBase;
		base = base.substring(0, base.lastIndexOf("/")); //$NON-NLS-1$
		base = base + "/search/index/version.txt"; //$NON-NLS-1$

		try
		{
			URL url = new URL(base);
			// handle UNC
			url = ViewerUtilities.handleURLForUNC( url );
			
			BufferedInputStream inputS = new BufferedInputStream( url.openStream());
			InputStreamReader iStreamReader = new InputStreamReader(inputS);
			BufferedReader read = new BufferedReader(iStreamReader);
			String version = read.readLine();
	
			read.close();
			
			String userStringVersion = ""; //$NON-NLS-1$
			File userVersion = new File(indexFolder + File.separator + "version.txt"); //$NON-NLS-1$
			if (userVersion.exists())
			{
				FileReader fRead = new FileReader(userVersion);
				read = new BufferedReader(fRead);
				userStringVersion = read.readLine();
				read.close();
			}
			if (version.equals(userStringVersion))
			{
				result = false;
				_productName = userStringVersion.substring(0, userStringVersion.indexOf("*")); //$NON-NLS-1$
			}
			else
			{
				userVersion.delete();
				userVersion.getParentFile().delete();
				userVersion.getParentFile().mkdirs();
				FileWriter fWrite = new FileWriter(userVersion);
				BufferedWriter write = new BufferedWriter(fWrite);
				write.write(version + "\n"); //$NON-NLS-1$
				write.close();
			}
		}
		catch (Exception e1) 
		{
			e1.printStackTrace();
			result = true;
		}

		return result;
	}
	
	protected String formatSimpleSearchString(String input, String operator, boolean encloseWithParentheses)
	{
		String result = ""; //$NON-NLS-1$
		StringBuffer together = new StringBuffer();
		int firstQuote = input.indexOf("\""); //$NON-NLS-1$
		if (firstQuote >-1)
		{
			int secondQuote = input.indexOf("\"", firstQuote + 1); //$NON-NLS-1$
			while(secondQuote > -1)
			{
				together.append(operator + input.substring(firstQuote, secondQuote + 1));
				input = input.substring(0, firstQuote) + input.substring(secondQuote +1);
				firstQuote = input.indexOf("\""); //$NON-NLS-1$
				secondQuote = input.indexOf("\"", firstQuote + 1); //$NON-NLS-1$
			}
		}
		
		// no more "" so we can just tokenize on whitespace, and make more "AND" additions.
		
		StringTokenizer tokens = new StringTokenizer(input, " "); //$NON-NLS-1$
		while (tokens.hasMoreElements())
		{
			String toAdd = tokens.nextToken();
			together.append(operator + toAdd);
		}
		
		result = together.toString().substring(operator.length());

		if (encloseWithParentheses)
		{
			result = "(" + result + ") "; //$NON-NLS-1$ //$NON-NLS-2$
		}

		return result;
	}
	
	
}
