package com.rational.rpw.processtools;

import java.awt.Component;
import java.awt.Dimension;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import com.rational.rpw.processviewer.ViewerUtilities;

/**
 * This class presents the extended help site chooser. It is the
 * first dialog that comes up when extended help is chosen <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ContextSiteChooser
{
	// the swing list containing the list of sites
	private JList _siteTableList = null;
	
	// the actual list containing the list of sites
	private Vector _siteList = null;
	
	// the checkbox for making the chosen site default
	private JCheckBox _defaultCheckBox = null;
	
	// the checkbox to not show this again
	private JCheckBox _doNotShowAgainCheckBox = null;
	
	// whether this dialog need to be shown; it needs
	// not be shown if there is only one site available
	private boolean _isChooserApplicable = true;
	
	// the panel to display
	private JPanel _contentsToShow = null;

	
	/**
	 * Constructor. Does nothing. Caller must call initialize.
	 */
	public ContextSiteChooser()
	{
		super();
	}
	
	/**
	 * Should the chooser be shown at all.
	 */
	public boolean isChooserApplicable()
	{
		try
		{
			// determine whether this dialog should be shown at all
			if( !ContextSiteRegistryLibrary.isShowChooserAgain() )
			{
				_isChooserApplicable = false;
			}
		
			// determine whether this dialog should be shown at all
			/*if( !ContextSiteRegistryLibrary.isShowChooserInSession() )
			{
				_isChooserApplicable = false;
			}*/
		}
		catch( Exception e )
		{
			e.printStackTrace();
			return( false );	
		}
		
		return( _isChooserApplicable );	
	}		
	
	/**
	 * Shows the chooser.
	 */
	/**
	 * Shows the dialog box. Returns one of JOptionPane.OK_OPTION
	 * or JOptionPane.CANCEL_OPTION.
	 */
	public int show( Component parent )
		throws FileNotFoundException, MalformedURLException, IOException, ClassNotFoundException
	{
		int value = JOptionPane.showConfirmDialog( parent,
												   _contentsToShow,
		                                           Translations.getString("PROCESSTOOLS_57"), //$NON-NLS-1$
		                                           JOptionPane.OK_CANCEL_OPTION );
		if( value == JOptionPane.OK_OPTION )
		{
			try
			{
				// see if make default is selected
				if( _defaultCheckBox.isSelected() )
				{
					// set to default
					ContextSiteRegistryLibrary.setDefaultSite( (String)_siteTableList.getSelectedValue() );
				}

				// see if do not show again is selected
				if( _doNotShowAgainCheckBox.isSelected() )
				{
					ContextSiteRegistryLibrary.doNotShowChooserAgain();
				}		
				
				// after the site has been chosen, we now register that the site
				// will be launched from extended help
				ContextSiteRegistryLibrary.setFromExtendedHelpLaunch();		
			}
			catch( FileNotFoundException fe )
			{
				fe.printStackTrace();
				throw( fe );	
			}
			catch( MalformedURLException me )
			{
				me.printStackTrace();
				throw( me );	
			}
			catch( IOException ie )
			{
				ie.printStackTrace();
				throw( ie );	
			}
			catch( ClassNotFoundException ce )
			{
				ce.printStackTrace();
				throw( ce );	
			}			
		}
		return( value );
	}		 
	
	/**
	 * Gets the chosen site.
	 */
	public String getChosenSite()
	{
		int selectedIndex = _siteTableList.getSelectedIndex();
		return( (String)_siteList.elementAt( selectedIndex ) );	
	}
	
	/**
	 * Gathers site list and create display.
	 */
	public void initialize( boolean isDoNotShowEnabled )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
		try
		{
			// gather the site list
			ContextSiteRegistry siteRegistry = ContextSiteRegistryLibrary.readContextSiteRegistry();
			_siteList = siteRegistry.getSiteList();
			if( _siteList.size() <= 1 )
			{
				// only one site available, this chooser needs not be shown
				_isChooserApplicable = false;
				return;
			}
			String defaultSite = siteRegistry.getDefaultSite();
			int defaultIndex = 0;
			if( null != defaultSite )
			{
				for( int i = 0; i < _siteList.size(); i++ )
				{
					String site = (String)_siteList.elementAt( i );
					if( site.equals( defaultSite ) )
					{
						defaultIndex = i;
						break;	
					}
				}
			}
			
			// remove %20, etc. marks from the string
			Vector cleanSiteList = new Vector();
			for( int i = 0; i < _siteList.size(); i++ )
			{
				String site1 = (String)_siteList.elementAt( i );
				String site2 = ViewerUtilities.convertFileSpacing( site1 );
				String site3 = ViewerUtilities.convertFileSeparator( site2 );
				cleanSiteList.addElement( site3 );
			}
			
			// construct table list with the site list
			_siteTableList = new JList( cleanSiteList );
			_siteTableList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
			_siteTableList.setSelectedIndex( defaultIndex );
			JScrollPane listScroller = new JScrollPane( _siteTableList );
			listScroller.setPreferredSize( new Dimension( 250, 100 ) );
			listScroller.setMinimumSize( new Dimension( 250, 100 ) );
			listScroller.setAlignmentX( Component.LEFT_ALIGNMENT );
			
			// construct checkboxes
			_defaultCheckBox = new JCheckBox( Translations.getString("PROCESSTOOLS_58") ); //$NON-NLS-1$
			_defaultCheckBox.setSelected( true );	
			
			_doNotShowAgainCheckBox = new JCheckBox( Translations.getString("PROCESSTOOLS_59") ); //$NON-NLS-1$
			_doNotShowAgainCheckBox.setSelected( false );
			
			// put all together
			JLabel chooseLabel = new JLabel( Translations.getString("PROCESSTOOLS_60") ); //$NON-NLS-1$
			JPanel choosePanel = new JPanel();
			choosePanel.setLayout( new BoxLayout( choosePanel, BoxLayout.Y_AXIS ) );			
			choosePanel.add( chooseLabel );
			choosePanel.add( listScroller );
			chooseLabel.setAlignmentX( Component.LEFT_ALIGNMENT );
			listScroller.setAlignmentX( Component.LEFT_ALIGNMENT );
			
			JPanel checkboxPanel = new JPanel();
			checkboxPanel.setLayout( new BoxLayout( checkboxPanel, BoxLayout.Y_AXIS ) );			
			checkboxPanel.add( _defaultCheckBox );
			if( isDoNotShowEnabled )
			{
				checkboxPanel.add( _doNotShowAgainCheckBox );
			}
			checkboxPanel.setBorder( BorderFactory.createEtchedBorder() );
			_defaultCheckBox.setAlignmentX( Component.LEFT_ALIGNMENT );
			_doNotShowAgainCheckBox.setAlignmentX( Component.LEFT_ALIGNMENT );
			
			_contentsToShow = new JPanel();
			_contentsToShow.setLayout( new BoxLayout( _contentsToShow, BoxLayout.Y_AXIS ) );			
			_contentsToShow.add( choosePanel );			
			_contentsToShow.add( checkboxPanel );
			choosePanel.setAlignmentX( Component.LEFT_ALIGNMENT );
			checkboxPanel.setAlignmentX( Component.LEFT_ALIGNMENT );
		}
		catch( MalformedURLException me )
		{
			me.printStackTrace();
			throw( me );	
		}
		catch( IOException ie )
		{
			ie.printStackTrace();
			throw( ie );	
		}
		catch( ClassNotFoundException ce )
		{
			ce.printStackTrace();
			throw( ce );	
		}			
	}
	
}
