package com.rational.rpw.processtools;

import java.io.ObjectInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.URLEncoder;

import javax.swing.JOptionPane;

import com.rational.rpw.processviewer.DocumentBase;

/**
 * This class registers published web sites for Extended Help. It
 * also provides means to retrieve the web sites list and default
 * chosen site. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ContextSiteRegistryLibrary
{
	public static final String REGISTER_FILE_NAME = "xhelp.sites"; //$NON-NLS-1$
	public static final String DO_NOT_SHOW_FILE_NAME = "xhelp_chooser_show_ever.state"; //$NON-NLS-1$
	public static final String FROM_EXTENDED_HELP_FILE_NAME = "xhelp_from_xhelp.state"; //$NON-NLS-1$
	public static final String CONTEXT_LAUNCH_FILE = "index_contextrup.htm"; //$NON-NLS-1$
		
	/**
	 * Private constructor.
	 */
	private ContextSiteRegistryLibrary()
	{
	}
	
	/**
	 * Add a site root to the registry.
	 */
	public static void addSiteRoot( String siteRoot )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
		// first read current one in
		ContextSiteRegistry registry = readContextSiteRegistry();
		registry.addSite( siteRoot +
		                  CONTEXT_LAUNCH_FILE );
		// write it back
		writeContextSiteRegistry( registry );
	}
	
	/**
	 * Removes a site from the registry.
	 */
	public static void removeSite( String site )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
		// first read current one in
		ContextSiteRegistry registry = readContextSiteRegistry();
		registry.removeSite( site );
		// write it back
		writeContextSiteRegistry( registry );
	}
	
	/**
	 * Sets the default site root.
	 */
	public static void setDefaultSiteRoot( String siteRoot )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
		// first read current one in
		ContextSiteRegistry registry = readContextSiteRegistry();
		registry.setDefaultSite( siteRoot +
		                         CONTEXT_LAUNCH_FILE  );
		// write it back
		writeContextSiteRegistry( registry );
	}
	
	/**
	 * Sets the default site.
	 */
	public static void setDefaultSite( String site )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
		// first read current one in
		ContextSiteRegistry registry = readContextSiteRegistry();
		registry.setDefaultSite( site );
		// write it back
		writeContextSiteRegistry( registry );
	}
	
	/**
	 * Add a site root to the registry and set it as the default.
	 */
	public static void addSetDefaultSiteRoot( String siteRoot )
			throws MalformedURLException, IOException, ClassNotFoundException
	{
		// first read current one in
		ContextSiteRegistry registry = readContextSiteRegistry();
		registry.addSite( siteRoot +
		                  CONTEXT_LAUNCH_FILE  );
		registry.setDefaultSite( siteRoot +
		                         CONTEXT_LAUNCH_FILE  );
		// write it back
		writeContextSiteRegistry( registry );
	}
	
	
	
    /**
     * Serializes the given contextSiteRegistry object into a file.
     */
    public static void writeContextSiteRegistry( ContextSiteRegistry contextSiteRegistry )
 			throws FileNotFoundException, IOException
    {
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	// create it
        	file.mkdir();
        }
        
        String filename = homeDir + REGISTER_FILE_NAME;
        file = new File( filename );
        if( file.exists() )
        {
        	file.delete();		                             
        }

        // opens a file output stream and serializes the file
        FileOutputStream fos = null;
    	ObjectOutputStream oos = null;
        fos = new FileOutputStream(filename);
   	    oos = new ObjectOutputStream(fos);

   	    oos.writeObject( contextSiteRegistry );

        fos.close();
        oos.close();
    }

	/**
	 * Reads from a persisted ContextSiteRegistry.
	 */
	public static ContextSiteRegistry readContextSiteRegistry()
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        ContextSiteRegistry contextSiteRegistry = null;
        
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	// create it
        	file.mkdir();
        }
        
        String filename = homeDir + REGISTER_FILE_NAME;
		
        file = new File( filename );
        if( !file.exists() )
        {
        	// create new context registry
        	contextSiteRegistry = new ContextSiteRegistry();
        	String justOneSite = DocumentBase.getDocumentReadRoot() +
			                     CONTEXT_LAUNCH_FILE;
			contextSiteRegistry.addSite( justOneSite );
			// serialize this one
			writeContextSiteRegistry( contextSiteRegistry );
			// just return this one
			return( contextSiteRegistry );			                             
        }
        
		InputStream inStream = file.toURI().toURL().openStream();
						
//		JOptionPane.showMessageDialog(null, tt, Translations.getString("PROCESSTOOLS_38"), //$NON-NLS-1$
//						JOptionPane.INFORMATION_MESSAGE);
            		        
//        InputStream inStream = new URL( "file", "localhost", filename ).openStream(); //$NON-NLS-1$ //$NON-NLS-2$
       	ObjectInputStream ois = new ObjectInputStream( inStream );

        // Attempt to read the data that has been serialized out to disk.
   	    contextSiteRegistry = (ContextSiteRegistry)ois.readObject();

       	// Close the input streams.
        inStream.close();
   	    ois.close();	        
        
        return( contextSiteRegistry );
	}

	/**
	 * Set to not show dialog again. Inherently, this simply deposits
	 * a file into the application directory.	 
	 */
	public static void doNotShowChooserAgain()
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	// create it
        	file.mkdir();
        }
        
        String filename = homeDir + DO_NOT_SHOW_FILE_NAME;
        file = new File( filename );
        if( file.exists() )
        {
        	// already there, no need to do anything else
        	return;    
        }

        // opens a file output stream and serializes a simple Boolean object
        FileOutputStream fos = null;
    	ObjectOutputStream oos = null;
        fos = new FileOutputStream(filename);
   	    oos = new ObjectOutputStream(fos);

   	    oos.writeObject( new Boolean( true ) );

        fos.close();
        oos.close();        
	}
	
	/**
	 * Find out whether chooser is to be shown again. 
	 */
	public static boolean isShowChooserAgain()
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	return( true );
        }
        
        String filename = homeDir + DO_NOT_SHOW_FILE_NAME;
        file = new File( filename );
        if( file.exists() )
        {
        	// already there, so do not show
        	return( false );    
        }
        
        return( true );
	}
	
	/**
	 * Find out whether chooser is to be shown in this session. 
	 */
	/*public static boolean isShowChooserInSession()
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	return( false );
        }
        
        String filename = homeDir + CHOOSER_SHOW_IN_SESSION_FILE_NAME;
        file = new File( filename );
        if( file.exists() )
        {
        	// is there, so do show, but delete file before returning
        	// so same session won't show again
        	file.delete();        	
        	return( true );    
        }
        
        return( false );
	}*/
	
	/**
	 * Set to come from extended help itself. Inherently, this simply deposits
	 * a file into the application directory.	 
	 */
	public static void setFromExtendedHelpLaunch()
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	// create it
        	file.mkdir();
        }
        
        String filename = homeDir + FROM_EXTENDED_HELP_FILE_NAME;
        file = new File( filename );
        if( file.exists() )
        {
        	// already there, no need to do anything else
        	return;    
        }

        // opens a file output stream and serializes a simple Boolean object
        FileOutputStream fos = null;
    	ObjectOutputStream oos = null;
        fos = new FileOutputStream(filename);
   	    oos = new ObjectOutputStream(fos);

   	    oos.writeObject( new Boolean( true ) );

        fos.close();
        oos.close();
	}
	
	/**
	 * Find out whether launching from extended help.
	 */
	public static boolean isFromExtendedHelp()
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	return( false );
        }
        
        String filename = homeDir + FROM_EXTENDED_HELP_FILE_NAME;
        file = new File( filename );
        if( file.exists() )
        {
        	// is there, so ignore default, but delete file before returning
        	// so next time it won't ignore until it's explicitly set again
        	file.delete();        	
        	return( true );    
        }
        
        return( false );
	}
	
	/**
	 * Remove launch from extended help setting.
	 */
	public static void removeFromExtendedHelpLaunch()
			throws MalformedURLException, IOException, ClassNotFoundException
	{
        // directory
        String homeDir = getHomeDirectory();
        // file
        File file = new File( homeDir );
        if( !file.exists() )
        {
        	return;
        }
        
        String filename = homeDir + FROM_EXTENDED_HELP_FILE_NAME;
        file = new File( filename );
        if( file.exists() )
        {
        	file.delete();        	
        }
	}
	
	/**
	 * Gets the home directory.
	 */
	private static String getHomeDirectory()
	{
        // directory
        String homeDir = System.getProperty( "user.home" ) + //$NON-NLS-1$
                         File.separator +
                         "rup" + //$NON-NLS-1$
                         File.separator;
		
		return( homeDir );
	}
	
}

