package com.rational.rpw.processtools;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.ImageIcon;

import com.rational.rpw.dnd.CustomDnDConstants;
import com.rational.rpw.processview.Bookmark;
import com.rational.rpw.processview.BookmarkLibrary;
import com.rational.rpw.processviewer.DocumentBase;
import com.rational.rpw.processview.IconManager;
import com.rational.rpw.processviewer.ViewerTabPanel;

/**
 * This class allows the user to select a bookmark node from the
 * tree. It provides the neccessary frame and UI view. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class DefaultViewTreeSelector extends JDialog
	implements ActionListener
{
	// the bookmark to show and select from
	private Bookmark _currentBookmark = null;
	
	// the bookmarks to choose from
	private Vector _bookmarksToChoose = null;
	
	// the icon manager
	private IconManager _iconManager = null;	
	
	// the combo box
	private JComboBox _bookmarkComboBox = null;
	
	// first time this is launched
	private static boolean _isFirstTime = true;
		
	/**
	 * Default constructor that takes in a list of chooseable bookmarks.
	 */
	public DefaultViewTreeSelector( Vector bookmarksToChoose,
						 			IconManager iconManager,
						 			JFrame owner,
						 			String title )
	{
		super( owner, title, false );
		
		// initialize
		_bookmarksToChoose = new Vector();
		for( int i = 0; i < bookmarksToChoose.size(); i++ )
		{
			_bookmarksToChoose.addElement( (Bookmark)bookmarksToChoose.elementAt( i ) );
		}
		_iconManager = iconManager;
		
		// set icon
		ImageIcon icon = _iconManager.getResource( "rup1.gif" );
		if( null != icon )
		{
			owner.setIconImage( icon.getImage() );
		}
		
		initialize();
	}
	
	/**
	 * Initializes the view, by building a frame and
	 * associated UI widgets.
	 */
	private void initialize()
	{
		Vector loadedBookmarks = new Vector();
		// first make sure that all bookmarks are loaded
		for( int i = 0; i < _bookmarksToChoose.size(); i++ )
		{
			Bookmark bookmark = (Bookmark)_bookmarksToChoose.elementAt( i );
			if( bookmark.getChildCount() == 0 )
			{
				try
				{
					Component parent = SwingUtilities.getRoot( (Component)this );
					parent.setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR) );	
							
					// this bookmark was not loaded, load it now
					Bookmark loadedBookmark = BookmarkLibrary.readBookmark( DocumentBase.getDocumentBase(),
				    	                                                    bookmark.getPresentationName() );					    	                                     					    

					loadedBookmarks.addElement( loadedBookmark );

					parent.setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
				}
				catch( Exception ex )
				{
					ex.printStackTrace();
				}
			}
			else
			{
				loadedBookmarks.addElement( bookmark );
			}	
		}
		_bookmarksToChoose = loadedBookmarks;
		
		if( null == _bookmarkComboBox &&
		    null != _bookmarksToChoose && 
		    null == _currentBookmark )
		{
			// creates the combo box for multiple bookmarks
			Vector bookmarkNameList = new Vector();
			for( int i = 0; i < _bookmarksToChoose.size(); i++ )
			{
				bookmarkNameList.addElement( ((Bookmark)_bookmarksToChoose.elementAt( i )).getPresentationName() );
			}
	        _bookmarkComboBox = new JComboBox( bookmarkNameList );
	        _bookmarkComboBox.setSelectedIndex( 0 );
 	        _bookmarkComboBox.addActionListener( this );
	        
	        _currentBookmark = (Bookmark)_bookmarksToChoose.elementAt( 0 );	        	        
		}
		else if( null == _bookmarkComboBox &&
		         null == _bookmarksToChoose && 
		         null != _currentBookmark )
		{
			// creates the combo box for one bookmark
			_bookmarksToChoose = new Vector();
			_bookmarksToChoose.addElement( _currentBookmark.getPresentationName() );
			_bookmarkComboBox = new JComboBox( _bookmarksToChoose );			
		}
		else if( null != _bookmarkComboBox )
		{
			// combo box already created, do nothing
		}

        
        // create the tree as a draggable source with a renderer
        // using the current bookmark
        ViewerTabPanel processView = new ViewerTabPanel();        
        processView.addView( _currentBookmark, 
                             true,
                             CustomDnDConstants.ACTION_MOVE_NO_REMOVE,
                             false, 
                             CustomDnDConstants.ACTION_MOVE_NO_REMOVE,
                             false, 
                             null, 
                             new RupPresenterTreeCellRenderer( _iconManager ) );

		// put all on the frame
		this.getContentPane().removeAll();
    	BorderLayout borderLayout = new BorderLayout();
    	borderLayout.setVgap( 5 );    
        this.getContentPane().setLayout( borderLayout );
        
        this.getContentPane().add( _bookmarkComboBox, BorderLayout.NORTH );
        this.getContentPane().add( processView.getViewPane( _currentBookmark.getPresentationName(),
                                                            false,
                                                            Color.white ),
                                   BorderLayout.CENTER );
		    
	    this.pack();
	    if( _isFirstTime )
	    {
	    	// center on screen the first time, but if user moves it later,
	    	// don't re-center it
	    	this.setLocationRelativeTo( null );
	    	_isFirstTime = false;
	    }
	}


	/**
	 * Action listener.
	 */
	public void actionPerformed( ActionEvent e )
	{
		JComboBox cb = (JComboBox)e.getSource();
	    String bookmarkName = (String)cb.getSelectedItem();
	    if( _bookmarksToChoose != null )
	    {
	    	for( int i = 0; i < _bookmarksToChoose.size(); i++ )
	    	{
	    		Bookmark bookmark = (Bookmark)_bookmarksToChoose.elementAt( i );
	    		if( bookmark.getPresentationName().equals( bookmarkName ) )
	    		{
	    			_currentBookmark = bookmark;
	    			break;
	    		}
	    	}
	    }
	    initialize();
	    this.validate();
	    this.repaint();
	}
	

		
}

