package com.rational.rpw.processtools;

import java.io.File;

import com.rational.rpw.processviewer.ViewerUtilities;
import com.rational.rpw.processview.Bookmark;

/**
 * This class finds the node specified by the given url
 * in the specified bookmark. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class NodeFinder
{
	/**
	 * Default constructor.
	 */
	public NodeFinder()
	{
	}
	
    /**
     * Finds the node with the url.
     */
    public Bookmark findNode( String documentUrl,
    						  String documentReadRoot,
                              Bookmark rootBookmark )
    {
    	// first account for netscape
    	String browserDocumentUrl = ViewerUtilities.handleNetscapeFilePrefix( documentUrl );
    	
    	// first convert any %20 spacings
    	String tempDocumentUrl = ViewerUtilities.convertFileSpacing( browserDocumentUrl );
    	//System.out.println( "tempDocumentUrl = " + tempDocumentUrl );
    	String uncDocument = ViewerUtilities.convertToUNC( tempDocumentUrl );    	    	    	
    	//System.out.println( "uncDocument = " + uncDocument );
    	String tempUncDocument = uncDocument.replace( ViewerUtilities.UNC_SEPARATOR.charAt( 0 ), 
    	                                              File.separator.charAt( 0 ) );
    	//System.out.println( "tempUncDocument = " + tempUncDocument );
  		//System.out.println( "documentReadRoot = " + documentReadRoot );  	                                              
    	String tempDocumentReadRoot = ViewerUtilities.convertFileSpacing( documentReadRoot );
    	//System.out.println( "tempDocumentReadRoot = " + tempDocumentReadRoot );
    	String finalDocumentReadRoot = tempDocumentReadRoot.replace( ViewerUtilities.UNC_SEPARATOR.charAt( 0 ), 
    	                                                             File.separator.charAt( 0 ) );

    	//System.out.println( "finalDocumentReadRoot = " + finalDocumentReadRoot );
    	String documentOnly = null;
    	// start checking after any file or http prefix
    	if( tempUncDocument.startsWith( finalDocumentReadRoot ) )
    	{
	    	// strip to just the file name after the document root
    		documentOnly = ViewerUtilities.convertFilename(
    							uncDocument.substring( finalDocumentReadRoot.length() ) );
    	}
    	else
    	{
    		documentOnly = ViewerUtilities.convertFilename( uncDocument );
    	}		
     	//System.out.println( "documentOnly = " + documentOnly );
     	    	
    	// now find it in the bookmark
    	Bookmark foundBookmark = findBookmark( rootBookmark, documentOnly );
    	if( null == foundBookmark )
    	{
    		// this might be a special case; on Windows XP, the document read root
    		// doesn't match the document url usage of the %20 spacing
    		foundBookmark = findNodeWithSpacing( browserDocumentUrl, documentReadRoot, rootBookmark );
    	}
    	
    	if( null == foundBookmark )
    	{
    		// this might be a special case involving UNC paths;
    		// last ditch attempt
    		if( finalDocumentReadRoot.startsWith( "file:\\\\" ) )
    		{
    			if( tempUncDocument.startsWith( "file:\\\\\\\\" ) )
    			{
    				tempUncDocument = "file:\\\\" + tempUncDocument.substring( 9 );
    			}
    			else if( tempUncDocument.startsWith( "file:\\\\\\" ) )
    			{
    				tempUncDocument = "file:\\\\" + tempUncDocument.substring( 8 );
    			}
    			else if( tempUncDocument.startsWith( "file:\\" ) )
    			{
    				tempUncDocument = "file:\\\\" + tempUncDocument.substring( 6 );
    			}
    			//System.out.println( "last ditch tempUncDocument = " + tempUncDocument );
    			// now find bookmark again
		    	if( tempUncDocument.startsWith( finalDocumentReadRoot ) )
    			{
	    			// strip to just the file name after the document root
    				documentOnly = ViewerUtilities.convertFilename(
    									tempUncDocument.substring( finalDocumentReadRoot.length() ) );
		    	}
		    	//System.out.println( "last ditch documentOnly = " + documentOnly );
		    	foundBookmark = findBookmark( rootBookmark, documentOnly );
    			
    		}
    	}
    	
    	if( null == foundBookmark )
    	{
    		// see if this is a mismatch of slashes; this situation
    		// occurs when a site from windows is used on unix
    		String convertedDocumentOnly = documentOnly.replace( '\\', '/' );
    		foundBookmark = findBookmark( rootBookmark, convertedDocumentOnly );
    		if( null == foundBookmark )
    		{
    			// try the other way
    			convertedDocumentOnly = documentOnly.replace( '/', '\\' );
    			foundBookmark = findBookmark( rootBookmark, convertedDocumentOnly );
    		}
    	}
    	
    	return( foundBookmark );	
    }
    
    /**
     * The special case find node.
     */
    private Bookmark findNodeWithSpacing( String documentUrl,
    						  			  String documentReadRoot,
                                          Bookmark rootBookmark )
    {
    	boolean status = false;

    	String uncDocument = ViewerUtilities.convertToUNC( documentUrl );
    	String uncWithSpacing = ViewerUtilities.convertFileSpacing( uncDocument );
    	String uncFinal = ViewerUtilities.convertToUNC( uncWithSpacing );

    	String documentOnly = null;
    	String tempUncDocument = uncDocument.replace( ViewerUtilities.UNC_SEPARATOR.charAt( 0 ), 
    	                                              File.separator.charAt( 0 ) );
    	String tempDocumentReadRoot = documentReadRoot.replace( ViewerUtilities.UNC_SEPARATOR.charAt( 0 ), 
    	                                                         File.separator.charAt( 0 ) );
    	// start checking after any file or http prefix
    	if( tempUncDocument.startsWith( tempDocumentReadRoot ) )
    	{
	    	// strip to just the file name after the document root
    		documentOnly = uncDocument.substring( documentReadRoot.length() );
    	}
    	else
    	{
    		documentOnly = ViewerUtilities.convertFilename( uncDocument );
    	}

    	// now find it in the bookmark
    	Bookmark foundBookmark = findBookmark( rootBookmark, documentOnly );
    	return( foundBookmark );
     }
     
    /**
     * Recursively finds a bookmark that matches the given filename.
     */
    private Bookmark findBookmark( Bookmark bookmarkToFind,
    							   String filename )
    {
    	for( int i = 0; i < bookmarkToFind.getChildCount(); i++ )
    	{
    		Bookmark childBookmark = (Bookmark)bookmarkToFind.getChildAt( i );
    		String childFilename = childBookmark.getFileName();
    		if( null != childFilename &&
    		    childFilename.trim().equals( filename.trim() ) )
    		{
    			// found, no need to go further
    			return( childBookmark );
    		}
    		else
    		{
    			// recursive find
    			Bookmark foundBookmark = findBookmark( childBookmark, filename );
    			if( null != foundBookmark )
    			{
    				return( foundBookmark );
    			}
    		}
    	}
    	
    	return( null );	
    }
    
     
}

