package com.rational.rpw.processtools;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.dnd.DnDConstants;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.rational.rpw.processview.Bookmark;
import com.rational.rpw.processview.BookmarkLibrary;
import com.rational.rpw.processview.BookmarkProperties;
import com.rational.rpw.processview.IconManager;
import com.rational.rpw.processview.TopicFile;
import com.rational.rpw.processview.TopicFileParser;
import com.rational.rpw.processviewer.DocumentBase;
import com.rational.rpw.processviewer.IViewerActionListener;
import com.rational.rpw.processviewer.ViewerPopupMenuAdapter;
import com.rational.rpw.processviewer.ViewerTabPanel;
import com.rational.rpw.processviewer.ViewerTreeModelListener;
import com.rational.rpw.processviewer.ViewerUtilities;
import com.rational.rpw.rpwapplication.LimitedTextField;
import com.rational.rpw.rpwapplication.RunExternalProgram;

/**
 * This class is the applet for the myRup and Extended Help features
 * on the process website. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class RupPresenterApplet
	extends JApplet
	implements IViewerActionListener, ChangeListener
{
	protected static final String APPLET_DIRECTORY = "applet"; //$NON-NLS-1$
	protected static final String RUP_PRESENTER_DIRECTORY = "rup"; //$NON-NLS-1$
	protected static final String RUP_EMPTY_PAGE = "applet/node_empty_page.htm"; //$NON-NLS-1$

	// the static variables are used so that refresh of the applet
	// won't cause a reload of them (time-consuming); the primary
	// assumption is that the website does not change in between
	// refreshes (for example, no new default bookmarks are added during
	// runtime)
	protected static DefaultViewTreeSelector _selector = null;
	protected static Bookmark _xhelpBookmark = null;
	protected static Vector _defaultFileList = null;
	protected static Vector _defaultBookmarkList = null;

	protected TreeListener _treeSelectionListener = null;
	protected MouseSelectionListener _mouseSelectionListener = null;
	protected KeySelectionListener _keySelectionListener = null;

	protected ViewerTabPanel _tabPanel = null;
	protected RupPresenterIconMenu _mainIconMenu = null;
	protected RupPresenterTopIconMenu _topIconMenu = null;
	protected IconManager _defaultIconManager = null;

	protected String _documentReadRoot = null;

	protected String _applicationDirectory = null;
	protected URL _applicationDirectoryUrl = null;

	protected String _imageFile = null;

	// variables for threads (needed because unsigned javascript
	// cannot call signed applet methods so they need to be in 
	// a thread)
	protected StringBuffer _treePathArrayBuffer = null;
	protected String _scriptDocUrl = null;
	protected boolean _scriptStatus = false;
	protected Bookmark _scriptFoundBookmark = null;
	protected String _viewStateFile = null;

	protected boolean _isFileExistSet = false;
	protected boolean _isShowXhelp = false;
	protected boolean _isAccessExceptionCaught = false;

	protected boolean _isToStart = true;

	protected static ResourceBundle _resourceBundle = ResourceBundle.getBundle("com.rational.rpw.processtools.RupPresenterAppletBundle"); //$NON-NLS-1$

	protected String lookAndFeelClass = UIManager.getSystemLookAndFeelClassName();

	/**
	 * Default constructor.
	 */
	public RupPresenterApplet()
	{
		super();
		// disable system event queque check
		this.getRootPane().putClientProperty("defeatSystemEventQueueCheck", //$NON-NLS-1$
		Boolean.TRUE);
	}

	/**
	 * Init method needed by applet.
	 */
	public void init()
	{
		try {
			final RupPresenterApplet test = this;
			
						AccessController
										.doPrivileged(new PrivilegedAction() {
											public Object run() {
			
		try
		{			
			// set look and feel
			//UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

			// set look and feel. 
			// system look and feel caused problem on windows xp. change to use the platform independent one
			try
			{
				String osName = System.getProperty("os.name");
				if (osName.equalsIgnoreCase("Windows XP"))
				{
					String javaVersion = System.getProperty("java.version");
					int indx;
					while ((indx = javaVersion.indexOf('.')) >= 0)
					{
						javaVersion =
							javaVersion.substring(0, indx)
								+ javaVersion.substring(indx + 1);
					}
					int length = javaVersion.length();
					if (length == 1)
					{
						javaVersion += "00";
					}
					else if (length == 2)
					{
						javaVersion += "0";
					}
					else if (length > 3)
					{
						javaVersion = javaVersion.substring(0, 3);
					}
					int version = Integer.parseInt(javaVersion);
					if (version > 141)
					{
						lookAndFeelClass = "javax.swing.plaf.metal.MetalLookAndFeel";
					}
				}
			}
			catch (Throwable th)
			{
				th.printStackTrace();
			}

			UIManager.setLookAndFeel(lookAndFeelClass);

			// set up multiple selection for tree drag and drop
			UIDefaults uidef = UIManager.getLookAndFeelDefaults();
			uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$

			// detect and handle JRE version
			String jreVersion = System.getProperty("java.version");
			// strip out everything before underscore to just get master version
			StringTokenizer versionTokenizer = new StringTokenizer(jreVersion, "_");
			if (versionTokenizer.hasMoreTokens())
			{
				jreVersion = versionTokenizer.nextToken();
			}
			// strip out periods to generate numeric version	
			StringTokenizer stripTokenizer = new StringTokenizer(jreVersion, ".");
			StringBuffer strippedVersion = new StringBuffer();
			while (stripTokenizer.hasMoreTokens())
			{
				strippedVersion.append(stripTokenizer.nextToken());
			}
			int numericVersion = Integer.parseInt(strippedVersion.toString());
			if (numericVersion < 141)
			{
				// not 1.4.1 and higher, must inform user and exit
				JOptionPane.showMessageDialog(null, Translations.getString("PROCESSTOOLS_122"), //$NON-NLS-1$
				Translations.getString("PROCESSTOOLS_121"), //$NON-NLS-1$
				JOptionPane.INFORMATION_MESSAGE);
				test.stop();
				test.destroy();
				System.exit(0);
			}

			// set the document base
			DocumentBase.setDocumentBase(test.getDocumentBase());

			// determine the document root
			String documentBase = test.getDocumentBase().toString();
			int last = documentBase.lastIndexOf(APPLET_DIRECTORY);
			_documentReadRoot = documentBase.substring(0, last);
			DocumentBase.setDocumentReadRoot(_documentReadRoot);

			// set the application directory
				_applicationDirectory = System.getProperty("user.home") + //$NON-NLS-1$
	File.separator + RUP_PRESENTER_DIRECTORY + File.separator;
			File appDir = new File(_applicationDirectory);
			if (!appDir.exists())
			{
				appDir.mkdir();
			}
			if (_applicationDirectory.startsWith(ViewerUtilities.UNC_SEPARATOR))
			{
				_applicationDirectoryUrl =
					new URL(
						ViewerUtilities.UNC_FILE_ONLY
							+ ViewerUtilities.convertFilenameToUNC(_applicationDirectory));
			}
			else
			{
				_applicationDirectoryUrl =
					new URL(
						ViewerUtilities.UNC_FILE_PREFIX
							+ ViewerUtilities.convertFilenameToUNC(_applicationDirectory));
			}

			// show the default page
			showPage(test.getParameter("loadpage")); //$NON-NLS-1$

			// default file names
			_imageFile = test.getParameter("imagefile"); //$NON-NLS-1$
			_viewStateFile = test.getParameter("viewstatefile"); //$NON-NLS-1$

			// register site for extended help
			ContextSiteRegistryLibrary.addSiteRoot(_documentReadRoot);

			// determine whether to show extended help
			String showXhelp = test.getParameter("showxhelp"); //$NON-NLS-1$
			if (showXhelp.equals("yes")) //$NON-NLS-1$
			{
				_isShowXhelp = true;
				// if it's xhelp, remove any view state cookies
				removeViewState();
			}
			if (!_isShowXhelp)
			{
				// set up context for extended help to any, any
				writeXhelpFile();
			}
			else
			{
				// make sure the file exists
				File xhelpFile = new File(_applicationDirectory + "xhelp.context"); //$NON-NLS-1$
				if (!xhelpFile.exists())
				{
					writeXhelpFile();
				}

				// launch the site chooser if applicable
				ContextSiteChooser siteChooser = new ContextSiteChooser();
				String chosenSite = null;
				try
				{
					siteChooser.initialize(true);
					if (ContextSiteRegistryLibrary.isFromExtendedHelp())
					{
						// don't do anything, just continue using this site that
						// was launched from extended help applet itself
					}
					else if (siteChooser.isChooserApplicable())
					{
						// came from suite point product, XDE, etc.
						int value = siteChooser.show(getAppletHandle());
						if (value == JOptionPane.OK_OPTION)
						{
							// launch on chosen site and quit this one
							chosenSite = siteChooser.getChosenSite();
						}
					}
					else if (!siteChooser.isChooserApplicable())
					{
						// came from suite point product, XDE, etc., but
						// the dialog was not shown, so use default that was
						// set in the registry
						String defaultSite =
							ContextSiteRegistryLibrary
								.readContextSiteRegistry()
								.getDefaultSite();
						String thisSite =
							_documentReadRoot
								+ ContextSiteRegistryLibrary.CONTEXT_LAUNCH_FILE;
						if (null != defaultSite && !defaultSite.equals(thisSite))
						{
							chosenSite = defaultSite;
						}
					}
				}
				catch (Exception e)
				{
					e.printStackTrace();
					showErrorMessage(Translations.getString("PROCESSTOOLS_117")); //$NON-NLS-1$
				}
				if (null != chosenSite)
				{
					// check whether chosen site is available; sometimes, they might
					// have been deleted or moved by the user outside of the application
					URL chosenUrl = new URL(chosenSite);
					// handle UNC
					chosenUrl = ViewerUtilities.handleURLForUNC(chosenUrl);

					try
					{
						chosenUrl.openConnection().getInputStream();
						_isToStart = false;
						ContextSiteRegistryLibrary.setFromExtendedHelpLaunch();
						// launch the chosen site only when it's available here
						test.getAppletContext().showDocument(chosenUrl, "_top"); //$NON-NLS-1$
						test.stop();
						test.destroy();
						return null;
					}
					catch (IOException ioe)
					{
						ioe.printStackTrace();
						ContextSiteRegistryLibrary.removeFromExtendedHelpLaunch();
						// chosen site not available, just continue this one
						chosenUrl = null;
						_isToStart = true;
						Object[] messageArguments = { chosenSite };
						String message = MessageFormat.format(Translations.getString("PROCESSTOOLS_119"), messageArguments); //$NON-NLS-1$		
						showMessage(message);
					}
				}
			}

			// set up the process view
			ContextSiteRegistryLibrary.removeFromExtendedHelpLaunch();
			setupView();								
		}
		catch (java.security.AccessControlException ae)
		{
			handleAccessException();
		}
		catch (Exception e)
		{
			e.printStackTrace();
			showErrorMessage(Translations.getString("PROCESSTOOLS_61")); //$NON-NLS-1$
		}
		return null;
											}
																							});
		} catch (Exception e) {
					e.printStackTrace();
				}
	}

	/**
	 * Applet method called.
	 */
	public void start()
	{
		if (_isAccessExceptionCaught)
		{
			// access exception caught earlier, do not proceed
			return;
		}

		if (_isToStart)
		{
			// construct frame, leave focus to the refresh view handler
			refreshView(null);

			// if finishpage not available, then show first node in 
			// first tree. Otherwise, show finishpage. This handles
			// both cases when the applet is started standalone or
			// from a "Display Tree Browser" page
			String finishPage = this.getParameter("finishpage"); //$NON-NLS-1$
			if (null == finishPage)
			{
				// show finish page as first node in first tree in focus
				showFirstNodeInActiveTree();
			}
			else
			{
				// show the finish page
				showPage(finishPage);
			}

			//showPage( this.getParameter( "finishpage" ) ); //$NON-NLS-1$
		}
	}

	/**
	 * Applet method called.
	 */
	public void stop()
	{
		_tabPanel = null;
	}

	protected JApplet getAppletHandle()
	{
		return (this);
	}

	/**
	 * Show the page.
	 */
	public void showPage(String pageName)
	{
		String target = this.getParameter("target"); //$NON-NLS-1$
		if (null != pageName)
		{
			try
			{
				String uncPageName = ViewerUtilities.convertFilenameToUNC(pageName);

				URL fullUrl = null;
				// first check for existence
				if (uncPageName.startsWith("../")) //$NON-NLS-1$
				{
					// we know this came from the RUP
					fullUrl = new URL(_documentReadRoot + uncPageName.substring(3));
				}
				else if (uncPageName.startsWith("file") || uncPageName.startsWith("http")) //$NON-NLS-1$
				{
					// this is already with file url format or came from a webserver
					// replace spaces by standard %20
					if (uncPageName.indexOf(" ") > 0)
					{
						String name = uncPageName.replaceAll(" ", "%20");
						fullUrl = new URL(name);
					}
					else
					{
						fullUrl = new URL(uncPageName);
					}
				}
				else if (uncPageName.startsWith("www.")) //$NON-NLS-1$
				{
					// this came from a web server somewhere
					fullUrl = new URL("http://" + uncPageName);
				}
				else
				{
					// this came from a file somewhere
					fullUrl = new URL("file", "", uncPageName); //$NON-NLS-1$ //$NON-NLS-2$
				}

				// handle special case for Mozilla over UNC
				fullUrl = ViewerUtilities.handleURLForUNC(fullUrl);

				// check for types of files
				Enumeration configKeys = _resourceBundle.getKeys();
				while (configKeys.hasMoreElements())
				{
					String key = (String) configKeys.nextElement();
					String fileTypeForNewTarget = _resourceBundle.getString(key);
					if (fullUrl.toString().endsWith(fileTypeForNewTarget))
					{
						target = "_blank"; //$NON-NLS-1$
						break;
					}
				}

				// check for extended help sites
				if (pageName.endsWith(ContextRupPresenter.SELECT_OTHER_SITES))
				{
					// create another thread to obtain user input to save the tree 	
					Runnable chooseSiteUI = new Runnable()
					{
						public void run()
						{
							try
							{
								UIManager.setLookAndFeel(lookAndFeelClass);
								// set up multiple selection for tree drag and drop
								UIDefaults uidef = UIManager.getLookAndFeelDefaults();
								uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$
								// launch the chooser
								ContextSiteChooser siteChooser = new ContextSiteChooser();
								siteChooser.initialize(false);
								String chosenSite = null;
								int value = siteChooser.show(null);
								if (value == JOptionPane.OK_OPTION)
								{
									// launch on chosen site and quit this one
									chosenSite = siteChooser.getChosenSite();
									RunExternalProgram program = new RunExternalProgram();
									String name =
										ViewerUtilities.convertFilename(chosenSite);
									String finalName =
										ViewerUtilities.convertFileSpacing(name);
									// set to launch just this site and set that this is
									// launched from extended help applet
									ContextSiteRegistryLibrary
										.setFromExtendedHelpLaunch();
									try
									{
										program.invokeBrowser(finalName);
									}
									catch (IOException ex)
									{
										ex.printStackTrace();
										ContextSiteRegistryLibrary
											.removeFromExtendedHelpLaunch();
										JOptionPane.showMessageDialog(null, Translations.getString("PROCESSTOOLS_62")); //$NON-NLS-1$
										ContextSiteRegistryLibrary.removeSite(chosenSite);
									}
								}
							}
							catch (Exception e)
							{
								e.printStackTrace();
								try
								{
									ContextSiteRegistryLibrary
										.removeFromExtendedHelpLaunch();
								}
								catch (Exception ee)
								{
								}
								showErrorMessage(Translations.getString("PROCESSTOOLS_118")); //$NON-NLS-1$
							}
						}
					};

					SwingUtilities.invokeLater(chooseSiteUI);
					return;
				}

				this.getAppletContext().showDocument(fullUrl, target);
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
				showErrorMessage(Translations.getString("PROCESSTOOLS_14")); //$NON-NLS-1$
			}
		}
		else
		{
			// show an "empty" page from the RUP
			try
			{
				URL fullUrl = new URL(_documentReadRoot + RUP_EMPTY_PAGE);
				this.getAppletContext().showDocument(fullUrl, target);
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
				showErrorMessage(Translations.getString("PROCESSTOOLS_14")); //$NON-NLS-1$
			}
		}
	}

	/**
	 * Show the page pointed by the bookmark.
	 */
	public void showPage(Bookmark bookmark)
	{
		if (null != bookmark)
		{
			String filename = null;
			if (bookmark.isFromContentLibrary() && bookmark.getFileName() != null)
			{
				filename = _documentReadRoot + bookmark.getFileName();
			}
			else
			{
				filename = bookmark.getFileName();
			}
			showPage(filename);
		}
	}

	/**
	 * Handles viewer actions.
	 */
	/**
	 * Handles action that the user has requested.
	 * @param actionType	One of the action types in this interface.
	 */
	public void handleAction(int actionType)
	{
		if (actionType == IViewerActionListener.ACTION_ADD_FROM_DEFAULT)
		{
			addFromDefault();
		}
		else if (actionType == IViewerActionListener.ACTION_ADD_NEW_NODE)
		{
			this.addNewNode();
		}
		else if (actionType == IViewerActionListener.ACTION_DELETE_VIEW)
		{
			this.deleteTree();
		}
		else if (actionType == IViewerActionListener.ACTION_SAVE_VIEW_AS)
		{
			this.saveAsTree();
		}
	}

	/**
	 * Handles action that the user has requested.
	 * @param actionType	One of the action types in this interface.
	 * @param objectForAction	An object that is required for the action.
	 */
	public void handleAction(int actionType, Object objectForAction)
	{
	}

	/**
	 * Change in the tabbed pane. Specifically, loads a default bookmark
	 * if not already loaded.
	 */
	public void stateChanged(ChangeEvent e)
	{
		JTabbedPane tabPane = (JTabbedPane) e.getSource();
		String selectedName = tabPane.getTitleAt(tabPane.getSelectedIndex());
		// only care about default bookmarks
		for (int i = 0; i < _defaultBookmarkList.size(); i++)
		{
			Bookmark bookmark = (Bookmark) _defaultBookmarkList.elementAt(i);
			if (selectedName.equals(bookmark.getPresentationName()))
			{
				if (bookmark.getChildCount() == 0)
				{
					try
					{
						Component parent = SwingUtilities.getRoot((Component) this);
						parent.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

						// this bookmark was not loaded, load it now
						Bookmark loadedBookmark =
							BookmarkLibrary.readBookmark(
								this.getDocumentBase(),
								selectedName);
						_tabPanel.removeView(selectedName);
						_tabPanel.addView(
							loadedBookmark,
							false,
							new RupPresenterTreeCellRenderer(_defaultIconManager));

						_defaultBookmarkList.remove(bookmark);
						_defaultBookmarkList.add(loadedBookmark);
						refreshView(selectedName);

						parent.setCursor(
							Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						break;
					}
					catch (Exception ex)
					{
						ex.printStackTrace();
						showErrorMessage(Translations.getString("PROCESSTOOLS_63") + selectedName + Translations.getString("PROCESSTOOLS_64")); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			}
		}
	}

	/**
	 * Select and show first node in active tree.
	 */
	private void showFirstNodeInActiveTree()
	{
		try
		{
			if (null != _tabPanel)
			{
				// now point tree selection to first node
				Bookmark rootBookmark = _tabPanel.getBookmarkForSelectedView();
				Bookmark firstBookmark = (Bookmark) rootBookmark.getFirstChild();
				_tabPanel.setSelectedBookmarkInTree(firstBookmark);
				showPage(firstBookmark);
			}
		}
		catch (Exception e)
		{
			// usually node has no children, ignore it
		}
	}

	/**
	 * Saves the currently selected tree as ... Note only MyRup trees
	 * can be saved.
	 */
	private void saveAsTree()
	{
		// create another thread to obtain user input to save the tree 	
		Runnable saveTreeUI = new Runnable()
		{
			public void run()
			{
				try
				{
					UIManager.setLookAndFeel(lookAndFeelClass);
					// set up multiple selection for tree drag and drop
					UIDefaults uidef = UIManager.getLookAndFeelDefaults();
					uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

				// save the process view        
				Bookmark selectedBookmark =
					(Bookmark) _tabPanel.getBookmarkForSelectedView().clone();

				// prompts the user for a title
				SaveAsPanel panel = new SaveAsPanel();
				String notDefined = "not defined"; //$NON-NLS-1$
				String inputValue = notDefined;
				while (inputValue.equals(notDefined))
				{
						int value = JOptionPane.showConfirmDialog(null, panel, Translations.getString("PROCESSTOOLS_18"), //$NON-NLS-1$
	JOptionPane.OK_CANCEL_OPTION);

					if (value == JOptionPane.OK_OPTION)
					{
						// parse for non alpha numeric characters
						inputValue = panel.getName();
						for (int i = 0; i < inputValue.length(); i++)
						{
							char character = inputValue.charAt(i);
							if (!Character.isLetterOrDigit(character)
								&& !Character.isSpace(character))
							{
								// neither space or alpha numeric, try again
								inputValue = notDefined;
								JOptionPane.showMessageDialog(null, Translations.getString("PROCESSTOOLS_65")); //$NON-NLS-1$
								break;
							}
						}
						// parse for spaces at beginning or end
						char firstSpaceChar = inputValue.charAt(0);
						char lastSpaceChar = inputValue.charAt(inputValue.length() - 1);
						if (Character.isSpace(firstSpaceChar)
							|| Character.isSpace(lastSpaceChar))
						{
							// neither space or alpha numeric, try again
							inputValue = notDefined;
							JOptionPane.showMessageDialog(null, Translations.getString("PROCESSTOOLS_66")); //$NON-NLS-1$
						}
					}
					else
					{
						// user has pressed cancel so just return
						return;
					}
				}

				if (null != inputValue)
				{
					if (inputValue.equals("")) //$NON-NLS-1$
					{
						inputValue = Translations.getString("PROCESSTOOLS_20"); //$NON-NLS-1$
					}
					try
					{
						boolean isExist =
							BookmarkLibrary.isBookmarkExist(
								inputValue,
								_applicationDirectory);



						boolean isToSave = true;
						if (isExist)
						{
								String message = inputValue + " " + //$NON-NLS-1$
	Translations.getString("PROCESSTOOLS_21"); //$NON-NLS-1$
								int status = JOptionPane.showConfirmDialog(null, message, Translations.getString("PROCESSTOOLS_22"), //$NON-NLS-1$
	JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null);
							if (status != 0)
							{
								isToSave = false; // user chose not to override
							}
						}

						if (isToSave)
						{
							// ensure that name is not one of default trees
							if (isNameInDefaultList(inputValue))
							{
								String uniquePrompt = Translations.getString("PROCESSTOOLS_23"); //$NON-NLS-1$
								JOptionPane.showMessageDialog(null, uniquePrompt);
								return;
							}

							selectedBookmark.setPresentationName(inputValue);

							BookmarkLibrary.writeBookmark(
								selectedBookmark,
								inputValue,
								_applicationDirectory);



							// refresh the view, bring focus to newly added tree
							// remove view state so that view can be brought into focus
							removeViewState();
							setupView();
							refreshView(inputValue);
						}
					}
					catch (Exception e)
					{
						e.printStackTrace();
						showErrorMessage(Translations.getString("PROCESSTOOLS_24")); //$NON-NLS-1$
					}
				}
			}
		};

		SwingUtilities.invokeLater(saveTreeUI);
	}

	/**
	 * Deletes the currently selected tree. Note only MyRup trees can
	 * be deleted.
	 */
	private void deleteTree()
	{
		// create another thread to obtain user input to save the tree 	
		Runnable deleteTreeUI = new Runnable()
		{
			public void run()
			{
				try
				{
					UIManager.setLookAndFeel(lookAndFeelClass);
					// set up multiple selection for tree drag and drop
					UIDefaults uidef = UIManager.getLookAndFeelDefaults();
					uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

				Bookmark selectedBookmark = _tabPanel.getBookmarkForSelectedView();

				// make sure the currently selected tree is not one
				// of the default ones
				if (isDefaultBookmark(selectedBookmark))
				{
					showMessage(Translations.getString("PROCESSTOOLS_25")); //$NON-NLS-1$
					return;
				}
				if (isXhelpBookmark(selectedBookmark))
				{
					showMessage(Translations.getString("PROCESSTOOLS_67")); //$NON-NLS-1$
					return;
				}

				// prompts the user for confirmation
				Object[] messageArguments = { selectedBookmark.getPresentationName()};
					String message = MessageFormat.format(Translations.getString("PROCESSTOOLS_26"), //$NON-NLS-1$
	messageArguments);
					int status = JOptionPane.showConfirmDialog(null, message, Translations.getString("PROCESSTOOLS_28"), //$NON-NLS-1$
	JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null);
				if (status == 0)
				{
					try
					{
						BookmarkLibrary.deleteBookmark(


							selectedBookmark.getPresentationName(),
							_applicationDirectory);

						// remove view state; this is only effective when the user
						// is viewing one tree and that is the tree that is deleted;
						// we want to reset the view state at this point
						removeViewState();
					}
					catch (Exception e)
					{
						e.printStackTrace();
						showErrorMessage(Translations.getString("PROCESSTOOLS_29")); //$NON-NLS-1$
					}
					// refresh the view, leave focus to refresh view handler
					setupView();
					refreshView(null);
				}
			}
		};

		SwingUtilities.invokeLater(deleteTreeUI);
	}

	/**
	 * Adds a new node the currently selected tree.
	 */
	private void addNewNode()
	{
		// create another thread to obtain user input to save the tree 	
		Runnable addUI = new Runnable()
		{
			public void run()
			{
				try
				{
					UIManager.setLookAndFeel(lookAndFeelClass);
					// set up multiple selection for tree drag and drop
					UIDefaults uidef = UIManager.getLookAndFeelDefaults();
					uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

				// gets the currently selected bookmark
				Bookmark selectedBookmark = _tabPanel.getBookmarkForSelectedView();

				// make sure the currently selected tree is not one
				// of the default ones
				if (isDefaultBookmark(selectedBookmark))
				{
					showMessage(Translations.getString("PROCESSTOOLS_30")); //$NON-NLS-1$
					return;
				}
				if (isXhelpBookmark(selectedBookmark))
				{
					showMessage(Translations.getString("PROCESSTOOLS_68")); //$NON-NLS-1$
					return;
				}

				// initialize new bookmark
				Bookmark newBookmark = new Bookmark(Translations.getString("PROCESSTOOLS_31")); //$NON-NLS-1$
				newBookmark.setFromContentLibrary(false);
				String newIconName = "new_topic.gif"; //$NON-NLS-1$
				newBookmark.setOpenIconName(newIconName);
				newBookmark.setClosedIconName(newIconName);
				// prompt user for properties
				BookmarkProperties propertyDialog = new BookmarkProperties(newBookmark);
				int inputValue = propertyDialog.show(getAppletHandle());
				if (inputValue == JOptionPane.OK_OPTION)
				{
					Bookmark changedBookmark = propertyDialog.getChangedBookmark();
					String presentationName = null;
					if (changedBookmark.getPresentationName() == null || changedBookmark.getPresentationName().equals("")) //$NON-NLS-1$
					{
						presentationName = Translations.getString("PROCESSTOOLS_32"); //$NON-NLS-1$
					}
					else
					{
						presentationName = changedBookmark.getPresentationName();
					}
					newBookmark.setPresentationName(presentationName);
					newBookmark.setFileName(changedBookmark.getFileName());
					newBookmark.setOpenIconName(changedBookmark.getOpenIconName());
					newBookmark.setClosedIconName(changedBookmark.getClosedIconName());

					// add to the end of the tree
					selectedBookmark.add(newBookmark);
					_tabPanel.updateBookmark(selectedBookmark);

				}

			}
		};

		SwingUtilities.invokeLater(addUI);
	}

	/**
	 * Adds a new node to currently selected tree,
	 * from the default tree.
	 */
	private void addFromDefault()
	{
		// create another thread to obtain user input to save the tree 	
		Runnable addUI = new Runnable()
		{
			public void run()
			{
				try
				{
					UIManager.setLookAndFeel(lookAndFeelClass);
					// set up multiple selection for tree drag and drop
					UIDefaults uidef = UIManager.getLookAndFeelDefaults();
					uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

				// gets the currently selected bookmark
				Bookmark selectedBookmark = _tabPanel.getBookmarkForSelectedView();

				// make sure the currently selected tree is not one
				// of the default ones
				if (isDefaultBookmark(selectedBookmark))
				{
					showMessage(Translations.getString("PROCESSTOOLS_33")); //$NON-NLS-1$
					return;
				}
				if (isXhelpBookmark(selectedBookmark))
				{
					showMessage(Translations.getString("PROCESSTOOLS_81")); //$NON-NLS-1$
					return;
				}

				// have dialog to select the node
				if (null == _selector)
				{
					_selector = new DefaultViewTreeSelector(_defaultBookmarkList, _defaultIconManager, new JFrame(), Translations.getString("PROCESSTOOLS_34")); //$NON-NLS-1$
					_selector.show();
				}
				else
				{
					_selector.setVisible(true);
				}
			}
		};

		SwingUtilities.invokeLater(addUI);
	}

	/**
	 * Hides all but the currently selected tree.
	 */
	public void hideTrees()
	{
		// create another thread so unsigned javascript can call	
		Runnable scriptThread = new Runnable()
		{
			public void run()
			{
				if (null != _tabPanel)
				{
					try
					{
						UIManager.setLookAndFeel(lookAndFeelClass);
						// set up multiple selection for tree drag and drop
						UIDefaults uidef = UIManager.getLookAndFeelDefaults();
						uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$
						// store the bookmark to show, while others are hidden
						String filename =
							_applicationDirectory
								+ ViewerUtilities.UNC_SEPARATOR
								+ _viewStateFile;

						Bookmark selectedBookmark =
							_tabPanel.getBookmarkForSelectedView();

						FileOutputStream fos = null;
						ObjectOutputStream oos = null;
						fos = new FileOutputStream(filename);
						oos = new ObjectOutputStream(fos);

						oos.writeObject(selectedBookmark.getPresentationName());

						fos.close();
						oos.close();

						refreshView(selectedBookmark.getPresentationName());
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
			}
		};

		SwingUtilities.invokeLater(scriptThread);
	}

	/**
	 * Show all trees.
	 */
	public void showAll()
	{
		// create another thread so unsigned javascript can call	
		Runnable scriptThread = new Runnable()
		{
			public void run()
			{
				if (null != _tabPanel)
				{
					try
					{
						UIManager.setLookAndFeel(lookAndFeelClass);
						// set up multiple selection for tree drag and drop
						UIDefaults uidef = UIManager.getLookAndFeelDefaults();
						uidef.put("TreeUI", "com.rational.rpw.compositetreeview.MultiDragMetalTreeUI"); //$NON-NLS-1$ //$NON-NLS-2$
						// remove the view state file
						File file = new File(_applicationDirectory, _viewStateFile);
						file.delete();

						Bookmark selectedBookmark =
							_tabPanel.getBookmarkForSelectedView();
						refreshView(selectedBookmark.getPresentationName());
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
			}
		};

		SwingUtilities.invokeLater(scriptThread);
	}

	/**
	 * The isReady feature.
	 */
	public boolean isReady()
	{
		return (true);
	}

	/**
	 * Supports the tree path. Return a string of the following
	 * format:
	 * icon##*presentationName##*filename##*...
	 */
	public String getTreePathArray(String documentUrl)
	{
		// first strip out HTML anchor
		StringTokenizer anchorTokenizer = new StringTokenizer(documentUrl, "#");
		if (anchorTokenizer.hasMoreTokens())
		{
			_scriptDocUrl = anchorTokenizer.nextToken();
		}
		else
		{
			_scriptDocUrl = documentUrl;
		}
		
		//showMessage(_scriptDocUrl);
		
		

		_treePathArrayBuffer = new StringBuffer();
		// create another thread so unsigned javascript can call	
		Runnable scriptThread = new Runnable()
		{
			public void run()
			{
					// first see if currently selected bookmark matches the given url
	Bookmark currentSelectedBookmark = null;
				if (null != _treeSelectionListener)
				{
					currentSelectedBookmark =
						_treeSelectionListener.getCurrentSelectedBookmark();
				}

				String delimiter = "##*"; //$NON-NLS-1$
				NodeFinder finder = new NodeFinder();
				Bookmark foundBookmark =
					finder.findNode(
						_scriptDocUrl,
						_documentReadRoot,
						_tabPanel.getBookmarkForSelectedView());
				TreeNode[] bookmarkPath = null;
				if (null != foundBookmark)
				{
					if (null != currentSelectedBookmark)
					{
						String foundFilename = foundBookmark.getFileName();
						String currentFilename = currentSelectedBookmark.getFileName();
						if (currentFilename.endsWith(foundFilename))
						{
							// this is what is currently shown in the tree
							bookmarkPath = currentSelectedBookmark.getPath();
						}
						else
						{
							bookmarkPath = foundBookmark.getPath();
						}
					}
					else
					{
						bookmarkPath = foundBookmark.getPath();
					}
				}

				if (null != foundBookmark)
				{
//					showMessage(currentSelectedBookmark.getPresentationName());
//					showMessage(currentSelectedBookmark.getFileName());
//					showMessage(foundBookmark.getPresentationName());
//					showMessage(bookmarkPath[0].toString());
										
					// ignore the root
					for (int i = 1; i < bookmarkPath.length; i++)
					{
						Bookmark bookmark = (Bookmark) bookmarkPath[i];
						_treePathArrayBuffer.append(bookmark.getClosedIconName());
						_treePathArrayBuffer.append(delimiter);
						_treePathArrayBuffer.append(bookmark.getPresentationName());
						_treePathArrayBuffer.append(delimiter);
						_treePathArrayBuffer.append(bookmark.getFileName());
						_treePathArrayBuffer.append(delimiter);
						
						//showMessage(_treePathArrayBuffer.toString());
					}
				}
			}
		};

		try
		{
			SwingUtilities.invokeAndWait(scriptThread);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return (_treePathArrayBuffer.toString());
	}

	/**
	 * The where am I feature.
	 */
	public boolean selectNode(String documentUrl)
	{
		// ignore the no-node empty page
		if (documentUrl.endsWith(RUP_EMPTY_PAGE))
		{
			return (true);
		}

		NodeFinder finder = new NodeFinder();
		Bookmark bookmark =
			finder.findNode(
				documentUrl,
				_documentReadRoot,
				_tabPanel.getBookmarkForSelectedView());
		boolean status = false;
		if (null != bookmark)
		{
			// found it, now select it in the tree
			_tabPanel.setSelectedBookmarkInTree(bookmark);
			_tabPanel.srollToBookmarkInTree(bookmark);
			status = true;
		}
		return (status);
	}

	/**
	 * Create the appropriate process view.
	 */
	protected void setupView()
	{
		try
		{
			// first copy over image file to application directory,
			// for future myrup uses and setup icon manager
			setupImageFile();

			// create the top icon menus
			if (null == _mainIconMenu)
			{
				_mainIconMenu = new RupPresenterIconMenu(this);
			}
			if (null == _topIconMenu)
			{
				_topIconMenu =
					new RupPresenterTopIconMenu(
						this,
						_defaultIconManager,
						_applicationDirectory);
			}

			// create the view
			_tabPanel = null;
			_tabPanel = new ViewerTabPanel();

			// add views for default files (these views are readable only)
			if (null == _defaultFileList)
			{
				//_defaultFileList = ViewerUtilities.getFileList( _documentReadRoot + APPLET_DIRECTORY,
				//	                                            BookmarkLibrary.BOOKMARK_SUFFIX );
				_defaultFileList =
					BookmarkLibrary.readPublishedBookmarkNames(this.getDocumentBase());
			}
			// get myrup views
			Vector localFileList =
				ViewerUtilities.getFileList(
					_applicationDirectory,
					BookmarkLibrary.BOOKMARK_SUFFIX);

			if (_defaultFileList.size() == 0 && localFileList.size() == 0)


			{
				// problem, no views available to show
				_defaultBookmarkList = new Vector();
				showErrorMessage(Translations.getString("PROCESSTOOLS_120")); //$NON-NLS-1$
				this.stop();
				this.destroy();
				System.exit(0);
			}
			if ((_defaultFileList.size() + localFileList.size()) == 1)
			{
				// only one, so remove any previous view states
				this.removeViewState();
			}

			if (null == _defaultBookmarkList)
			{

				_defaultBookmarkList = new Vector();
				// find out what the first bookmark to show is
				String firstBookmarkName = null;
				try
				{
					firstBookmarkName =
						BookmarkLibrary.readDefaultBookmarkName(this.getDocumentBase());
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

				for (int i = 0; i < _defaultFileList.size(); i++)
				{
					String filename = (String) _defaultFileList.elementAt(i);
					if (null == firstBookmarkName
						|| (null != firstBookmarkName
							&& filename.equals(firstBookmarkName)))
					{
						String bookmarkName =
							BookmarkLibrary.convertFileToPresentationName(filename);
						Bookmark bookmark =
							BookmarkLibrary.readBookmark(
								this.getDocumentBase(),
								bookmarkName);



						_defaultBookmarkList.addElement(bookmark);
					}
					else
					{
						Bookmark tempBookmark =
							new Bookmark(
								BookmarkLibrary.convertFileToPresentationName(filename));
						_defaultBookmarkList.addElement(tempBookmark);
					}
				}
			}


			for (int i = 0; i < _defaultBookmarkList.size(); i++)
			{
				Bookmark tt = (Bookmark) _defaultBookmarkList.elementAt(i);
				_tabPanel.addView(
					(Bookmark) _defaultBookmarkList.elementAt(i),
					false,
					new RupPresenterTreeCellRenderer(_defaultIconManager));
			}


			// add view for each available myrup file
			for (int i = 0; i < localFileList.size(); i++)
			{
				String filename = (String) localFileList.elementAt(i);

				String bookmarkName =
					filename.substring(
						0,
						filename.indexOf(BookmarkLibrary.BOOKMARK_SUFFIX));
				Bookmark bookmark =
					BookmarkLibrary.readBookmark(_applicationDirectoryUrl, bookmarkName);




				// check whether the bookmark name exists as a default one too
				if (isDefaultBookmark(bookmark))
				{
					// yes, so change the name and inform the user
					String originalName = bookmark.getPresentationName();
					bookmark.setPresentationName(originalName + Translations.getString("PROCESSTOOLS_69")); //$NON-NLS-1$
					BookmarkLibrary.writeBookmark(
						bookmark,
						bookmark.getPresentationName(),
						_applicationDirectory);
					BookmarkLibrary.deleteBookmark(originalName, _applicationDirectory);
					Object[] messageArguments =
						{ originalName, bookmark.getPresentationName()};
						String message = MessageFormat.format(Translations.getString("PROCESSTOOLS_70"), //$NON-NLS-1$
	messageArguments);
					showMessage(message);
				}

				ViewerPopupMenuAdapter popupAdapter = new ViewerPopupMenuAdapter();
				popupAdapter.setParentComponent(getAppletHandle());
				_tabPanel.addView(
					bookmark,
					true,
					DnDConstants.ACTION_MOVE,
					true,
					DnDConstants.ACTION_MOVE,
					true,
					popupAdapter,
					new RupPresenterTreeCellRenderer(_defaultIconManager));
			}

			// remember that file existence has already been determined,
			// as this just needs to be done once
			_isFileExistSet = true;

			// if extended help is requested, put it in
			if (_isShowXhelp)
			{
				if (null == _xhelpBookmark)
				{
					// only get it the first time
					ContextRupPresenter contextPresenter =
						new ContextRupPresenter(
							_applicationDirectoryUrl,
							_documentReadRoot,
							this.getDocumentBase(),
							APPLET_DIRECTORY);
					_xhelpBookmark = contextPresenter.getContextBookmark();
				}
				if (_xhelpBookmark.getChildCount() == 0)
				{
					// if no results for xhelp, put in node saying so
					Bookmark noHelpBookmark = new Bookmark("No content found"); //$NON-NLS-1$
					noHelpBookmark.setFileName("process/ovu_xhelp.htm"); //$NON-NLS-1$
					noHelpBookmark.setClosedIconName("bookc.gif"); //$NON-NLS-1$
					noHelpBookmark.setOpenIconName("booko.gif"); //$NON-NLS-1$
					_xhelpBookmark.add(noHelpBookmark);
				}
				_tabPanel.addView(
					_xhelpBookmark,
					false,
					new RupPresenterTreeCellRenderer(_defaultIconManager));
			}
		}
		catch (java.security.AccessControlException ae)
		{
			handleAccessException();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			if (_isShowXhelp)
			{
				showErrorMessage(Translations.getString("PROCESSTOOLS_35")); //$NON-NLS-1$
			}
			else
			{
				showErrorMessage(Translations.getString("PROCESSTOOLS_76")); //$NON-NLS-1$
			}
		}
	}

	/**
	 * Adds view to the applet frame.
	 */
	protected void refreshView(String nameToFocus)
	{
		this.getContentPane().removeAll();

		// add icon menus
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.Y_AXIS));
		menuPanel.add(_topIconMenu.getMenuPanel());
		menuPanel.add(_mainIconMenu.getMenuPanel());
		this.getContentPane().add(menuPanel, BorderLayout.NORTH);

		String bookmarkName = null;
		if (null != _tabPanel)
		{
			try
			{
				// read in view state file, if it exists for the bookmark to show
				InputStream inStream =
					new URL(_applicationDirectoryUrl, _viewStateFile).openStream();
				ObjectInputStream ois = new ObjectInputStream(inStream);

				// Attempt to read the data that has been serialized out to disk.
				bookmarkName = (String) ois.readObject();

				// Close the input streams.
				inStream.close();
				ois.close();
				inStream = null;
			}
			catch (Exception e) // no problem, just means file is not there
			{
			}
		}

			ImageIcon editIcon = _defaultIconManager.getIcon(File.separator + APPLET_DIRECTORY + File.separator + "images" + //$NON-NLS-1$
	File.separator + "tab_pencil.gif"); //$NON-NLS-1$
		if (null != bookmarkName && null != _tabPanel.getBookmark(bookmarkName))
		{
			// single tab view
			ImageIcon icon = null;
			if (!isDefaultBookmark(_tabPanel.getBookmark(bookmarkName)))
			{
				icon = editIcon;
			}
			// make sure that single bookmark is loaded
			// only care about default bookmarks because local my rup ones always loaded
			for (int i = 0; i < _defaultBookmarkList.size(); i++)
			{
				Bookmark bookmark = (Bookmark) _defaultBookmarkList.elementAt(i);
				if (bookmarkName.equals(bookmark.getPresentationName()))
				{
					if (bookmark.getChildCount() == 0)
					{
						try
						{
							// this bookmark was not loaded, load it now
							Bookmark loadedBookmark =
								BookmarkLibrary.readBookmark(
									this.getDocumentBase(),
									bookmarkName);
							_tabPanel.removeView(bookmarkName);
							_tabPanel.addView(
								loadedBookmark,
								false,
								new RupPresenterTreeCellRenderer(_defaultIconManager));

							_defaultBookmarkList.remove(bookmark);
							_defaultBookmarkList.add(loadedBookmark);
							break;
						}
						catch (Exception ex)
						{
							ex.printStackTrace();
							Object[] messageArguments = { bookmarkName };
							String message = MessageFormat.format(Translations.getString("PROCESSTOOLS_63"), messageArguments); //$NON-NLS-1$
							showErrorMessage(message);
						}
					}
				}
			}

			this.getContentPane().add(
				_tabPanel.getViewPane(bookmarkName, true, Color.lightGray, icon),
				BorderLayout.CENTER);
		}
		else
		{
			// multiple tab views
			JComponent pane = _tabPanel.getViewPane();
			if (pane instanceof JTabbedPane)
			{
				JTabbedPane tabPane = (JTabbedPane) pane;
				for (int i = 0; i < tabPane.getTabCount(); i++)
				{
					String name = tabPane.getTitleAt(i);
					tabPane.setBackgroundAt(i, Color.lightGray);
					if (null != _xhelpBookmark)
					{
						String xhelpName = _xhelpBookmark.getPresentationName();
						if (!isDefaultBookmark(_tabPanel.getBookmark(name))
							&& !name.equals(xhelpName))
						{
							tabPane.setIconAt(i, editIcon);
						}
					}
					else if (!isDefaultBookmark(_tabPanel.getBookmark(name)))
					{
						tabPane.setIconAt(i, editIcon);
					}
				}
			}
			this.getContentPane().add(pane, BorderLayout.CENTER);
		}

		// set focus
		int indexToSelect = 0;
		if (null != bookmarkName && null != _tabPanel.getBookmark(bookmarkName))
		{
			// this overrides name to focus, because it's only one bookmark
			// to display
			indexToSelect = _tabPanel.getIndexOfBookmark(bookmarkName);
		}
		else if (null != nameToFocus)
		{
			indexToSelect = _tabPanel.getIndexOfBookmark(nameToFocus);
		}
		else
		{
			if (null != _xhelpBookmark)
			{
				indexToSelect =
					_tabPanel.getIndexOfBookmark(_xhelpBookmark.getPresentationName());
			}
			else
			{
				for (int i = 0; i < _defaultBookmarkList.size(); i++)
				{
					Bookmark bookmark = (Bookmark) _defaultBookmarkList.elementAt(i);
					if (bookmark.isCurrent())
					{
						indexToSelect =
							_tabPanel.getIndexOfBookmark(bookmark.getPresentationName());
					}
				}
			}
		}
		_tabPanel.setSelectedIndex(indexToSelect);

		// add tree listeners
		addTreeListeners();
		// add tab change listener
		_tabPanel.addTabChangeListener(this);

		if (null != bookmarkName && null == _tabPanel.getBookmark(bookmarkName))
		{
			// view state was from before, so clear it now for future
			removeViewState();
		}

		this.getContentPane().setBackground(Color.white);
		this.getContentPane().validate();
		this.getContentPane().repaint();
	}

	/**
	 * Add tree listeners.
	 */
	protected void addTreeListeners()
	{
		if (null == _treeSelectionListener)
		{
			_treeSelectionListener = new TreeListener(this);
			_mouseSelectionListener =
				new MouseSelectionListener(this, _treeSelectionListener);
			_keySelectionListener =
				new KeySelectionListener(this, _treeSelectionListener);
		}
		if (null != _tabPanel)
		{
			// add tree selection and mouse listeners
			_tabPanel.addTreeListener(_treeSelectionListener);
			_tabPanel.addMouseSelectionListener(_mouseSelectionListener);
			_tabPanel.addKeySelectionListener(_keySelectionListener);

			// add tree model listener for changes
			ViewerTreeModelListener treeModelListener =
				new ViewerTreeModelListener(_tabPanel, _applicationDirectory);
			_tabPanel.addTreeModelListener(treeModelListener);
		}
	}

	/**
	 * Copies over the image file and sets up the icon manager.
	 */
	protected void setupImageFile()
	{
		try
		{
			// setup icon manager
			if (null == _defaultIconManager)
			{
				// applet images directory
				String appletImagesDir =
					_documentReadRoot
						+ File.separator
						+ APPLET_DIRECTORY
						+ File.separator
						+ "images"
						+ File.separator;
				_defaultIconManager = new IconManager(appletImagesDir, _documentReadRoot);


			}
			// also set zip file
			URL imageUrl = new URL(this.getDocumentBase(), _imageFile);
			_defaultIconManager.setResourceFile(imageUrl);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			showMessage(Translations.getString("PROCESSTOOLS_36")); //$NON-NLS-1$
			String message = Translations.getString("PROCESSTOOLS_37"); //$NON-NLS-1$
		}
	}

	/**
	 * Cleans up view state file and cookie.
	 */
	protected void removeViewState()
	{
		File viewCookie = new File(_applicationDirectory, this.getParameter("viewstatecookiefile")); //$NON-NLS-1$
		if (viewCookie.exists())
		{
			viewCookie.delete();
		}
		File viewStateFile = new File(_applicationDirectory, _viewStateFile);
		if (viewStateFile.exists())
		{
			viewStateFile.delete();
		}
	}

	/**
	 * Simply puts up a dialog with the specified message
	 */
	protected void showMessage(String message)
	{
		JOptionPane.showMessageDialog(null, message, Translations.getString("PROCESSTOOLS_38"), //$NON-NLS-1$
		JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * Simply puts up an error dialog.
	 */
	protected void showErrorMessage(String message)
	{
		JOptionPane.showMessageDialog(null, message, Translations.getString("PROCESSTOOLS_39"), //$NON-NLS-1$
		JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Retrieves the root presentation name from the filename.
	 */
	protected String parsePresentationName(String filename)
	{
		// take the first part of the filename as the presentation name
		StringTokenizer tokenizer = new StringTokenizer(filename, "."); //$NON-NLS-1$
		return ((String) tokenizer.nextElement());
	}

	/**
	* Sets the existences flag for all bookmark topics.
	 */
	protected void setBookmarkExistence(Bookmark bookmark)
	{
		for (int i = 0; i < bookmark.getChildCount(); i++)
		{
			Bookmark childBookmark = (Bookmark) (bookmark.getChildAt(i));
			setBookmarkExistence(childBookmark);
		}
		setFileExists(bookmark);
	}
	/**
	 * Determines whether file exists in the file system.
	 */
	private void setFileExists(Bookmark bookmark)
	{
		String filename = bookmark.getFileName();
		File fileToCheck = null;
		boolean isExist = false;

		if (null != filename)
		{
			try
			{
				String uncPageName = ViewerUtilities.convertFilenameToUNC(filename);
				URL fullUrl = null;
				if (bookmark.isFromContentLibrary())
				{
					// we know this came from the RUP
					fullUrl = new URL(_documentReadRoot + filename);
					if (this.getDocumentBase().toExternalForm().startsWith("http")) //$NON-NLS-1$
					{
						fullUrl.openStream();
						// if this passes, then it's there	
						isExist = true;
					}
					else
					{
						fileToCheck = new File(fullUrl.getFile());
						isExist = fileToCheck.exists();
					}
				}
				else if (uncPageName.startsWith("../")) //$NON-NLS-1$
				{
					// we know this came from the RUP					
					fullUrl = new URL(_documentReadRoot + uncPageName.substring(3));
					if (this.getDocumentBase().toExternalForm().startsWith("http")) //$NON-NLS-1$
					{
						fullUrl.openStream();
						// if this passes, then it's there	
						isExist = true;
					}
					else
					{
						fileToCheck = new File(fullUrl.getFile());
						isExist = fileToCheck.exists();
					}
				}
				else if (uncPageName.startsWith("http")) //$NON-NLS-1$
				{
					// this came from a web server somewhere
					fullUrl = new URL(uncPageName);
					fullUrl.openStream(); // if this passes, then it's there
					isExist = true;
				}
				else
				{
					// this came from a file somewhere local
					fullUrl = new URL("file", "", uncPageName); //$NON-NLS-1$ //$NON-NLS-2$
					fileToCheck = new File(fullUrl.getFile());
					isExist = fileToCheck.exists();
				}

				fullUrl = null;
				fileToCheck = null;
			}
			catch (Exception ex)
			{
			}
		}

		bookmark.setExist(isExist);
	}

	/**
	 * Determines whether bookmark is a default one.
	 */
	private boolean isDefaultBookmark(Bookmark bookmark)
	{
		String selectedFilename =
			bookmark.getPresentationName() + BookmarkLibrary.BOOKMARK_SUFFIX;
		// adjust for space and space replacment
		String finalFilename =
			selectedFilename.replace(
				BookmarkLibrary.SPACE_STRING.charAt(0),
				BookmarkLibrary.SPACE_REPLACEMENT.charAt(0));
		if (_defaultFileList.contains(finalFilename))
		{
			return (true);
		}
		return (false);
	}

	/**
	 * Determines whether bookmark is an extended help one.
	 */
	private boolean isXhelpBookmark(Bookmark bookmark)
	{
		if (_xhelpBookmark != null
			&& bookmark.getPresentationName().equals(_xhelpBookmark.getPresentationName()))
		{
			return (true);
		}
		return (false);
	}

	/**
	 * Determines whether bookmark with existing name already exists.
	 */
	private boolean isNameInDefaultList(String name)
	{
		boolean isExist = isDefaultBookmark(new Bookmark(name));

		return (isExist);
	}

	/**
	 * Handles access control exceptions.
	 */
	private void handleAccessException()
	{
		JOptionPane.showMessageDialog(null, Translations.getString("PROCESSTOOLS_77"), Translations.getString("PROCESSTOOLS_44"), //$NON-NLS-1$
		JOptionPane.INFORMATION_MESSAGE);
		_isAccessExceptionCaught = true;
	}

	/**
	 * Writes xhelp file to any, any.
	 */
	private void writeXhelpFile()
	{
		TopicFile topics = new TopicFile();
		topics.setTool("Any"); //$NON-NLS-1$
		topics.addTopic("Any"); //$NON-NLS-1$
		TopicFileParser.writeTopicFile(_applicationDirectory, "xhelp.context", //$NON-NLS-1$
		topics);
	}

	/**
	 * Tree listener.  Puts the document out when node is selected.
	 */
	protected class TreeListener implements TreeSelectionListener
	{
		private RupPresenterApplet _applet = null;
		private Bookmark _currentSelectedBookmark = null;

		public TreeListener(RupPresenterApplet applet)
		{
			_applet = applet;
		}

		public void valueChanged(TreeSelectionEvent e)
		{
			if (e.isAddedPath())
			{
				// a selection was made
				_currentSelectedBookmark = (Bookmark) e.getPath().getLastPathComponent();
			}
		}

		public Bookmark getCurrentSelectedBookmark()
		{
			return (_currentSelectedBookmark);
		}

		public void showBookmark(Bookmark currentBookmark)
		{
			_currentSelectedBookmark = currentBookmark;
			_applet.showPage(_currentSelectedBookmark);
		}

		public void showCurrentBookmark()
		{
			_applet.showPage(_currentSelectedBookmark);
		}
	}

	/**
	 * Mouse listener for tree selection. Puts the document out
	 * when node is selected.
	 */
	protected class MouseSelectionListener extends MouseAdapter
	{
		private RupPresenterApplet _applet = null;
		private TreeListener _selectionListener = null;

		public MouseSelectionListener(
			RupPresenterApplet applet,
			TreeListener selectionListener)
		{
			_applet = applet;
			_selectionListener = selectionListener;
		}

		public void mousePressed(MouseEvent e)
		{
			// want to ignore double-clicks which are meant to open
			// and close folders
			if (e.getClickCount() != 1)
			{
				return;
			}

			// select if and only if it is a single node that is clicked;
			// click can be on both left and right mouse buttons
			if (SwingUtilities.isLeftMouseButton(e)
				|| SwingUtilities.isRightMouseButton(e))
			{
				// ignore CTRL and SHIFT keys
				if (!e.isControlDown() && !e.isShiftDown())
				{
					TreePath selPath =
						((JTree) e.getComponent()).getPathForLocation(e.getX(), e.getY());
					if (null != selPath)
					{
						_selectionListener.showBookmark(
							(Bookmark) selPath.getLastPathComponent());
					}
				}
			}
		}
	}

	/**
	 * Key listener for tree selection. Coordinates with the
	 * mouse listener to puts the document out when node is selected.
	 */
	protected class KeySelectionListener extends KeyAdapter
	{
		private RupPresenterApplet _applet = null;
		private TreeListener _selectionListener = null;

		public KeySelectionListener(
			RupPresenterApplet applet,
			TreeListener selectionListener)
		{
			_applet = applet;
			_selectionListener = selectionListener;
		}

		public void keyReleased(KeyEvent e)
		{
			int keyCode = e.getKeyCode();
			if (keyCode == KeyEvent.VK_UP
				|| keyCode == KeyEvent.VK_DOWN
				|| keyCode == KeyEvent.VK_RIGHT
				|| keyCode == KeyEvent.VK_LEFT)
			{
				if (!e.isControlDown() && !e.isShiftDown())
				{
					// show the currently selected bookmark	        		
					_selectionListener.showCurrentBookmark();
				}
			}

		}

	}

	/**
	 * Protected class for save as panel.
	 */
	protected class SaveAsPanel extends JPanel
	{
		private LimitedTextField _name = null;

		public SaveAsPanel()
		{
			JLabel nameLabel = new JLabel(Translations.getString("PROCESSTOOLS_17")); //$NON-NLS-1$
			// limit name to 32 characters
				_name = new LimitedTextField(Translations.getString("PROCESSTOOLS_19"), //$NON-NLS-1$
	32);

			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.add(nameLabel);
			this.add(_name);
			_name.requestFocusInWindow();
		}

		public String getName()
		{
			return (_name.getText());
		}
	}
	
	// resize the applet frames (Sun bug_id=4936777)
	public void setSize(int w, int h)
	{        
		super.setSize(w, h);
		//resize  the containers
		java.awt.Container c = this;
		while (!(c instanceof java.awt.Frame)) 
		{
			if (c!=null) 
				c.setSize(w, h);
			
			c = c.getParent();
			
			if (c==null) 
				break;
		}
		java.awt.Frame f = (java.awt.Frame)c;
		f.setSize(w, h);
		f.invalidate();
		resize(w, h);
		invalidate();
	}
	
	public void selectTab(int bookmarkIndex)
	{		
		_tabPanel.selectTab(bookmarkIndex);
	}

}