package com.rational.rpw.processtools;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import java.util.StringTokenizer;

import netscape.javascript.JSObject;

import com.rational.rpw.processview.BookmarkLibrary;
import com.rational.rpw.processview.IconManager;
import com.rational.rpw.processviewer.ViewerUtilities;
import com.rational.rpw.environment.FontFactory;

/**
 * This class provides the top menu icon strip for view actions. 
 * Actions here include where am i and view tree sets. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class RupPresenterTopIconMenu
	implements ActionListener
{	
	private RupPresenterApplet _applet = null;
	private IconManager _iconManager = null;
	private String _applicationDirectory = null;
	
	private File _viewStateCookieFile = null;
	private JButton _whereAmIButton = null;
	private JButton _viewStateButton = null;
	
	private JPanel _buttonPanel = null;		
	
	/**
	 * Default constructor.
	 */
	public RupPresenterTopIconMenu( RupPresenterApplet applet,
	                                 IconManager iconManager,
	                                 String applicationDirectory )
	{
		_applet = applet;
		_iconManager = iconManager;
		_applicationDirectory = applicationDirectory;
		
		initialize();
	}

	/**
	 * Retrieve the icon panel.
	 */
	public JPanel getMenuPanel()
	{		
		initialize();
		return( _buttonPanel );
	}
	
	/**
	 * Creates the icon buttons.
	 */
	private void initialize()
	{
		// initializes the view state
		_viewStateCookieFile = new File( _applicationDirectory,
		                                 _applet.getParameter( "viewstatecookiefile" ) ); //$NON-NLS-1$
		
		// insets, colors, fonts
		java.awt.Insets insets = new java.awt.Insets( 0, 0, 0, 0 );
		Color background = new Color( 0, 102,153 );
		Font font = FontFactory.getCommonFont( Font.PLAIN, 10 );
		
		// applet image directory
		String imageDir = BookmarkLibrary.LIBRARY_DIRECTORY +
                          ViewerUtilities.UNC_SEPARATOR +
       		              "images" + //$NON-NLS-1$
       		              ViewerUtilities.UNC_SEPARATOR;
       		              
		// where am I
		if( null == _whereAmIButton )
		{
       		ImageIcon whereAmIIcon = _iconManager.getIcon( imageDir  + "ni_where.gif" ); //$NON-NLS-1$
			_whereAmIButton = new JButton( Translations.getString("PROCESSTOOLS_83"), whereAmIIcon );			 //$NON-NLS-1$
			_whereAmIButton.setFont( font );						
			_whereAmIButton.setToolTipText( Translations.getString("PROCESSTOOLS_84") ); //$NON-NLS-1$
			_whereAmIButton.setMargin( insets );	 
			_whereAmIButton.setBackground( background );
			_whereAmIButton.setForeground( Color.white );
			_whereAmIButton.setBorder( BorderFactory.createLineBorder( Color.white ) );			
			_whereAmIButton.addActionListener( this );
		}
		
		// view state
 		ImageIcon viewStateIcon = null;
 		if( _viewStateCookieFile.exists() )
 		{
 			viewStateIcon = _iconManager.getIcon( imageDir + "action_unhide.gif" ); //$NON-NLS-1$
 		}
 		else
 		{
 			viewStateIcon = _iconManager.getIcon( imageDir + "action_hide.gif" ); //$NON-NLS-1$
 		}
		if( null != _viewStateButton )
		{
			_viewStateButton.removeActionListener( this );			
			_viewStateButton = null;
		}
		_viewStateButton = new JButton( Translations.getString("PROCESSTOOLS_85"), viewStateIcon ); //$NON-NLS-1$
		_viewStateButton.setFont( font );
		_viewStateButton.setToolTipText( Translations.getString("PROCESSTOOLS_86") ); //$NON-NLS-1$
		_viewStateButton.setMargin( insets );
		_viewStateButton.setBackground( background );
		_viewStateButton.setForeground( Color.white );
		_viewStateButton.setBorder( BorderFactory.createLineBorder( Color.white ) );
		_viewStateButton.addActionListener( this );
	
		// add all to button panel
		_buttonPanel = new JPanel();
		_buttonPanel.setLayout( new GridLayout( 1, 0 ) );
		_buttonPanel.setBackground( Color.white );
		_buttonPanel.add( _whereAmIButton );
		_buttonPanel.add( _viewStateButton );
	}
	
	/**
 	 * Handle the where am i button feature.
 	 */  
	private void whereAmI() 
	{
		try
		{
			JSObject theJSObject = (JSObject)JSObject.getWindow( (Applet)_applet );
			String theCommandString = "parent." + _applet.getParameter( "target" ) + ".location"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	
			//execute the javascript command to get the location of the target frame
			Object theReturnValue = theJSObject.eval( theCommandString ); 
		
			//check to see if the return type is a string(Netscape implementation) or
			//a JSObject(IE implementation)
			String theTarget;
			if( theReturnValue instanceof String ) 
			{
				theTarget = (String)theReturnValue;
			}
			else 
			{
				JSObject theTmpObject = (JSObject)theReturnValue;
				theTarget = (String)theTmpObject.getMember( "href" ); //$NON-NLS-1$
			}	
	
			//format the path if it's a location file access
			if( theTarget.indexOf( "file:///") == -1 && //$NON-NLS-1$
				theTarget.indexOf( "http:") == -1 )  //$NON-NLS-1$
			{
				if( theTarget.indexOf( "file://" ) == -1 ) //$NON-NLS-1$
				{
					theTarget = theTarget.substring( 0, 5) + "//" + theTarget.substring( 5 ); //$NON-NLS-1$
				}
				else 
				{
					theTarget = theTarget.substring( 0, 5) + "/" + theTarget.substring( 5 ); //$NON-NLS-1$
				}
			}
			
			// strip out the HTML anchor tag
			StringTokenizer anchorTokenizer = new StringTokenizer( theTarget, "#" );
			String finalTarget = null;
			if( anchorTokenizer.hasMoreTokens() )
			{
				finalTarget = anchorTokenizer.nextToken();	
			}
			else
			{
				finalTarget = theTarget;	
			}
			
			// select the node on the tree
			boolean status = _applet.selectNode( finalTarget );
			if( !status )
			{
				JOptionPane.showMessageDialog( null, 
				                               Translations.getString("PROCESSTOOLS_87"), //$NON-NLS-1$
				                               Translations.getString("PROCESSTOOLS_88"), //$NON-NLS-1$
				                               JOptionPane.INFORMATION_MESSAGE );
			}
		}		
		catch( Throwable t )
		{
			t.printStackTrace();	
		}
	}
	
	/**
	 * Action listener.
	 */
	public void actionPerformed( ActionEvent e )
	{
		if( e.getSource() == _whereAmIButton )
		{
			whereAmI();	
		}	
		else if( e.getSource() == _viewStateButton )
		{
			if( _viewStateCookieFile.exists() )
			{
				_viewStateCookieFile.delete();
				_applet.showAll();
			}
			else
			{
				try
				{
					_viewStateCookieFile.createNewFile();
					_applet.hideTrees();
				}
				catch( Exception ex )
				{
				}
			}
		}
	}
			
	
}
