package com.rational.rpw.processtools;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.net.URL;
import java.text.MessageFormat;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLEditorKit;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.search.Hits;

import com.rational.rpw.processview.IconManager;
import com.rational.rpw.processviewer.ViewerUtilities;
import com.rational.rpw.search.SearchRup;
import com.rational.rpw.search.indexutil.IndexLoader;

/**
 * This class is the applet for the search engine. It is the
 * manager of the UI and knows how to interact with other
 * search engine related classes to obtain search results.
 * 
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class SearchApplet extends JApplet
{	
	public static final int HITS_PER_PAGE = 10;
	public static final double LABEL_WIDTH = 0.0;
	public static final double DATA_WIDTH = 1;
	
	public static final String AND_OPERATOR = " AND "; //$NON-NLS-1$
	public static final String OR_OPERATOR = " OR "; //$NON-NLS-1$
	public static final String NOT_OPERATOR = " NOT "; //$NON-NLS-1$
	
	public static final String RUP_SECTION = "Rational Unified Process"; //$NON-NLS-1$
	public static final String RDN_SECTION = "Rational Developer Network"; //$NON-NLS-1$

	// UI features
	protected JPanel _advancedPane;
	protected String _productName;
	
	protected JComboBox _section;
	protected JComboBox _perPageResults;

	protected JTextField _allWords;
	protected JTextField _exactPhrase;
	protected JTextField _anyWords;
	protected JTextField _notWords;
	protected JEditorPane _resultsPane;
	protected JPanel _buttonsPanel;
	protected JButton _nextButton;
	protected JButton _previousButton;
	protected Hits _hits;
	protected int _numHits;
	protected int _currentPage;
	protected int _hitsPerPage;
	protected String _currentSearchString;
	protected String _indexPath;
	protected boolean _enabled = false;
	
	/**
	 * Constructor for SearchApplet
	 */
	public SearchApplet()
	{
		super();
	}
	
	/**
	 * Applet init method.
	 */
	public void init()
	{	
		super.init();
		try
		{
	        UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
		}
		catch( Exception e1 )
		{
			e1.printStackTrace();
		}

		_hitsPerPage = HITS_PER_PAGE;
		
		createAdvancedSearchPane();		
	}
	
	
	/** 
	 * Applet start method.
	 */
	public void start() 
	{
		super.start();

		String userRupFolder = System.getProperty( "user.home" ) + //$NON-NLS-1$
			                   File.separator +"rup";  //$NON-NLS-1$
		String indexFolder =   userRupFolder + File.separator + "index"; //$NON-NLS-1$
		_indexPath = indexFolder;
		String indexFile = indexFolder  + File.separator + "index.jar"; //$NON-NLS-1$
					                        
		// download the jar file		
		String dBase = getDocumentBase().toString();
		// check the version of the index
		if (versionOutOfDate(indexFolder, dBase))
		{
			JDialog tempFrame = new JDialog();
			IndexLoader loader = new IndexLoader(
				dBase,
				indexFolder,
				indexFile);
			_productName = loader.go();
			if( null != _productName )
			{
				// successful loading, just close
				loader.setVisible( false );
				loader = null;
			}
		}
		
		// add the search pane
		getContentPane().add( _advancedPane );
		getContentPane().validate();
		_allWords.requestFocus();		
		_enabled = true;
		if (_productName == null)
		{
			_advancedPane.setEnabled(false);
			_resultsPane.setText(Translations.getString("PROCESSTOOLS_90") ); //$NON-NLS-1$
			_enabled=false;
		}
	}

	/**
	 * Creates the UI for the advanced search pane.
	 */
	protected void createAdvancedSearchPane()
	{
		_advancedPane = new JPanel();
		GridBagLayout gLayout = new GridBagLayout();
		GridBagConstraints gConstraints = new GridBagConstraints();
		GridBagLayout outerLayout = new GridBagLayout();
		_advancedPane.setLayout(outerLayout);

		gConstraints.fill = GridBagConstraints.HORIZONTAL;
		gConstraints.insets = new Insets(2, 8, 2, 8);
		gConstraints.weightx = 1;
		gConstraints.weighty = 0;

        String documentBase =  this.getDocumentBase().toString();
        int last = documentBase.lastIndexOf( "/" ); //$NON-NLS-1$
        String imageReadRoot = documentBase.substring( 0,last ) + "/images/"; //$NON-NLS-1$
		ImageIcon searchIcon = null;
		IconManager iconManager = new IconManager( imageReadRoot );
		searchIcon = iconManager.getIcon( "ni_search.gif" ); //$NON-NLS-1$
		JLabel topLabel = new JLabel("<html><body><font size=+1>" + Translations.getString("PROCESSTOOLS_92") + "</font></body></html>", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		                             searchIcon,
		                             SwingConstants.LEFT );
		outerLayout.setConstraints(topLabel, gConstraints);
		_advancedPane.add(topLabel);
		
		JPanel interiorPane = new JPanel();
		interiorPane.setBackground(Color.white);
		interiorPane.setLayout(gLayout);
		
		gConstraints.weightx = 0;
		gConstraints.weighty = 0;
		gConstraints.gridx = 0;
		gConstraints.gridy = 0;
		gConstraints.fill = GridBagConstraints.HORIZONTAL;
		
		JLabel searchLabel = new JLabel("<html><body><b>" + Translations.getString("PROCESSTOOLS_93") + "</b></body></html>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		JLabel label1 = new JLabel(Translations.getString("PROCESSTOOLS_94")); //$NON-NLS-1$
		JLabel label2 = new JLabel(Translations.getString("PROCESSTOOLS_95")); //$NON-NLS-1$
		JLabel label3 = new JLabel(Translations.getString("PROCESSTOOLS_96")); //$NON-NLS-1$
		JLabel label4 = new JLabel(Translations.getString("PROCESSTOOLS_97")); //$NON-NLS-1$
		JLabel label5 = new JLabel(Translations.getString("PROCESSTOOLS_98")); //$NON-NLS-1$
		JLabel label6 = new JLabel(Translations.getString("PROCESSTOOLS_99")); //$NON-NLS-1$
		
		_allWords = new JTextField();
		_exactPhrase = new JTextField();
		_anyWords = new JTextField();
		_notWords = new JTextField();
		
		// default search button
		JButton searchButton = new JButton( Translations.getString("PROCESSTOOLS_100"), searchIcon ); //$NON-NLS-1$
		getRootPane().setDefaultButton( searchButton );
		searchButton.setBorder( BorderFactory.createEtchedBorder() );
		AdvancedSearchButtonListener asblisten = new AdvancedSearchButtonListener(searchButton);
		
		Vector perPageChoices = new Vector();
		perPageChoices.add( "10 " + Translations.getString("PROCESSTOOLS_101") ); //$NON-NLS-1$ //$NON-NLS-2$
		perPageChoices.add( "20 " + Translations.getString("PROCESSTOOLS_101") ); //$NON-NLS-1$ //$NON-NLS-2$
		perPageChoices.add( "30 " + Translations.getString("PROCESSTOOLS_101") ); //$NON-NLS-1$ //$NON-NLS-2$
		perPageChoices.add( "40 " + Translations.getString("PROCESSTOOLS_101") ); //$NON-NLS-1$ //$NON-NLS-2$
		perPageChoices.add( "50 " + Translations.getString("PROCESSTOOLS_101") ); //$NON-NLS-1$ //$NON-NLS-2$
		_perPageResults = new JComboBox( perPageChoices );

		Vector choices = new Vector();
		choices.add(RUP_SECTION);
		choices.add(RDN_SECTION);		
		_section = new JComboBox(choices);
		_section.addActionListener(new SectionChangeListener());

		gConstraints.weightx = 1;
		gConstraints.gridwidth = 4;
		gLayout.setConstraints(searchLabel, gConstraints);		
		interiorPane.add(searchLabel);
		
		gConstraints.weightx = LABEL_WIDTH;
		gConstraints.weighty = 0.5;

		gConstraints.gridwidth = 1;
		gConstraints.gridy = 1;
		gLayout.setConstraints(label2, gConstraints);
		interiorPane.add(label2);
		
		gConstraints.weightx = DATA_WIDTH;
		gConstraints.gridx = 1;
		gLayout.setConstraints(_allWords, gConstraints);
		interiorPane.add(_anyWords);

		gConstraints.weightx = LABEL_WIDTH;
		gConstraints.gridy = 2;
		gConstraints.gridx = 0;
		gLayout.setConstraints(label1, gConstraints);
		interiorPane.add(label1);
		
		gConstraints.weightx = DATA_WIDTH;
		gConstraints.gridx = 1;
		gLayout.setConstraints(_anyWords, gConstraints);
		interiorPane.add(_exactPhrase);

		gConstraints.weightx = LABEL_WIDTH;
		gConstraints.gridy = 3;
		gConstraints.gridx = 0;
		gLayout.setConstraints(label3, gConstraints);
		interiorPane.add(label3);

		gConstraints.weightx = DATA_WIDTH;
		gConstraints.gridx = 1;
		gLayout.setConstraints(_exactPhrase, gConstraints);
		interiorPane.add(_allWords);

		gConstraints.weightx = LABEL_WIDTH;
		gConstraints.gridx = 0;
		gConstraints.gridy = 4;
		gLayout.setConstraints(label4, gConstraints);
		interiorPane.add(label4);

		gConstraints.weightx = DATA_WIDTH;
		gConstraints.gridx = 1;
		gLayout.setConstraints(_notWords, gConstraints);
		interiorPane.add(_notWords);
		
		gConstraints.weightx = LABEL_WIDTH;
		gConstraints.gridx = 3;
		gConstraints.gridy = 1;
		gLayout.setConstraints(label5, gConstraints);
		interiorPane.add(label5);
		
		gConstraints.gridy = 2;
		gLayout.setConstraints(label6, gConstraints);
		interiorPane.add(label6);
		
		gConstraints.weightx = DATA_WIDTH /6;
		gConstraints.gridx = 4;
		gConstraints.gridy = 2;
		gLayout.setConstraints(_perPageResults, gConstraints);
		interiorPane.add(_perPageResults);
		
		gConstraints.gridy = 1;
		gLayout.setConstraints(_section, gConstraints);
		interiorPane.add(_section);
		_section.setEnabled(true);
			
		// a link to the search help page
		JEditorPane helpPane = new JEditorPane();		
		helpPane.setEditable( false );
		helpPane.setEditorKit(new HTMLEditorKit());
		helpPane.setBackground( Color.white );
		String siteBase = getDocumentBase().toString();
		siteBase = siteBase.substring(0, siteBase.indexOf("/applet/search.htm")); //$NON-NLS-1$
		String helpPage = siteBase + "/applet/help.htm"; //$NON-NLS-1$
		helpPane.setText( "<html><a href=\"" + helpPage + "\">" + Translations.getString("PROCESSTOOLS_106") + "</a></html>" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		LinkListener helpLink = new LinkListener();
		helpPane.addHyperlinkListener(helpLink);
		
		// add search button and tips on one panel
		JPanel searchPanel = new JPanel();
		searchPanel.setBackground( Color.white );
		searchPanel.setLayout( new BoxLayout( searchPanel, BoxLayout.X_AXIS ) );
		searchPanel.add( searchButton );
		searchPanel.add( Box.createRigidArea( new Dimension( 8, 0 ) ) );
		searchPanel.add( helpPane );
		
		gConstraints.gridy = 4;
		gConstraints.gridx = 3;
		gConstraints.fill = GridBagConstraints.NONE;
		gConstraints.gridwidth = 2;
		gConstraints.anchor = GridBagConstraints.WEST;
		gLayout.setConstraints(searchPanel, gConstraints);
		interiorPane.add(searchPanel);		
		
		interiorPane.setBorder( BorderFactory.createEtchedBorder() );
		
		gConstraints.anchor = GridBagConstraints.CENTER;
		gConstraints.weightx = 1;
		gConstraints.weighty = 0.02;
		gConstraints.gridx = 0;
		gConstraints.gridy = 1;
		gConstraints.ipady = 3;

		gConstraints.fill = GridBagConstraints.BOTH;
		outerLayout.setConstraints(interiorPane, gConstraints);
		_advancedPane.add(interiorPane);
		
		// create the _buttonsPanel, and populate it with our labels and buttons.
		_buttonsPanel = new JPanel();
		_buttonsPanel.setLayout(new BoxLayout(_buttonsPanel, BoxLayout.X_AXIS));		

		_previousButton = new JButton(" " + Translations.getString("PROCESSTOOLS_107") + " " + _hitsPerPage); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		_buttonsPanel.add(_previousButton);
		_buttonsPanel.add(Box.createRigidArea(new Dimension(3,1)));
		_nextButton = new JButton(" " + Translations.getString("PROCESSTOOLS_108") + " " + _hitsPerPage); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		_buttonsPanel.add(_nextButton);
		_previousButton.setEnabled(false);
		_nextButton.setEnabled(false);
		_nextButton.setVisible(false);
		_previousButton.setVisible(false);
		_nextButton.setBorder( BorderFactory.createEtchedBorder() );
		_previousButton.setBorder( BorderFactory.createEtchedBorder() );		

		NextButtonListener nlisten = new NextButtonListener(_nextButton);
		
		PrevButtonListener plisten = new PrevButtonListener(_previousButton);
		
		gConstraints.gridy = 2;
		gConstraints.weighty = 0;
		//gConstraints.weightx = 0.2;
		outerLayout.setConstraints(_buttonsPanel, gConstraints);
		_advancedPane.add(_buttonsPanel);
		
		// editor pane used to display results
		_resultsPane = new JEditorPane();
		_resultsPane.setEditable(false);
		//_resultsPane.setEditorKit(new HTMLEditorKit());
		String encoding = System.getProperty( "file.encoding" );
		if( encoding.equals( "MS932" ) )
		{
			// special provision for Japanese handling
			encoding = "Shift_jis";
		}
		//_resultsPane.setContentType( "text/html; charset=" + encoding );
		_resultsPane.setContentType( "text/html; charset=utf-8" );
		
		LinkListener listenToLinks = new LinkListener();
		_resultsPane.addHyperlinkListener(listenToLinks);
		JScrollPane resultsScrollPane = new JScrollPane(_resultsPane);
		gConstraints.gridy = 3;
		gConstraints.weighty = 1;
		outerLayout.setConstraints( resultsScrollPane, gConstraints );
		_advancedPane.add(resultsScrollPane);
	}
	
	/**
	 * Formats the output string.
	 */
	protected String getOutputString(int numHits, int pageNum)
	{
		String result = null;
		int init = pageNum * _hitsPerPage;
		int max = Math.min(numHits, init + _hitsPerPage);
		
		/*StringBuffer results = new StringBuffer("<body><font face=\"Arial Unicode MS\">" + _numHits + " " + Translations.getString("PROCESSTOOLS_109")); //$NON-NLS-1$ //$NON-NLS-2$
		results.append(" " + Translations.getString("PROCESSTOOLS_110") + "<b>"); //$NON-NLS-1$ //$NON-NLS-2$
		results.append(_currentSearchString);
		results.append("</b>"); //$NON-NLS-1$*/
		
		StringBuffer results = new StringBuffer("<body><font face=\"Arial Unicode MS\">" );
		Object[] messageArguments = { new Integer(_numHits), _currentSearchString };
		String message = MessageFormat.format( Translations.getString("PROCESSTOOLS_109"),
		                                       messageArguments );	
		results.append(message);
		
		if (max > 0)
		{
			/*results.append(". " + Translations.getString("PROCESSTOOLS_111") + (init + 1) + "-"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			results.append(max);
			results.append(", " + Translations.getString("PROCESSTOOLS_112")); //$NON-NLS-1$ //$NON-NLS-2$*/
			Integer initInt = new Integer( init + 1 );
			Integer maxInt = new Integer( max );
			Object[] moreArguments = { initInt, maxInt };
			String moreMessage = MessageFormat.format( Translations.getString("PROCESSTOOLS_110"),
			                                           moreArguments );
			results.append( moreMessage );
		}
		results.append(".<hr><br>\n<font face=\"Arial Unicode MS\">"); //$NON-NLS-1$
		String siteBase = getDocumentBase().toString();
		siteBase = siteBase.substring(0, siteBase.indexOf("/applet/search.htm")); //$NON-NLS-1$
		
		try
		{
			for (int i=init; i<max; i++)
			{
				Document doc = _hits.doc(i);
				Field url = doc.getField("url"); //$NON-NLS-1$
				Field title = doc.getField("title"); //$NON-NLS-1$
				Field summary = doc.getField("summary"); //$NON-NLS-1$
				String test = url.stringValue();
				test = siteBase + test.substring(test.indexOf(_productName) + _productName.length());
				
				// convert to unc
				test = ViewerUtilities.convertFilenameToUNC( test );
				
				results.append("<b><a href=\"" + test + "\">"  //$NON-NLS-1$ //$NON-NLS-2$
					+ title.stringValue()
					+ "</a></b><br>\n"+ summary.stringValue()); //$NON-NLS-1$
				results.append("<p>\n"); //$NON-NLS-1$
			}
		}
		catch (java.io.IOException e1) {}
		results.append("</font></>"); //$NON-NLS-1$
		result= results.toString();
		
		return result;
	}
	
	/**
	 * Updates the results panel with the search output.
	 */
	protected void updateOutput(boolean safe)
	{
		if (safe)
		{
			String output = getOutputString(_numHits, _currentPage);
			
			_resultsPane.setText( output );
			_resultsPane.setCaretPosition(0);
					
			boolean nextEnabled = _numHits > (_currentPage + 1) * _hitsPerPage ;
			_nextButton.setVisible( true );
			_nextButton.setEnabled(nextEnabled);
			
			boolean prevEnabled = _currentPage > 0;
			_previousButton.setVisible( true );
			_previousButton.setEnabled(prevEnabled);
		}
		else
		{
			_resultsPane.setText(Translations.getString("PROCESSTOOLS_113") + 
			                     "<br><br>" + //$NON-NLS-1$ //$NON-NLS-2$
			                     Translations.getString("PROCESSTOOLS_114") + 
			                     _currentSearchString ); //$NON-NLS-1$
			_previousButton.setVisible(false);
			_nextButton.setVisible(false);
			_hits = null;
		}
	}
	
	/**
	 * Updates the hits per page display.
	 */
	protected void updateHitsPerPage()
	{
		try
		{
			String selectedPerPageCount = _perPageResults.getSelectedItem().toString();
			String perPageCount = selectedPerPageCount.substring( 0, selectedPerPageCount.indexOf( " " ) ); //$NON-NLS-1$
			_hitsPerPage = Integer.parseInt( perPageCount );
			if (_hitsPerPage <1)
			{
				_hitsPerPage = HITS_PER_PAGE;
			}
		}
		catch (Exception e1)
		{
			_hitsPerPage = HITS_PER_PAGE;
		}
		_nextButton.setText(" " + Translations.getString("PROCESSTOOLS_115") + " " + _hitsPerPage); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		_previousButton.setText(" " + Translations.getString("PROCESSTOOLS_116") + " " + _hitsPerPage); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	/**
	 * Runs the query.
	 */
	protected boolean runQuery(String searchIn)
	{
		boolean result = false;
		try
		{
			_hits = SearchRup.searchIndex( _indexPath, 
										   _currentSearchString, 
				                           searchIn);
			
			_numHits = _hits.length();
			result = true;
		}
		catch (Exception e1)
		{
			e1.printStackTrace();
			_numHits = 0;
			_hits = null;
		}
		_currentPage = 0;
		return result;
	}		
	
	/**
	 * Performs the advanced search.
	 */
	protected void doAdvancedSearch()
	{
		updateHitsPerPage();
			
		if (RDN_SECTION.equals(_section.getSelectedItem().toString()))
		{
			StringBuffer searchText = new StringBuffer();
			String input = _allWords.getText();
			StringTokenizer tokens = new StringTokenizer(input, " "); //$NON-NLS-1$
			searchText.append("http://www-106.ibm.com/search/searchResults.jsp?searchType=1&searchSite=dW&searchScope=rdd&query="); //$NON-NLS-1$
			searchText.append(tokens.nextToken());
			while (tokens.hasMoreElements())
			{
				String toAdd = tokens.nextToken();
				searchText.append("+" + toAdd); //$NON-NLS-1$
			}
			
			searchText.append("&USE_CASE=searchresults&JSP_PAGE_NAME=%2Fsearch%2Fjsp%2FSearch.jsp&JSP_NEXT_PAGE_NAME=%2Fsearch%2Fjsp%2FSearch.jsp&searchLocation=catapulse&GlobalSearch=true&NAV_ID=SEARCH_RESULTS"); //$NON-NLS-1$
			try
			{
				URL url = new URL(searchText.toString());
				getAppletContext().showDocument(url, "RUP_RDN_SEARCH"); //$NON-NLS-1$
			}
			catch (Exception e1) {e1.printStackTrace();}

			return;
		}

		String ANDString = _allWords.getText();
		StringBuffer queryStringBuffer = new StringBuffer();

		if (ANDString.length() > 0)
		{
			queryStringBuffer.append(formatSimpleSearchString(ANDString, AND_OPERATOR, false));				
		}
		
		String PHRASEString = _exactPhrase.getText();
		if (PHRASEString.length() > 0)
		{
			if (PHRASEString.indexOf("\"") == -1) //$NON-NLS-1$
			{
				PHRASEString = "\"" + PHRASEString + "\""; //$NON-NLS-1$ //$NON-NLS-2$
			}
			if (queryStringBuffer.length() > 0)
			{
				queryStringBuffer.append(AND_OPERATOR);
			}
			queryStringBuffer.append(formatSimpleSearchString(PHRASEString, AND_OPERATOR, false));
		}
	
		String ORString = _anyWords.getText();
		if (ORString.length() > 0)
		{
			if (queryStringBuffer.length() > 0)
			{
				queryStringBuffer.append(OR_OPERATOR);
			}
			queryStringBuffer.append(formatSimpleSearchString(ORString, OR_OPERATOR, false));
		}
		
		String NOTString = _notWords.getText();
		if (NOTString.length() > 0)
		{
			queryStringBuffer.append(NOT_OPERATOR);
			queryStringBuffer.append(formatSimpleSearchString(NOTString,NOT_OPERATOR, false));
		}

		// detect hyphenated words and if hyphenate, put quotes around it
		// so that they are considered one word
		// for example, "use-case" as opposed to use-case, which Lucene interprets
		// as "use" NOT "case" which is not what we want
		// 
		// algorithm: first tokenize by space to isolate words
		// next, if words have hyphens, put quotes at the beginning and end
		// append to buffer		
		StringTokenizer spaceTokenizer = new StringTokenizer( queryStringBuffer.toString() );
		StringBuffer wordBuffer = new StringBuffer();
		while( spaceTokenizer.hasMoreTokens() )
		{
			// isolate into words
			String word = spaceTokenizer.nextToken();
			if( -1 != word.indexOf( '-' ) &&
			    !word.startsWith( "\"" ) )
			{
				// hyphen found, and word is not already an exact phrase,
				// so add quotes to beginning and end of word
				wordBuffer.append( "\"" );
				wordBuffer.append( word );
				wordBuffer.append( "\"" );				
			}
			else
			{
				wordBuffer.append( word );	
			}
			if( spaceTokenizer.hasMoreTokens() )
			{
				wordBuffer.append( " " );
			}
		}
		
		_currentSearchString = wordBuffer.toString();
					
		if (_currentSearchString.length() == 0)
		{
			return;
		}
			
		boolean safe = runQuery(SearchRup.CONTENTS);
		
		updateOutput(safe);
	}
	
	/**
	 * Inner classes to handle button clicks.
	 */
	public class AdvancedSearchButtonListener extends SearchButtonListener
	{
		public AdvancedSearchButtonListener(AbstractButton b)
		{
			super(b);
		}
		
		protected void doWork(ActionEvent e)
		{
			doAdvancedSearch();
		}
	}

	
	public class SearchButtonListener implements ActionListener
	{
		public SearchButtonListener(AbstractButton b)
		{
			b.addActionListener( this );
		}
		
		public void actionPerformed( ActionEvent e )
		{
			getContentPane().setCursor(new Cursor(Cursor.WAIT_CURSOR));
			try
			{
				doWork(e);
			}
			finally
			{
				getContentPane().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}
		}
		
		protected void doWork(ActionEvent e)
		{
			if (!_enabled)
			{
				return;
			}
		}
	}
	
	public class NextButtonListener extends SearchButtonListener
	{
		public NextButtonListener(AbstractButton b)
		{
			super(b);
		}
		
		protected void doWork(ActionEvent e)
		{
			_currentPage++;
			updateOutput(true);
		}
	}
	
	public class PrevButtonListener extends SearchButtonListener
	{
		public PrevButtonListener(AbstractButton b)
		{
			super(b);
		}

		protected void doWork(ActionEvent e)
		{
			_currentPage--;
			updateOutput(true);
		}
	}

	/**
	 * Formats the search string with the proper operators.
	 */
	protected String formatSimpleSearchString( String input, 
	                                            String operator, 
	                                            boolean encloseWithParentheses )
	{
		String result = ""; //$NON-NLS-1$
		StringBuffer together = new StringBuffer();
		int firstQuote = input.indexOf("\""); //$NON-NLS-1$
		if (firstQuote >-1)
		{
			int secondQuote = input.indexOf("\"", firstQuote + 1); //$NON-NLS-1$
			while(secondQuote > -1)
			{
				together.append(operator + input.substring(firstQuote, secondQuote + 1));
				input = input.substring(0, firstQuote) + input.substring(secondQuote +1);
				firstQuote = input.indexOf("\""); //$NON-NLS-1$
				secondQuote = input.indexOf("\"", firstQuote + 1); //$NON-NLS-1$
			}
		}
		
		// no more "" so we can just tokenize on whitespace, and make more "AND" additions.
		StringTokenizer tokens = new StringTokenizer(input, " "); //$NON-NLS-1$
		while (tokens.hasMoreElements())
		{
			String toAdd = tokens.nextToken();
			together.append(operator + toAdd);
		}
		
		if( together.toString().startsWith( operator ) )
		{
			result = together.toString().substring(operator.length());
		}
		else
		{
			result = together.toString();
		}

		if (encloseWithParentheses)
		{
			result = "(" + result + ") "; //$NON-NLS-1$ //$NON-NLS-2$
		}

		return result;
	}
	
	/**
	 * Determines whether the index version is out of date.
	 */	
	protected boolean versionOutOfDate(String indexFolder, String httpBase)
	{
		boolean result = true;
		
		String base = httpBase;
		base = base.substring(0, base.lastIndexOf("/")); //$NON-NLS-1$
		base = base + "/search/index/version.txt"; //$NON-NLS-1$

		try
		{
			URL url = new URL(base);
			// handle UNC
			url = ViewerUtilities.handleURLForUNC( url );
			
			BufferedInputStream inputS = new BufferedInputStream( url.openStream());
			InputStreamReader iStreamReader = new InputStreamReader(inputS);
			BufferedReader read = new BufferedReader(iStreamReader);
			String version = read.readLine();
	
			read.close();
			
			String userStringVersion = ""; //$NON-NLS-1$
			File userVersion = new File(indexFolder + File.separator + "version.txt"); //$NON-NLS-1$
			if (userVersion.exists())
			{
				FileReader fRead = new FileReader(userVersion);
				read = new BufferedReader(fRead);
				userStringVersion = read.readLine();
				read.close();
			}
			if (version.equals(userStringVersion))
			{
				result = false;
				_productName = userStringVersion.substring(0, userStringVersion.indexOf("*")); //$NON-NLS-1$
			}
			else
			{
				userVersion.delete();
				userVersion.getParentFile().delete();
				userVersion.getParentFile().mkdirs();
				FileWriter fWrite = new FileWriter(userVersion);
				BufferedWriter write = new BufferedWriter(fWrite);
				write.write(version + "\n"); //$NON-NLS-1$
				write.close();
			}
		}
		catch (Exception e1) 
		{
			e1.printStackTrace();
			result = true;
		}

		return result;
	}

	/**
	 * Inner class to perform search.
	 */
	public class AdvancedSearchListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			getContentPane().setCursor(new Cursor(Cursor.WAIT_CURSOR));
			try
			{
				doWork(e);
			}
			finally
			{
				getContentPane().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}
		}
		
		protected void doWork(ActionEvent e)
		{
			doAdvancedSearch();
		}
	}
	
	
	public class LinkListener implements HyperlinkListener
	{
		String _target = null;
		
		public void hyperlinkUpdate(HyperlinkEvent e)
		{
			if( null == _target )
			{
				_target = getParameter( "target" );
			}			
			try
			{
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
				{
					URL url = new URL(e.getDescription());
					// handle UNC
					url = ViewerUtilities.handleURLForUNC( url );
					getAppletContext().showDocument(url, _target); //$NON-NLS-1$
				}
			}
			catch (Exception e2)
			{
				e2.printStackTrace();
			}
		}
	}
	
	public class SectionChangeListener implements ActionListener
	{
		protected String saveNot;
		protected String saveAny;
		protected String savePhr;
		public static final String DISABLED = "Not supported for RDN search"; //$NON-NLS-1$
		
		public void actionPerformed(ActionEvent evt)
		{
			String selected = _section.getSelectedItem().toString();
			if (RUP_SECTION.equals(selected) && !_notWords.isEnabled())
			{
				_notWords.setText(saveNot);
				_anyWords.setText(saveAny);
				_exactPhrase.setText(savePhr);
				_notWords.setEnabled(true);
				_anyWords.setEnabled(true);
				_exactPhrase.setEnabled(true);
			}
			else if (RDN_SECTION.equals(selected) && _notWords.isEnabled())
			{
				saveNot = _notWords.getText();
				saveAny = _anyWords.getText();
				savePhr = _exactPhrase.getText();
				_notWords.setText(DISABLED);
				_anyWords.setText(DISABLED);
				_exactPhrase.setText(DISABLED);
				_notWords.setEnabled(false);
				_anyWords.setEnabled(false);
				_exactPhrase.setEnabled(false);
			}
		}
	}
	
}

