package com.rational.rpw.processview;

import java.util.Enumeration;
import javax.swing.tree.TreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultMutableTreeNode;

//import com.rational.rpw.compositetree.CompositeNode;


/**
 * This class designates a file in the file system, or alternatively,
 * an entry in a process layout or closure.  Bookmarks form the tree 
 * nodes of a process view. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class Bookmark extends DefaultMutableTreeNode
{
 	static final long serialVersionUID = -3652582791326323863L;

    // points to the description file in the file system
    private String _fileName = null;

    // used as "closed" or "default" icon
    private String _closedIconName = null;

    // used as "open" icon
    private String _openIconName = null;

    // presentation name of the entry
    private String _presentationName = null;

	// whether mouse is over the bookmark
	// this is for backwards compatibility
	//private boolean _isMouseOver = false;
	
	// whether the file exists in the file system
	private boolean _isExist = true;
	
	// whether the file is from content library
	private boolean _isFromContentLibrary = true;
	
	// unique id. 
	// A bookmark may not have a uniqueID in which case the 
	// bookmark has no relation to any layout elements.
	private String _uniqueId = null;
	
	// whether the bookmark is visible
	private boolean _isVisible = true;
	
	// whether the bookmark is enabled
	// this is to support a special case where bookmarks
	// are to remain hidden and cannot be unhidden as
	// well as transparency
	private boolean _isEnabled = true;
	
	// whether the bookmark is transparent
	private boolean _isTransparent = false;
	
	// whether the bookmark is a default one
	// default means that it is published and is read-only
	private boolean _isDefault = false;
	
	// whether the bookmark is the current one
	// current means the first one to be focused on
	private boolean _isCurrent = false;
	
	// description field
	//private String _description = null;
	
   /**
     * Default constructor.  Takes in the presentation name
     * of the process layout entry.
     */
    public Bookmark( String name)
    {
        super( name );        
        _uniqueId = null;        
        _presentationName = name;
    }

    /**
     * Name and uniqueId for a layout node.
     */
    public Bookmark( String presentationName,
   				      String uniqueId)
    {
        super( presentationName );        
        _presentationName = presentationName;
        _uniqueId = uniqueId;        
    }
    
    /**
     * Override to return presentation name as the user object.
     */
    public Object getUserObject()
    {
        return( _presentationName );
    }

	public void setUserObject( Object userObject )
	{
		_presentationName = (String)userObject;
		super.setUserObject( _presentationName );	
	}
	
    public void setPresentationName( String name )
    {
        _presentationName = name;
        super.setUserObject( _presentationName );
    }

    public String getPresentationName()
    {
        return( _presentationName );
    }

    /*public void setDescription( String description )
    {
        _description = description;
    }

    public String getDescription()
    {
        return( _description );
    }*/

    public void setFileName( String name )
    {
        _fileName = name;
    }

    public String getFileName()
    {
        return( _fileName );
    }

    public void setClosedIconName( String name )
    {
        _closedIconName = name;
    }

    public String getClosedIconName()
    {
        return( _closedIconName );
    }

    public String getOpenIconName()
    {
        return( _openIconName );
    }

    public void setOpenIconName( String name )
    {
        _openIconName = name;
    }

	/*public void setMouseOver( boolean isMouseOver )
	{
		_isMouseOver = isMouseOver;
	}
	
	public boolean isMouseOver()
	{
		return( _isMouseOver );	
	}*/
	
	public void setExist( boolean isExist )
	{
		_isExist = isExist;
	}
	
	public boolean isExist()
	{
		return( _isExist );	
	}
	
	public void setFromContentLibrary( boolean flag )
	{
		_isFromContentLibrary = flag;
	}
	
	public boolean isFromContentLibrary()
	{
		return( _isFromContentLibrary );	
	}
	
    public String getUniqueId()
    {
        return( _uniqueId );
    }

	public void setUniqueId( String uniqueId )
	{
		_uniqueId = uniqueId;	
	}
	
	public void setVisible( boolean isVisible )
	{
		_isVisible = isVisible;
		// special case where all children need to change as well
        for( int i = 0; i < super.getChildCount(); i++ )
        {
        	Bookmark child = (Bookmark)((Bookmark)super.getChildAt( i ));
        	child.setVisible( isVisible );
        }
	}
	
	public boolean isVisible()
	{
		return( _isVisible );	
	}
	
	public void setEnabled( boolean isEnabled )
	{
		_isEnabled = isEnabled;
		// special case where all children need to change as well
        for( int i = 0; i < super.getChildCount(); i++ )
        {
        	Bookmark child = (Bookmark)((Bookmark)super.getChildAt( i ));
        	child.setEnabled( isEnabled );
        }
	}
	
	public boolean isEnabled()
	{
		return( _isEnabled );	
	}
	
	public void setTransparency( boolean isTransparent )
	{
		// set at this bookmark only, not children
		_isTransparent = isTransparent;
	}
	
	public boolean isTransparent()
	{
		return( _isTransparent );	
	}
	
	public void setDefault( boolean isDefault )
	{
		_isDefault = isDefault;
	}
	
	public boolean isDefault()
	{
		return( _isDefault );	
	}
	
	public void setCurrent( boolean isCurrent )
	{
		_isCurrent = isCurrent;
	}
	
	public boolean isCurrent()
	{
		return( _isCurrent );	
	}
		
	/**
	 * Override to handle visibility.
	 */
	public void add( MutableTreeNode newChild )
	{
        BookmarkVisibilityFilter.applyActualOnly();
        super.add( newChild );
	    BookmarkVisibilityFilter.removeActualOnly();		
	}

	/**
	 * Override to handle visibility.
	 */
	public void remove( int index )
	{
        BookmarkVisibilityFilter.applyActualOnly();
        super.remove( index );
	    BookmarkVisibilityFilter.removeActualOnly();		
	}

	/**
	 * Override to handle visibility.
	 */
	public void remove( MutableTreeNode node )
	{
        BookmarkVisibilityFilter.applyActualOnly();
        super.remove( node );
	    BookmarkVisibilityFilter.removeActualOnly();		
	}

	/**
	 * Override to handle visibility.
	 */
	public void removeFromParent()
	{
        BookmarkVisibilityFilter.applyActualOnly();
        super.removeFromParent();
	    BookmarkVisibilityFilter.removeActualOnly();		
	}


	/**
	 * Override to handle visiblity.
	 */
  	public TreeNode getChildAt( int index ) 
  	{
  		if( BookmarkVisibilityFilter.isActualOnly() )
  		{
  			return( getActualChildAt( index ) );	
  		}

		boolean isApplyFilter = BookmarkVisibilityFilter.isApplyVisibilityFilter();
    	if(children == null) 
    	{
      		throw new ArrayIndexOutOfBoundsException( "This bookmark has no children." ); //$NON-NLS-1$
    	}
      
	    int realIndex = -1;
	    int visibleIndex = -1;
	    Enumeration enum = children.elements();      
	    while( enum.hasMoreElements() ) 
	    {
	    	Bookmark bookmark = (Bookmark)enum.nextElement();
	    	if( isApplyFilter )
	    	{
	      		if( bookmark.isVisible() && bookmark.isEnabled() && !bookmark.isTransparent() ) 
	      		{
	      			// not hidden, not disabled, and not transparent
					visibleIndex++;
	      		}
	    	}
	    	else
	    	{
	    		// show enabled and not transparent ones, ignore hidden only
	      		if( bookmark.isEnabled() && !bookmark.isTransparent() ) 
	      		{
	      			// enabled, and not transparent
					visibleIndex++;
	      		}
	    	}
	      	realIndex++;
	      	if( visibleIndex == index ) 
	      	{
				return (TreeNode)children.elementAt( realIndex );
	      	}
	    }
	    
	    return( super.getChildAt(index) );
  		//throw new ArrayIndexOutOfBoundsException( "Bookmark index unmatched" );
  	}

	/**
	 * Retrieve acutal regardless of visiblity.
	 */
  	public TreeNode getActualChildAt( int index ) 
	{
		return( super.getChildAt( index ) );
	}

	/**
	 * Override to handle visiblity.
	 */
  	public int getChildCount() 
  	{  	
  		if( BookmarkVisibilityFilter.isActualOnly() )
  		{
  			return( getActualChildCount() );	
  		}
  		
		boolean isApplyFilter = BookmarkVisibilityFilter.isApplyVisibilityFilter();
	    if (children == null) 
	    {
	    	return 0;
	    }
	     
	    int count = 0;
	    Enumeration enum = children.elements();      
	    while( enum.hasMoreElements() ) 
	    {
	    	Bookmark bookmark = (Bookmark)enum.nextElement();
	    	if( isApplyFilter )
	    	{
		      	if( bookmark.isVisible() && bookmark.isEnabled() && !bookmark.isTransparent() ) 
		      	{
		      		// filter applied, only show visible, enabled, and non transparent ones
					count++;
	      		}
	    	}
	    	else
	    	{
		      	if( bookmark.isEnabled() && !bookmark.isTransparent() ) 
		      	{
		      		// only show enabled, and non transparent ones
					count++;
	      		}
	    	}
	    }
	    return count;
  	}
	
	/**
	 * Retrieves real child count regardless of visibility.
	 */
	public int getActualChildCount()
	{
		return( super.getChildCount() );	
	}
	
	/**
	 * Retrieves the index.
	 */
	public int getIndex( TreeNode node )
	{
  		if( BookmarkVisibilityFilter.isActualOnly() )
  		{
  			return( getActualIndex( node ) );	
  		}

		boolean isApplyFilter = BookmarkVisibilityFilter.isApplyVisibilityFilter();
    	if(children == null) 
    	{
      		throw new ArrayIndexOutOfBoundsException( "This bookmark has no children." ); //$NON-NLS-1$
    	}
      
	    int realIndex = -1;
	    int visibleIndex = -1;
	    Enumeration enum = children.elements();      
	    while( enum.hasMoreElements() ) 
	    {
	    	Bookmark bookmark = (Bookmark)enum.nextElement();
	    	if( isApplyFilter )
	    	{
	      		if( bookmark.isVisible() && bookmark.isEnabled() && !bookmark.isTransparent() ) 
	      		{
	      			// not hidden, not disabled, and not transparent
					visibleIndex++;
	      		}
	    	}
	    	else
	    	{
	    		// show enabled and not transparent ones, ignore hidden only
	      		if( bookmark.isEnabled() && !bookmark.isTransparent() ) 
	      		{
	      			// enabled, and not transparent
					visibleIndex++;
	      		}
	    	}
	      	realIndex++;
			
			if( bookmark.equals( (Bookmark)node ) )
			{
				return( visibleIndex );	
			}
	    }
	    
	    return( super.getIndex(node) );
  		//throw new ArrayIndexOutOfBoundsException( "Bookmark index unmatched" );
	}
	
	/**
	 * Retrieves actual index.
	 */
	public int getActualIndex( TreeNode node )
	{
		return( super.getIndex(node) );	
	}
	
	/**
	 * Retrieve previous sibling, taking into account visiblity.
	 */
	public DefaultMutableTreeNode getPreviousSibling()
	{
		boolean isApplyFilter = BookmarkVisibilityFilter.isApplyVisibilityFilter();
		Bookmark parent = (Bookmark)super.getParent();
		int indexOfNode = parent.getActualIndex( this );
    	int levelToMove = indexOfNode - 1;
    	Bookmark upBookmark = null;
    	try
    	{
	    	upBookmark = (Bookmark)parent.getActualChildAt( levelToMove );
	    	if( isApplyFilter )
	    	{
		    	while( upBookmark != null && 
		    	       (!upBookmark.isVisible() || !upBookmark.isEnabled() || upBookmark.isTransparent()) )
		    	{
		    		// move this node up another level
		    		levelToMove = levelToMove - 1;
		    		upBookmark = (Bookmark)parent.getActualChildAt( levelToMove );
		    	}
	    	}
	    	else
	    	{
		    	while( upBookmark != null && 
		    	       (!upBookmark.isEnabled() || upBookmark.isTransparent()) )
		    	{
		    		// move this node up another level
		    		levelToMove = levelToMove - 1;
		    		upBookmark = (Bookmark)parent.getActualChildAt( levelToMove );
		    	}
	    	}
    	}
    	catch( ArrayIndexOutOfBoundsException ae )
    	{
    		return( null );	
    	}
    	
    	return( upBookmark );
	}
	
	/**
	 * Retrieve next sibling, taking into account visiblity.
	 */
	public DefaultMutableTreeNode getNextSibling()
	{
		boolean isApplyFilter = BookmarkVisibilityFilter.isApplyVisibilityFilter();
		Bookmark parent = (Bookmark)super.getParent();
		int indexOfNode = parent.getActualIndex( this );
    	int levelToMove = indexOfNode + 1;
    	Bookmark downBookmark = null;
    	try
    	{
	    	downBookmark = (Bookmark)parent.getActualChildAt( levelToMove );
	    	if( isApplyFilter )
	    	{
		    	while( downBookmark != null && 
		    	       (!downBookmark.isVisible() || !downBookmark.isEnabled() || downBookmark.isTransparent()) )
		    	{
		    		// move this node down another level
		    		levelToMove = levelToMove + 1;
		    		downBookmark = (Bookmark)parent.getActualChildAt( levelToMove );
		    	}
	    	}
	    	else
	    	{
		    	while( downBookmark != null && 
		    	       (!downBookmark.isEnabled() || downBookmark.isTransparent()) )
		    	{
		    		// move this node down another level
		    		levelToMove = levelToMove + 1;
		    		downBookmark = (Bookmark)parent.getActualChildAt( levelToMove );
		    	}
	    	}
    	}
    	catch( ArrayIndexOutOfBoundsException ae )
    	{
    		return( null );	
    	}
    	
    	return( downBookmark );
	}
	
	/**
	 * Finds the bookmark with the unique id.
	 */
	public Bookmark findBookmark( String uniqueId )
	{
		Bookmark foundBookmark = null;
		if( uniqueId != null &&
		    uniqueId.equals( this.getUniqueId() ) )
		{
			return( foundBookmark );
		}
		
		for( int i = 0; i < super.getChildCount(); i++ )
		{
			Bookmark childBookmark = (Bookmark)super.getChildAt( i );
			foundBookmark = findBookmark( childBookmark, uniqueId );
			if( foundBookmark != null )
			{
				return( foundBookmark );
			}
		}
		return( foundBookmark );	
	}
	
	/**
	 * Recursivelys finds the bookmark with the given unique id.
	 */
	private Bookmark findBookmark( Bookmark bookmark,
	                               String uniqueId )
	{
		Bookmark foundBookmark = null;
		
		String tempUniqueId = bookmark.getUniqueId();
		if( null != tempUniqueId && tempUniqueId.equals( uniqueId ) )
		{
			// found
			foundBookmark = bookmark;
			return( foundBookmark );
		}

		for( int i = 0; i < bookmark.getActualChildCount(); i++ )
		{
			Bookmark childBookmark = (Bookmark)bookmark.getActualChildAt( i );
			// recursive find
			foundBookmark = findBookmark( childBookmark, uniqueId );
			if( foundBookmark != null )
			{
				// found
				return( foundBookmark );
			}
		}
		return( foundBookmark );	
	}
  
    /**
     * Override to clone properly.  In particular, this
     * method provides a deep clone.
     */
    public Object clone()
    {
        Bookmark bookmark = new Bookmark( _presentationName, _uniqueId );
        bookmark.setFileName( _fileName );
        bookmark.setClosedIconName( _closedIconName );
        bookmark.setOpenIconName( _openIconName );
        //bookmark.setMouseOver( _isMouseOver );
        bookmark.setExist( _isExist );
        bookmark.setFromContentLibrary( _isFromContentLibrary );
        bookmark.setVisible( _isVisible );
        bookmark.setEnabled( _isEnabled );
        bookmark.setTransparency( _isTransparent );
        bookmark.setDefault( _isDefault );
        bookmark.setCurrent( _isCurrent );
        //bookmark.setDescription( _description );
        
        // deep clone()
        for( int i = 0; i < super.getChildCount(); i++ )
        {
        	Bookmark child = (Bookmark)(((Bookmark)this.getActualChildAt( i )).clone());
        	int childCount = bookmark.getActualChildCount();
        	bookmark.insert( child, childCount );
        }
        return( bookmark );
    }
    
}

