package com.rational.rpw.processview;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;
import javax.swing.UIManager;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.DefaultCellEditor;
import javax.swing.table.TableCellEditor;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.table.TableCellRenderer;
import javax.swing.SwingUtilities;
import java.awt.Dimension;
import java.util.Vector;

/**
 * This class is a modal properties editor which allows the user the 
 * edit My Rup topic properties, such as presentation name, link to 
 * file or url, icon name, etc. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class BookmarkProperties
{
	protected int PROPERTY_COLUMN = 0;
	protected int VALUE_COLUMN = 1;
	protected int BROWSE_COLUMN = 2;
	
	// bookmark (i.e. topic) for which properties are shown
	protected Bookmark _bookmark = null;
	
	// the properties table
	protected JTable _table = null;
	
	// the scroll pane containing the table
	protected JScrollPane _scrollPane = null;
	
	// the browse to select file button
	protected BrowseButton _browseButton = null;
	
	/**
	 * Default constructor.
	 */
    public BookmarkProperties( Bookmark topic ) 
    {
		_bookmark = topic;
		
		// set up custom button renderer
		_browseButton = new BrowseButton();

        TopicTableModel topicModel = new TopicTableModel();        
        _table = new JTable( topicModel );
		_table.setPreferredScrollableViewportSize( new Dimension(500, 70) );

        //Create the scroll pane and add the table to it. 
        _scrollPane = new JScrollPane( _table );

		// set up custom editor
		TopicTextField customField = new TopicTextField( _table );
        DefaultCellEditor customEditor = new DefaultCellEditor( customField );
        customEditor.setClickCountToStart( 1 );        
        _table.setDefaultEditor( String.class, customEditor );
		
		_table.getColumnModel().getColumn( BROWSE_COLUMN ).setCellRenderer( _browseButton );
		ButtonEditor buttonEditor = new ButtonEditor( new JCheckBox(), topicModel );
		buttonEditor.setClickCountToStart( 1 );
		_table.setDefaultEditor( BrowseButton.class, buttonEditor );
		
		// some UI setup - normally done by renderer but this is simple stuff
		// so we can just do it here
		// column width
		TableColumn column = _table.getColumnModel().getColumn( PROPERTY_COLUMN );
   		column.setMinWidth( 130 );
   		column.setMaxWidth( 130 );
   		
   		column = _table.getColumnModel().getColumn( BROWSE_COLUMN );
   		column.setMinWidth( 20 );
   		column.setMaxWidth( 20 );
		
		// cannot select row and column simultaneously
		_table.setCellSelectionEnabled( false );
    }

	/**
	 * Shows the dialog box. Returns one of JOptionPane.OK_OPTION
	 * or JOptionPane.CANCEL_OPTION.
	 */
	public int show()
	{
		int value = show( null );
		return( value );
	}
	
	/**
	 * Shows the dialog box. Returns one of JOptionPane.OK_OPTION
	 * or JOptionPane.CANCEL_OPTION.
	 */
	public int show( Component parent )
	{
		int value = JOptionPane.showConfirmDialog( parent,
												   _scrollPane,
		                                           Translations.getString("PROCESSVIEW_1"), //$NON-NLS-1$
		                                           JOptionPane.OK_CANCEL_OPTION );
		return( value );
	}
	
	/**
	 * Returns the (possibly) changed bookmark.
	 */
	public Bookmark getChangedBookmark()
	{
		return( _bookmark );
	}

	/**
	 * Inner class which knows how to detect user edits.
	 */
	protected class TopicTextField extends JTextField 
	{
		protected JTable _textTable = null;
		protected JTextField _containerField = null;
		protected TopicPlainDocument _document = null;
    	
    	public TopicTextField( JTable table ) 
    	{
        	super();
        	_textTable = table;
    	}

    	protected Document createDefaultModel() 
    	{
	   		_containerField = new JTextField();
    		_document = new TopicPlainDocument( _containerField );
         	return( _document );
    	}

    	protected class TopicPlainDocument extends PlainDocument 
    	{
    		private JTextField _containerField = null;
    		
    		public TopicPlainDocument( JTextField textField )
    		{
    			super();
    			_containerField = textField;
    		}
    		
        	public void insertString( int offs, 
                                      String str,
                                      AttributeSet a )
                throws BadLocationException 
            {            	
            	super.insertString( offs, str, a );            	
            	_containerField.setText( this.getText( 0, this.getLength() ) );
            	updateBookmark();
            }
            
            public void remove( int offs, int len ) 
            	throws BadLocationException
            {
            	super.remove( offs, len );
            	_containerField.setText( this.getText( 0, this.getLength() ) );
            	updateBookmark();
            }
            
            private void updateBookmark()
            {   
            	int row = _textTable.getEditingRow();
        		String valueString = (String)_containerField.getText(); 
        		if( row == 0 )
        		{
        			// presentation name
        			_bookmark.setPresentationName( valueString );
        		}
        		else if( row == 1 )
        		{
        			// link name
        			_bookmark.setFileName( valueString );
	        	}
 	       		else if( row == 2 )
        		{
        			// open icon name
        			_bookmark.setOpenIconName( valueString );
	        	}
	        	else if( row == 3 )
	        	{
	        		// closed icon name
	        		_bookmark.setClosedIconName( valueString );
	 	       	}
            }
    	}
    	
	}
	
	/**
	 * Inner class representing the table model for the topic
	 * properties.
	 */
    protected class TopicTableModel extends AbstractTableModel     	
    {       	
    	private Vector _columnNames = null;
    	private Vector _data = null;
    	private Vector _presentationNameRow = null;
    	private Vector _linkToRow = null;
    	private Vector _openIconNameRow = null;
    	private Vector _closedIconNameRow = null;
    	
    	public TopicTableModel()
    	{
    		super();
    	
    		_columnNames = new Vector();
    		_columnNames.addElement( Translations.getString("PROCESSVIEW_2") ); //$NON-NLS-1$
    		_columnNames.addElement( Translations.getString("PROCESSVIEW_3") ); //$NON-NLS-1$
    		_columnNames.addElement( "" ); //$NON-NLS-1$
    		
    		_data = new Vector();
    		
    		// presentation name
    		_presentationNameRow = new Vector();
    		_presentationNameRow.addElement( Translations.getString("PROCESSVIEW_4") ); //$NON-NLS-1$
    		if( null == _bookmark.getPresentationName() )
    		{
    			_presentationNameRow.addElement( "" ); //$NON-NLS-1$
    		}
    		else
    		{
   				_presentationNameRow.addElement( _bookmark.getPresentationName() );
    		}
    		_presentationNameRow.addElement( "" ); //$NON-NLS-1$
    		
    		// link to ...    		
    		_linkToRow = new Vector();
    		_linkToRow.addElement( Translations.getString("PROCESSVIEW_5") ); //$NON-NLS-1$
    		if( null == _bookmark.getFileName() )
    		{
    			_linkToRow.addElement( "" ); //$NON-NLS-1$
    		}
    		else
    		{
   				_linkToRow.addElement( _bookmark.getFileName() );
    		}
    		_linkToRow.addElement( _browseButton );
    		
    		// open icon name ...
    		_openIconNameRow = new Vector();
    		_openIconNameRow.addElement( Translations.getString("PROCESSVIEW_6") ); //$NON-NLS-1$
    		if( null == _bookmark.getOpenIconName() )
    		{
    			_openIconNameRow.addElement( "" ); //$NON-NLS-1$
    		}
    		else
    		{
   				_openIconNameRow.addElement( _bookmark.getOpenIconName() );
    		}
    		_openIconNameRow.addElement( _browseButton );
    		
    		// closed icon name ...
    		_closedIconNameRow = new Vector();
    		_closedIconNameRow.addElement( Translations.getString("PROCESSVIEW_7") ); //$NON-NLS-1$
    		if( null == _bookmark.getClosedIconName() )
    		{
    			_closedIconNameRow.addElement( "" ); //$NON-NLS-1$
    		}
    		else
    		{
   				_closedIconNameRow.addElement( _bookmark.getClosedIconName() );
    		}
    		_closedIconNameRow.addElement( _browseButton );
    		
    		// data
    		_data.addElement( _presentationNameRow );
    		_data.addElement( _linkToRow );
    		_data.addElement( _openIconNameRow );
    		_data.addElement( _closedIconNameRow );
    		
    	}
    	
        public int getColumnCount() 
        {
            return( _columnNames.size() );
        }
        
        public int getRowCount() 
        {
            return( _data.size() );
        }

        public String getColumnName( int col ) 
        {
            return( (String)_columnNames.elementAt( col ) );
        }

        public Object getValueAt( int row, int col ) 
        {
        	Vector rowData = (Vector)_data.elementAt( row );        	
            return( rowData.elementAt( col ) );
        }

        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.
         */
        public Class getColumnClass( int c ) 
        {
        	if( c == BROWSE_COLUMN )
        	{
        		// button class
        		return( BrowseButton.class );
        	}
        	else
        	{
            	// always return string
            	return( String.class );        		
        	}
        }

        /*
         * Determine which cells are editable.
         */
        public boolean isCellEditable( int row, int col ) 
        {
        	if( col == VALUE_COLUMN )
        	{
        		return( true );	
        	}
        	else if( row != 0 && col == BROWSE_COLUMN )
            { 
                return( true );
            } 
            else 
            {
                return( false );
            }
        }
                                 
        /*
         * Detect change in data.
         */
        public void setValueAt( Object value, int row, int col ) 
        {
			if( col == BROWSE_COLUMN )
			{
				return;	
			}
        	String valueString = (String)value;
        	if( row == 0 )
        	{
        		// presentation name
        		_bookmark.setPresentationName( valueString );
        		_presentationNameRow.setElementAt( valueString, col );
        	}
        	else if( row == 1 )
        	{
        		// link name
        		_bookmark.setFileName( valueString );
        		_linkToRow.setElementAt( valueString, col );
        	}
        	else if( row == 2 )
        	{
        		// open icon name
        		_bookmark.setOpenIconName( valueString );
        		_openIconNameRow.setElementAt( valueString, col );
        	}
        	else if( row == 3 )
        	{
        		// closed icon name
        		_bookmark.setClosedIconName( valueString );
        		_closedIconNameRow.setElementAt( valueString, col );
        	}

			// fire to update the table        	
        	fireTableCellUpdated( row, col );        	
     	}
    };
    
    /**
     * Inner class to render buttons.
     */
    protected class BrowseButton extends JButton 
    	implements TableCellRenderer
    {
		public BrowseButton()
		{
			super( "..." ); //$NON-NLS-1$
			setOpaque( true );
		}

	  	public Component getTableCellRendererComponent( JTable table, 
	  	                                                 Object value,
	                   									 boolean isSelected, 
	                   									 boolean hasFocus, 
	                   									 int row, 
	                   									 int column ) 
	    {
	    	if( !(value instanceof BrowseButton) )
	    	{
	    		return( new JLabel() );	
	    	}
	    	return this;
	  	}			
    };   
    
    /**
     * Inner class to detect button action.
     */
	public class ButtonEditor extends DefaultCellEditor 
	{
  		protected JButton _button;
		protected boolean _isPushed;
		protected TopicTableModel _tableModel;
		protected int _row = 1;
		
  		public ButtonEditor( JCheckBox checkBox,
  		                      TopicTableModel tableModel ) 
  		{
	    	super( checkBox );
	    	_tableModel = tableModel;
	    	_button = new JButton( "..." ); //$NON-NLS-1$
		    _button.setOpaque(true);		    
	    	_button.addActionListener( new ActionListener() 
	    	{
	      		public void actionPerformed(ActionEvent e) 
	      		{
	      			JFileChooser chooser = new JFileChooser( System.getProperty( "user.home" ) ); //$NON-NLS-1$
    				int returnVal = chooser.showOpenDialog( null );
    				if( returnVal == JFileChooser.APPROVE_OPTION ) 
    				{
    					_tableModel.setValueAt( chooser.getSelectedFile().getAbsolutePath(),
    					                        _row,
    					                        VALUE_COLUMN );
    				}
	       	 		fireEditingStopped( );
	      		}
	    	} );
  		}

	  	public Component getTableCellEditorComponent( JTable table, 
	  	                                               Object value,
	                   								   boolean isSelected, 
	                   								   int row, 
	                   								   int column ) 
	    {  
	    	if( isSelected ) 
	    	{
	      		_button.setForeground(table.getSelectionForeground() );
	      		_button.setBackground(table.getSelectionBackground() );
	    	} 
	    	else
	    	{
	      		_button.setForeground(table.getForeground() );
	      		_button.setBackground(table.getBackground() );
	    	}
		    _isPushed = true;
		    _row = row;
			return _button;
	  	}
	  	
	  	public Object getCellEditorValue() 
	  	{
    		_isPushed = false;
			return( _button );
	  	}
  
  		public boolean stopCellEditing() 
  		{
    		_isPushed = false;
    		return super.stopCellEditing();
  		}

  		protected void fireEditingStopped() 
  		{
    		super.fireEditingStopped();
  		}

	};   

}

