package com.rational.rpw.processview;

import java.util.Vector;
import java.util.Iterator;

/**
 * This class represents a context and is used for generating
 * a process view in a context-sensitive manner.  Context is
 * based on components, applicable to the moment when a context
 * sensitive process view is required:
 * <UL>
 * <LI>component list - the name of the process components to show </LI>
 * </UL>
 * <br>
 * 
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class Context
{
	// the tool name
	private String _tool = null;
	
	// the list of topics
	private Vector _topicList = null;
	
	/**
	 * Constructor.
	 */
	public Context()
	{
		_topicList = new Vector();
	}
		
	/**
	 * Adds a topic to the context.
	 */
	public void addTopic( String name )
	{
		// add only if it did not exist before
		if( !hasTopic( name ) )
		{
			_topicList.addElement( name );
		}
	}
	
	/**
	 * Removes a topic from the context.
	 */
	public void removeTopic( String name )
	{
		_topicList.remove( name );		
	}
	
	/**
	 * Determines if topic is present.
	 */
	public boolean hasTopic( String name )
	{
		return( _topicList.contains( name ) );	
	}
		
	// Get and Set methods.
	public String getTool()
	{
		return( _tool );
	}
	
	public void setTool( String tool )
	{
		_tool = tool;	
	}
	
	public Iterator getTopics()
	{
		return( _topicList.iterator() );
	}
	
}

