package com.rational.rpw.processview;

import java.net.URL;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Iterator;

/**
 * This class provides methods to read and parse the content of a topic file. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class TopicFileParser
{
	public static final String TOOL_IDENTIFIER = "tool"; //$NON-NLS-1$
	public static final String TOPIC_IDENTIFIER = "topic"; //$NON-NLS-1$
	public static final String VALUE_DELIMITER = "="; //$NON-NLS-1$
	
	/**
	 * Private constructor.
	 */
	private TopicFileParser()
	{
	}
	
	/**
	 * Writes the topic contents to the topic file.
	 */
	public static void writeTopicFile( String directory,
	                                   String topicFilename,
			                           TopicFile topicFile )
	{
    	String content = null;
    	try
    	{
    		// use proper directory
    		String finalDirectory = null;
    		if( !directory.endsWith( File.separator ) )
    		{
    			finalDirectory = directory + File.separator;
    		}
    		else
    		{
    			finalDirectory = directory;
    		}
    		// check directory existence
    		File directoryExist = new File( finalDirectory );
    		if( !directoryExist.exists() )
    		{
    			directoryExist.mkdirs();
    		}
    		directoryExist = null;    		
    		
    		// delete contents of the file and put new content in
    		FileOutputStream fos = new FileOutputStream( finalDirectory + topicFilename,
    		                                             false );
    		StringBuffer buffer = new StringBuffer();
    		buffer.append( TOOL_IDENTIFIER );
    		buffer.append( VALUE_DELIMITER );
    		buffer.append( topicFile.getTool() );
    		buffer.append( System.getProperty( "line.separator" ) ); //$NON-NLS-1$
    		
    		Iterator topics = topicFile.getTopics();
    		while( topics.hasNext() )
    		{
    			String topic = (String)topics.next();
	    		buffer.append( TOPIC_IDENTIFIER );
 	  	 		buffer.append( VALUE_DELIMITER );    		    		
  		  		buffer.append( topic );
	    		buffer.append( System.getProperty( "line.separator" ) ); //$NON-NLS-1$
    		}
    		
    		fos.write( buffer.toString().getBytes() );
    		fos.close();
    	}
    	catch( Exception e )
    	{
    		e.printStackTrace();	
    	}
    	
		//return( parseContent( content) );				
	}
	
	/**
	 * Reads and parses the given topic file and generates a
	 * TopicFile object from it.
	 */
	public static TopicFile parseTopicFile( URL documentBase,
			                                String topicFilename )
	{
    	String content = null;
    	try
    	{
			// read in the file
	        InputStream inStream = new URL( documentBase, topicFilename ).openStream();
      
      		// note this works for over the network  	
    		StringBuffer result = new StringBuffer();    
    		int c;
    		while( (c = inStream.read() ) != -1 )
    		{
    			result.append( (char)c );
    		}
    		content = result.toString();
        	
        	// close stream
        	inStream.close();
        	inStream = null;
    	}
    	catch( Exception e )
    	{
    		e.printStackTrace();
    		// create a topic file with Any, Any
    		TopicFile topic = new TopicFile();
    		topic.setTool( "Any" ); //$NON-NLS-1$
    		topic.addTopic( "Any" ); //$NON-NLS-1$
    		return( topic );
    	}
    	
		return( parseContent( content) );				
	}
	
	/**
	 * Parses the contents of the topic file and returns
	 * a TopicFile representation.
	 */
	private static TopicFile parseContent( String content )
	{
		TopicFile topicFile = new TopicFile();
		
		StringTokenizer lineTokenizer = new StringTokenizer( content, 
		                                                     System.getProperty( "line.separator" ) ); //$NON-NLS-1$

		while( lineTokenizer.hasMoreElements() )
		{
			String lineToParse = (String)lineTokenizer.nextElement();
			String value = parseForValue( lineToParse );
			if( lineToParse.startsWith( TOOL_IDENTIFIER ) )
			{
				// tool name
				topicFile.setTool( value );
			}
			else if( lineToParse.startsWith( TOPIC_IDENTIFIER ) )
			{
				topicFile.addTopic( value );
			}
		}
				
		return( topicFile );
	}
	
	/**
	 * Parses a line for the value.  The format of the
	 * line is "tool=myToolName" or "topic=myTopicName".
	 */
	private static String parseForValue( String lineToParse )
	{
		int delimiterIndex = lineToParse.indexOf( VALUE_DELIMITER );
		return( lineToParse.substring( delimiterIndex + 1 ).trim() );
	}

}

