package com.rational.rpw.processview;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import java.beans.*;

import java.io.InputStream;

/**
 * @author abencomo
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class XMLDataSerializer
{
	private static ClassLoader customClassLoader = null;
	private static StringBuffer exceptionMessage = new StringBuffer();

	/**
	 * Constructor for SaveLayout.
	 */
	public XMLDataSerializer()
	{
	}

	/**
	 * Persists the layout to an xml file
	 * @param pFile the location of the XML file
	 * @param pObject
	 */
	public static void persistObject(File pFile, Object pObject) throws IOException
	{
		persistObject(pFile, pObject, null);
	}

	/**
	 * Persists the layout to an xml file
	 * @param pFile the location of the XML file
	 * @param pObject
	 * @param pDelegate custom persistencedelegate
	 */
	public static void persistObject(
		File pFile,
		Object pObject,
		PersistenceDelegate pDelegate)
		throws IOException
	{
		exceptionMessage.setLength(0);

		FileOutputStream theOutStream = new FileOutputStream(pFile);
		BufferedOutputStream theBufOutStream = new BufferedOutputStream(theOutStream);
		XMLEncoder theEncoder = new XMLEncoder(theBufOutStream);

		if (pDelegate != null)
		{
			theEncoder.setPersistenceDelegate(pObject.getClass(), pDelegate);
		}

		//check to see if we need to use a custom class loader
		if (customClassLoader != null)
		{
			Thread.currentThread().setContextClassLoader(customClassLoader);
		}

		theEncoder.setExceptionListener(new ExceptionListener()
		{
			public void exceptionThrown(Exception e)
			{
				exceptionMessage.append(e.getMessage());
				//e.printStackTrace();
			}
		});

		theEncoder.writeObject(pObject);
		theEncoder.flush();
		theEncoder.close();
		theBufOutStream.close();
		theOutStream.close();
	}

	/**
	 * Constructs a Layout from an XML file
	 * @param pXMLFile The file to construct the Layout from
	 * @return the deserialized Object
	 */
	public static Object deSerializeObject(File pXMLFile)
	{
		if (!pXMLFile.exists())
		{
			return null;
		}
		exceptionMessage.setLength(0);

		Object theNewObject = null;
		try
		{
			FileInputStream theInStream = new FileInputStream(pXMLFile);
			BufferedInputStream theBufInStream = new BufferedInputStream(theInStream);
			XMLDecoder theDecoder = new XMLDecoder(theBufInStream);

			//check to see if we need to use a custom class loader
			if (customClassLoader != null)
			{
				Thread.currentThread().setContextClassLoader(customClassLoader);
			}

			theDecoder.setExceptionListener(new ExceptionListener()
			{
				public void exceptionThrown(Exception e)
				{
					exceptionMessage.append(e.getMessage());
					//e.printStackTrace();
				}
			});

			theNewObject = theDecoder.readObject();

			theDecoder.close();
			theBufInStream.close();
			theInStream.close();
		}
		catch (IOException ioe)
		{
			System.out.println(
				"XMLDataSerializer.deSerializeObject() => " + ioe.getMessage());
		}

		return theNewObject;
	}

	public static Object deSerializeObjectURL(InputStream inStream)
	{
		exceptionMessage.setLength(0);

		Object theNewObject = null;
		try
		{
			BufferedInputStream theBufInStream = new BufferedInputStream(inStream);
			XMLDecoder theDecoder = new XMLDecoder(theBufInStream);

			//check to see if we need to use a custom class loader
			if (customClassLoader != null)
			{
				Thread.currentThread().setContextClassLoader(customClassLoader);
			}

			theDecoder.setExceptionListener(new ExceptionListener()
			{
				public void exceptionThrown(Exception e)
				{
					exceptionMessage.append(e.getMessage());
					//e.printStackTrace();
				}
			});

			theNewObject = theDecoder.readObject();

			theDecoder.close();
			theBufInStream.close();
		}
		catch (IOException ioe)
		{
			System.out.println(
				"XMLDataSerializer.deSerializeObject() => " + ioe.getMessage());
		}
		catch (Exception e)
		{
			javax.swing.JOptionPane.showMessageDialog(null, e.toString());
			javax.swing.JOptionPane.showMessageDialog(null, inStream.toString());
		}

		return theNewObject;
	}

	/**
	 * Specifies the XMLEncoder to use the specified ClassLoader. 
	 * @param pLoader
	 */
	public static void setCustomClassLoader(ClassLoader pLoader)
	{
		customClassLoader = pLoader;
	}

	/**
	 * @return the Exception message from the decoder/encoder
	 */
	public static String getExceptionMessage()
	{
		return exceptionMessage.toString();
	}

	public static boolean hasExceptionMessage()
	{
		if (exceptionMessage.length() > 0)
		{
			return true;
		}
		return false;
	}
	
	protected static class LayoutPersistenceDelegate extends DefaultPersistenceDelegate
	{
		public Expression instantiate(Object oldInstance, Encoder encoder)
		{
			return new Expression(
				oldInstance,
				oldInstance.getClass(),
				"new",
				new Object[]{ oldInstance.toString() });
		}
	}
		
	protected static class BookmarkPersistenceDelegate extends DefaultPersistenceDelegate
	{
		protected Expression instantiate(Object oldInstance, Encoder encoder)
		{
			String pName = ((Bookmark) oldInstance).getPresentationName();
			String uId = ((Bookmark) oldInstance).getUniqueId();

			BookmarkVisibilityFilter.removeVisiblityFilter();

			return new Expression(
				oldInstance,
				oldInstance.getClass(),
				"new",
				new Object[] { pName, uId });
		}
	}
		
	public static PersistenceDelegate getLayoutPersistenceDelegate()
	{
		return new LayoutPersistenceDelegate();
	}
	
	public static PersistenceDelegate getBookmarkPersistenceDelegate()
	{
		return new BookmarkPersistenceDelegate();
	}		
}