package com.rational.rpw.processviewer;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import javax.swing.BorderFactory;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.io.File;
import java.awt.Color;

import com.rational.rpw.processview.*;

/**
 * This class provides the main menu icon strip for view actions. 
 * Actions here include save, save as, add new node, etc. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ViewerMainIconMenu
	implements ActionListener
{
	protected IViewerActionListener _actionListener = null;
	
	protected JPanel _buttonPanel = null;

	protected JButton _saveAsButton = null;
	protected JButton _deleteButton = null;
	protected JButton _addNewButton = null;
	
	/**
	 * Constructor.
	 */
	public ViewerMainIconMenu( IViewerActionListener listener )
	{
		_actionListener = listener;
	}
		
	/**
	 * Construct icon menu panel
	 */
	public JPanel getMenuPanel()
	{
		if( null == _buttonPanel )
		{
			IconManager iconManager = new IconManager();
			String documentBase = DocumentBase.getDocumentBase().toString();
			int index = documentBase.lastIndexOf( ViewerUtilities.UNC_SEPARATOR );
			if( -1 == index )
			{
				index = documentBase.lastIndexOf( File.separator );
			}
			String finalDocumentBase = null;
			if( -1 != index )
			{
				finalDocumentBase = documentBase.substring( 0,index );				
			}
			else
			{
				finalDocumentBase = documentBase;
			}
			String base = ViewerUtilities.convertFilename( finalDocumentBase + 
		    	                                           ViewerUtilities.UNC_SEPARATOR +
		    	                                           "images" + //$NON-NLS-1$
		    	                                           ViewerUtilities.UNC_SEPARATOR );
 	   		ImageIcon saveAsIcon = iconManager.getIcon( base + "action_saveas.gif" ); //$NON-NLS-1$
  	   		ImageIcon deleteIcon = iconManager.getIcon( base + "action_delete.gif" ); //$NON-NLS-1$
   	 		ImageIcon addNewIcon = iconManager.getIcon( base + "action_new.gif" ); //$NON-NLS-1$
   			ImageIcon addFromDefaultIcon = iconManager.getIcon( base + "action_add.gif" ); //$NON-NLS-1$
    		
			// insets and background
			java.awt.Insets insets = new java.awt.Insets( 0, 0, 0, 0 );
			Color background = new Color( 239, 239, 239 );
			
			// save, etc.
			_buttonPanel = new JPanel();			
			_buttonPanel.setLayout( new GridLayout( 1, 0 ) );
			_buttonPanel.setBackground( background );

			_saveAsButton = new JButton( saveAsIcon );
			_saveAsButton.setToolTipText( Translations.getString("PROCESSVIEWER_20") );  //$NON-NLS-1$
			_saveAsButton.setMargin( insets );	 
			_saveAsButton.setBackground( background );			
			_saveAsButton.setBorder( BorderFactory.createLineBorder( Color.white ) );
			_saveAsButton.addActionListener( this );
			_buttonPanel.add( _saveAsButton );
			
			_deleteButton = new JButton( deleteIcon );
			_deleteButton.setToolTipText( Translations.getString("PROCESSVIEWER_21") );  //$NON-NLS-1$
			_deleteButton.setMargin( insets );	 
			_deleteButton.setBackground( background );			
			_deleteButton.setBorder( BorderFactory.createLineBorder( Color.white ) );
			_deleteButton.addActionListener( this );
			_buttonPanel.add( _deleteButton );

			_addNewButton = new JButton( addNewIcon );
			_addNewButton.setToolTipText( Translations.getString("PROCESSVIEWER_22") );  //$NON-NLS-1$
			_addNewButton.setMargin( insets );	 
			_addNewButton.setBackground( background );			
			_addNewButton.setBorder( BorderFactory.createLineBorder( Color.white ) );
			_addNewButton.addActionListener( this );
			_buttonPanel.add( _addNewButton );
		}
		return( _buttonPanel );
	}
		
	/**
	 * Action listener.
	 */
	public void actionPerformed( ActionEvent e )
	{
		if( e.getSource() == _saveAsButton )
		{
			_actionListener.handleAction( IViewerActionListener.ACTION_SAVE_VIEW_AS );
		}
		else if( e.getSource() == _deleteButton )
		{
			_actionListener.handleAction( IViewerActionListener.ACTION_DELETE_VIEW );
		}
		else if( e.getSource() == _addNewButton )
		{
			_actionListener.handleAction( IViewerActionListener.ACTION_ADD_NEW_NODE );
		}
	}

	// FOR TEST ONLY
	public static void main( String[] args )
	{
		try
		{
			javax.swing.JFrame frame = new javax.swing.JFrame( "Test Menu Icon Panel" ); //$NON-NLS-1$
			DocumentBase.setDocumentBase( new java.net.URL( "file", "", "E:\\Program Files\\Rational\\RUPBuilder.mckb\\" ) ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
			ViewerMainIconMenu menu = new ViewerMainIconMenu( null );
			frame.getContentPane().add( menu.getMenuPanel(), java.awt.BorderLayout.CENTER );
			frame.pack();
			frame.show();
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}
	}
}

