package com.rational.rpw.processviewer;

import java.io.File;
import java.util.Vector;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.tree.MutableTreeNode;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import com.rational.rpw.compositetreeview.TreePopupMenuAdapter;
import com.rational.rpw.processview.Bookmark;
import com.rational.rpw.processview.BookmarkProperties;
import com.rational.rpw.processview.BookmarkVisibilityFilter;

/**
 * This class provides the popup menu handling for the process
 * view editor. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ViewerPopupMenuAdapter extends TreePopupMenuAdapter
    implements ActionListener
{
    protected final static String MOVE_NODE_UP = Translations.getString("PROCESSVIEWER_30"); //$NON-NLS-1$
    protected final static String MOVE_NODE_DOWN = Translations.getString("PROCESSVIEWER_31"); //$NON-NLS-1$
    protected final static String DELETE_NODE = Translations.getString("PROCESSVIEWER_32"); //$NON-NLS-1$
    protected final static String PROPERTIES_NODE = Translations.getString("PROCESSVIEWER_34"); //$NON-NLS-1$
    protected final static String ADD_NEW_NODE = Translations.getString("PROCESSVIEWER_35"); //$NON-NLS-1$
 
    protected JPopupMenu _popupMenu = null;
    protected Vector _nodeList = null;
    protected Component _parentComponent = null;
    
    /**
     * Default constructor.
     */
    public ViewerPopupMenuAdapter()
    {
        super();
           
        // set up popup menu
        _popupMenu = new JPopupMenu();
        
        JMenuItem propertiesMenuItem = new JMenuItem( PROPERTIES_NODE );
        propertiesMenuItem.addActionListener( this );        

        JMenuItem moveUpMenuItem = new JMenuItem( MOVE_NODE_UP );
        moveUpMenuItem.addActionListener( this );        
        
        JMenuItem moveDownMenuItem = new JMenuItem( MOVE_NODE_DOWN );
        moveDownMenuItem.addActionListener( this );        
        
        JMenuItem addMenuItem = new JMenuItem( ADD_NEW_NODE );
        addMenuItem.addActionListener( this );        

        JMenuItem deleteMenuItem = new JMenuItem( DELETE_NODE );
        deleteMenuItem.addActionListener( this );                

        _popupMenu.add( propertiesMenuItem );                
        _popupMenu.add( moveUpMenuItem );      
        _popupMenu.add( moveDownMenuItem );                
        _popupMenu.add( addMenuItem );                
        _popupMenu.add( deleteMenuItem );                
    }

	/**
	 * Set parent component of this popup menu.
	 */
	public void setParentComponent( Component parentComponent )
	{
		_parentComponent = parentComponent;	
	}
	
    /**
     * Gets the popup menu for the multiple nodes.
     * @param   node        The node for the popup menu.
     * @return  JPopupMenu  The popup menu for the node.
     */
    public JPopupMenu getMenu( Vector nodes )
    {
    	_nodeList = nodes;

		// right away, properties, add new node are disabled
       	_popupMenu.getComponent( 0 ).setEnabled( false );
       	_popupMenu.getComponent( 3 ).setEnabled( false );
        
        // delete always enabled
       	_popupMenu.getComponent( 4 ).setEnabled( true );
       	
       	// move up, move down - enable only if in a sequence
       	boolean isSiblings = true;
       	
   		Bookmark parent = (Bookmark)((Bookmark)_nodeList.elementAt( 0 )).getParent();
       	for( int i = 0; i < _nodeList.size() - 1; i++ )
       	{
       		Bookmark firstNode = (Bookmark)_nodeList.elementAt( i );
       		Bookmark secondNode = (Bookmark)_nodeList.elementAt( i + 1 );
       		if( !firstNode.isNodeSibling( secondNode ) )
       		{
       			isSiblings = false;
       			break;
       		}
       	}
       	if( isSiblings )
       	{
       		Bookmark topNode = (Bookmark)_nodeList.elementAt( 0 );
       		Bookmark bottomNode = (Bookmark)_nodeList.elementAt( 0 );
       		for( int i = 0; i < _nodeList.size(); i++ )
       		{
       			Bookmark comparatorNode = (Bookmark)_nodeList.elementAt( i );
       			int index = parent.getActualIndex( comparatorNode );
       			int topIndex = parent.getActualIndex( topNode );
       			int bottomIndex = parent.getActualIndex( bottomNode );
       			if( index < topIndex )
       			{
       				topNode = comparatorNode;
       			}
       			if( index > bottomIndex )
       			{
       				bottomNode = comparatorNode;
       			}
       		}
       		
       		// if top node has previous sibling, enable move up
       		// if top node has no previous sibling, disable move up       		
       		// if bottom node has next sibling, enable move down       		
       		// if bottom node has no next sibling, disable move down
       		if( topNode.getPreviousSibling() != null )
       		{
	        	_popupMenu.getComponent( 1 ).setEnabled( true );
       		}
       		else
       		{
	        	_popupMenu.getComponent( 1 ).setEnabled( false );
       		}
       		if( bottomNode.getNextSibling() != null )
       		{
	        	_popupMenu.getComponent( 2 ).setEnabled( true );
       		}
       		else
       		{
	        	_popupMenu.getComponent( 2 ).setEnabled( false );
       		}       		
       	}
       	else
       	{
       		// not siblings, so disable move up, move down
        	_popupMenu.getComponent( 1 ).setEnabled( false );
        	_popupMenu.getComponent( 2 ).setEnabled( false );       		
       	}
        
        
        return( _popupMenu );   
    }

    /**
     * Checks whether a menu is available for this node.
     * @param   node        The node for which the menu is available or not.
     * @return  boolean     <code>true</code> if so, <code>false</code> if not.
     */
    public boolean isMenuAvailable( Vector nodes )
    {
        return( true );   
    }
    	
    /**
     * Gets the popup menu.
     */
    public JPopupMenu getMenu( MutableTreeNode node )
    {
    	_nodeList = new Vector();
    	_nodeList.addElement( node );        
        
		Bookmark bookmark = (Bookmark)_nodeList.elementAt( 0 );

        // logic to enable or disable menu items depending
        // on node
        if( !bookmark.isVisible() )
        {
        	// everything is disabled
        	_popupMenu.getComponent( 0 ).setEnabled( false );
        	_popupMenu.getComponent( 1 ).setEnabled( false );
        	_popupMenu.getComponent( 2 ).setEnabled( false );
        	_popupMenu.getComponent( 3 ).setEnabled( false );
        	_popupMenu.getComponent( 4 ).setEnabled( false );
        	return( _popupMenu );
        }
        
        int indexOfNode = ((Bookmark)node.getParent()).getActualIndex( node );
        int nodeCount = ((Bookmark)node.getParent()).getActualChildCount();
		
        if( indexOfNode == (nodeCount - 1) && nodeCount > 1 )
        {
        	// if node is last, enable move up but disable move down
        	_popupMenu.getComponent( 1 ).setEnabled( true );
        	_popupMenu.getComponent( 2 ).setEnabled( false );
        }
        else if( indexOfNode == 0 && nodeCount > 1 )
        {
        	// if node is first, disable move up but enable move down
        	_popupMenu.getComponent( 1 ).setEnabled( false );
        	_popupMenu.getComponent( 2 ).setEnabled( true );
        }
        else if( indexOfNode == 0 && nodeCount == 1 )
        {
        	// the only node, disable both
        	_popupMenu.getComponent( 1 ).setEnabled( false );
        	_popupMenu.getComponent( 2 ).setEnabled( false );
        }
        else
        {
        	// in between
			Bookmark upBookmark = (Bookmark)bookmark.getPreviousSibling();
			Bookmark downBookmark = (Bookmark)bookmark.getNextSibling();
			if( null == upBookmark )
			{
				// node is second, but first node is invisible
				// disable move up
	        	_popupMenu.getComponent( 1 ).setEnabled( false );
			}
			else
			{
	        	// in between, enabled
 		       	_popupMenu.getComponent( 1 ).setEnabled( true );				
			}
			if( null == downBookmark )
			{
				// node is second last, but last node is invisible
				// disable move down
	        	_popupMenu.getComponent( 2 ).setEnabled( false );
			}
			else
			{
	        	// in between, enabled
	        	_popupMenu.getComponent( 2 ).setEnabled( true );			
			}			
        }
        
		// delete is always enabled
      	_popupMenu.getComponent( 4 ).setEnabled( true );
		// add is always enabled
      	_popupMenu.getComponent( 0 ).setEnabled( true );
		// properties is always enabled
      	_popupMenu.getComponent( 3 ).setEnabled( true );
        
        return( _popupMenu );   
    }

    /**
     * Checks whether a menu is available for this node.
     * @param   node        The node for which the menu is available or not.
     * @return  boolean     <code>true</code> if so, <code>false</code> if not.
     */
    public boolean isMenuAvailable( MutableTreeNode node )
    {
        return( true );   
    }
    

    /**
     * ActionListener interface implementation.
     */
    public void actionPerformed( ActionEvent e ) 
    {
    	if( _nodeList.size() == 1 )
    	{
    		handleActionForOne( e );
    	}
    	else
    	{
    		handleActionForMulti( e );
    	}
    }
    
    /**
     * Handles action when one node is selected.
     */
    protected void handleActionForOne( ActionEvent e )
    {
        JMenuItem source = (JMenuItem)(e.getSource());
        MutableTreeNode node = (MutableTreeNode)_nodeList.elementAt( 0 );        
        MutableTreeNode parentNode = (MutableTreeNode)node.getParent();

		Bookmark bookmark = (Bookmark)node;
        if( source.getText().equals( MOVE_NODE_UP ) )
        {
			Bookmark upBookmark = (Bookmark)bookmark.getPreviousSibling();
			int levelToMove = ((Bookmark)parentNode).getActualIndex( upBookmark );
            parentNode.insert( (Bookmark)bookmark.clone(), levelToMove );
            _treeChangeListener.nodeStructureChanged( parentNode );
            parentNode.remove( bookmark );
            _treeChangeListener.nodeStructureChanged( parentNode );
        }
        else if( source.getText().equals( MOVE_NODE_DOWN ) )
        {
			Bookmark downBookmark = (Bookmark)bookmark.getNextSibling();
			int levelToMove = ((Bookmark)parentNode).getActualIndex( downBookmark );
            parentNode.insert( (Bookmark)bookmark.clone(), levelToMove + 1 );
            _treeChangeListener.nodeStructureChanged( parentNode );
            parentNode.remove( bookmark );
            _treeChangeListener.nodeStructureChanged( parentNode );            
        }
        else if( source.getText().equals( DELETE_NODE ) )
        {
            // deletes this node             	
            node.removeFromParent();
            _treeChangeListener.nodeStructureChanged( parentNode );
        }
        else if( source.getText().equals( ADD_NEW_NODE ) )
        {
            // insert a new node into this node
            Bookmark newBookmark = new Bookmark( Translations.getString("PROCESSVIEWER_41") ); //$NON-NLS-1$
			newBookmark.setFromContentLibrary( false );				
			String newIconName = "new_topic.gif"; //$NON-NLS-1$
			newBookmark.setOpenIconName( newIconName );
			newBookmark.setClosedIconName( newIconName );
            // provide a dialog box to get the properties
            BookmarkProperties propertyDialog = new BookmarkProperties( newBookmark );
  		  	int inputValue = propertyDialog.show( _parentComponent );
      		if( inputValue == JOptionPane.OK_OPTION )
       		{
       			Bookmark changedBookmark = propertyDialog.getChangedBookmark();
       			String presentationName = null;
       			if( changedBookmark.getPresentationName() == null ||
       			    changedBookmark.getPresentationName().equals( "" ) ) //$NON-NLS-1$
       			{
       				presentationName = Translations.getString("PROCESSVIEWER_41"); //$NON-NLS-1$
       			}
       			else
       			{
       				presentationName = changedBookmark.getPresentationName();
       			}
       			newBookmark.setPresentationName( presentationName );
            	newBookmark.setFileName( changedBookmark.getFileName() );
            	newBookmark.setOpenIconName( changedBookmark.getOpenIconName() );
            	newBookmark.setClosedIconName( changedBookmark.getClosedIconName() );
            	
            	newBookmark.setFromContentLibrary( false );
	            ((Bookmark)node).add( newBookmark );				
 	           _treeChangeListener.nodeStructureChanged( node );
       		}
        }
        else if( source.getText().equals( PROPERTIES_NODE ) )
        {
            // provide a dialog box to get the properties
            BookmarkProperties propertyDialog = new BookmarkProperties( (Bookmark)bookmark.clone() );
  		  	int inputValue = propertyDialog.show( _parentComponent );
      		if( inputValue == JOptionPane.OK_OPTION )
       		{
       			Bookmark changedBookmark = propertyDialog.getChangedBookmark();
       			bookmark.setPresentationName( changedBookmark.getPresentationName() );
            	bookmark.setFileName( changedBookmark.getFileName() );
            	bookmark.setOpenIconName( changedBookmark.getOpenIconName() );
            	bookmark.setClosedIconName( changedBookmark.getClosedIconName() );
            	
            	bookmark.setFromContentLibrary( false );
            	            	
            	_treeChangeListener.nodeChanged( bookmark );
            	_treeChangeListener.nodeSelectionChanged( bookmark );
       		}
       		
       		propertyDialog = null;
        }            	
    }

    /**
     * Handles action when multiple nodes selected.
     */
    protected void handleActionForMulti( ActionEvent e )
    {
        JMenuItem source = (JMenuItem)(e.getSource());

   		Bookmark parentNode = (Bookmark)((Bookmark)_nodeList.elementAt( 0 )).getParent();
   		Bookmark topNode = (Bookmark)_nodeList.elementAt( 0 );
   		Bookmark bottomNode = (Bookmark)_nodeList.elementAt( 0 );
   		for( int i = 0; i < _nodeList.size(); i++ )
   		{
   			Bookmark comparatorNode = (Bookmark)_nodeList.elementAt( i );
   			int index = parentNode.getActualIndex( comparatorNode );
   			int topIndex = parentNode.getActualIndex( topNode );
   			int bottomIndex = parentNode.getActualIndex( bottomNode );
   			if( index < topIndex )
   			{
   				topNode = comparatorNode;
   			}
   			if( index > bottomIndex )
   			{
   				bottomNode = comparatorNode;
   			}
   		}

        if( source.getText().equals( MOVE_NODE_UP ) )
        {
			Bookmark anchorNode = (Bookmark)topNode.getPreviousSibling();
			int levelToMove = parentNode.getActualIndex( anchorNode );
			for( int i = 0; i < _nodeList.size(); i++ )
			{
				Bookmark bookmark = (Bookmark)_nodeList.elementAt( i );
            	parentNode.insert( (Bookmark)bookmark.clone(), levelToMove );
            	parentNode.remove( bookmark );
            	levelToMove++;
			}
           	_treeChangeListener.nodeStructureChanged( parentNode );
        }
        else if( source.getText().equals( MOVE_NODE_DOWN ) )
        {
			Bookmark anchorNode = (Bookmark)bottomNode.getNextSibling();
			int levelToMove = parentNode.getActualIndex( anchorNode ) + 1;
			for( int i = 0; i < _nodeList.size(); i++ )
			{
				Bookmark bookmark = (Bookmark)_nodeList.elementAt( i );
            	parentNode.insert( (Bookmark)bookmark.clone(), levelToMove );
            	parentNode.remove( bookmark );            	
			}
           	_treeChangeListener.nodeStructureChanged( parentNode );
        }
        else if( source.getText().equals( DELETE_NODE ) )
        {
            // deletes all the given nodes
            for( int i = 0; i < _nodeList.size(); i++ )
            {
            	Bookmark node = (Bookmark)_nodeList.elementAt( i );
            	node.removeFromParent();
            }
           	_treeChangeListener.nodeStructureChanged( parentNode );
        }
    }

    
}