package com.rational.rpw.processviewer;

import java.awt.Component;
import java.awt.Color;
import java.awt.Font;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.JLabel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultMutableTreeNode;
import java.io.File;
import java.net.URL;
import java.util.HashMap;

import com.rational.rpw.processview.Bookmark;
import com.rational.rpw.processview.IconManager;
import com.rational.rpw.environment.FontFactory;

/**
 * This class renders tree cells for the process view trees.
 * It "knows" how to handle bookmarks in order to render each tree cell. <br>
 *
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class ViewerTreeCellRenderer extends DefaultTreeCellRenderer
{
	// icon manager
	protected IconManager _iconManager = null;
	
	// cache icons (these are icons that are not already cached 
	// by the icon manager; this helps to speed up icon retrieval
	// during subsequent rendering
	protected HashMap _usedIcons = null;
	
	public ViewerTreeCellRenderer()
	{
		super();
		_usedIcons = new HashMap( 300 );
	}
	
	public ViewerTreeCellRenderer( IconManager iconManager )
	{
		_iconManager = iconManager;	
		_usedIcons = new HashMap( 300 );
	}
	
    /**
     * Sets the current tree cell to value.
     */
    public Component getTreeCellRendererComponent( JTree tree,
                                                   Object value,
                                                   boolean sel,
                                                   boolean expanded,
                                                   boolean leaf,
                                                   int row,
                                                   boolean hasFocus)
    {
        // set all the defaults first
        super.getTreeCellRendererComponent( tree,
                                            value,
                                            sel,
                                            expanded,
                                            leaf,
                                            row,
                                            hasFocus );
        Bookmark bookmark = (Bookmark)value;
        this.setText( bookmark.getPresentationName() );
        this.setFont( FontFactory.getCommonPlainFont() );
        
        String iconName = null;
        if( expanded )
        {
        	iconName = bookmark.getOpenIconName();
        }
        else
        {
        	iconName = bookmark.getClosedIconName();        	
        }
        if( null != iconName && null != _iconManager )
        {
       		ImageIcon icon = null;
       		if( _iconManager.hasResourceFile() )
       		{
       			icon = _iconManager.getResource( iconName );
       		}
       		if( null == icon || icon.getIconWidth() == -1 )
       		{
       			// see if it has been cached
       			if( _usedIcons.containsKey( iconName ) )
       			{
       				icon = (ImageIcon)_usedIcons.get( iconName );
       			}
       			else
       			{
       				// if doesn't exist, try getting it from the file system
					icon = _iconManager.getIcon( iconName );
					if( null != icon && icon.getIconWidth() != -1 )
					{
						// cache it for future use
						_usedIcons.put( iconName, icon );
					}
				}   			
       		}
       		this.setIcon( icon );
        }
        else if( null != iconName )
        {
        	this.setIcon( new ImageIcon( iconName ) );
        }
        
        return this;
    }

}