package com.rational.rpw.rpwapplication;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

/**
 * This class is a JTextField that limits the number of
 * characters.
 *
 * Copyright 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */
public class LimitedTextField extends JTextField
{
	protected static int _maximumCharacters = 64;
	
	/**
	 * Default constructor.
	 */
	public LimitedTextField( int maxCharacters )
	{
		super();
		_maximumCharacters = maxCharacters;	
	}
	
	/**
	 * Default constructor.
	 */
	public LimitedTextField( String text,
	                          int maxCharacters )
	{
		super( text );
		_maximumCharacters = maxCharacters;	
	}
	
	/**
	 * Override to return a limited document.
	 */
	protected Document createDefaultModel() 
	{		
		return new LimitedDocument();	
	}
	
	/**
	 * Inner class to handle the limit.
	 */
	protected static class LimitedDocument extends PlainDocument 
	{
		private int _current = 0;

		public void insertString( int offs, 
		                           String str, 
		                           AttributeSet a ) 
			throws BadLocationException 
		{
			if( str == null || 
			    _current >= _maximumCharacters ||
			    (_current + str.length()) > _maximumCharacters ) 
			{
				return;
			}
			_current += str.length();
			super.insertString(offs, str, a);
		}
		
			public void remove( int offs, 
			                     int len )
				throws BadLocationException 
			{
				if( offs >= _maximumCharacters || len > _maximumCharacters || offs < 0 || len < 0 ) 
				{
					return;	
				}
				_current -= len;
				super.remove(offs, len);
			}
		}
	
}
