package com.rational.rpw.rpwapplication;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JFrame;

import com.rational.rpw.environment.EnvironmentException;
import com.rational.rpw.environment.RegistryService;
import com.rational.rpw.environment.StringConstants;
import com.rational.rpw.environment.UserPreferences;
import com.rational.rpw.utilities.CommonFunctions;

/**
 *
 * Invokes an editor on the selected file. The path to the desired
 * application is cached so that the user only needs to select the
 * application once.
 *
 * @exception throws an exception if the input file does not exist.
 *
 * We first look in the registry under the key HKCR\htmlfile\shell\Edit\command
 * and if not found then we prompt the user for the correct location.
 *
 */

//---------------------------------------------------------------------------------------------------
public class RunExternalProgram
{
	private String pathToHTMLExecutable		= null;
	private String pathToGIFExecutable			= null;
	private String pathToJPEGExecutable		= null;
	private String pathToUnknown				= null;
	private String pathToBrowser				= null;
	private UserPreferences theUserPref		= UserPreferences.getInstance();

	//-----------------------------------------------------------------------------------------------
	// Constructor.
	public RunExternalProgram()
	{
		return;
	}

	//-----------------------------------------------------------------------------------------------
	// Pass the file to invoke the external program on. Need to check in this dialog what type of
	// file it is and to get the correct program for that association.
	// For example:
	//		HTML --- Front Page.
	//		GIF ---	 Graphics Editor.
	//		etc....
	public void invoke(String fileName)
		throws IOException
	{
		// Check if the constructed filename actually exists in the local filesystem and if
		// not then return an error.
		File myFile = new File(fileName);
		if ( !myFile.exists() )
		{
			throw new IOException( Translations.getString("RPWAPPLICATION_41") //$NON-NLS-1$
				+ " " //$NON-NLS-1$
				+ fileName
				+ " " //$NON-NLS-1$
				+ Translations.getString("RPWAPPLICATION_42")); //$NON-NLS-1$
		}

		// Get the User Associations for file types to applications.
		// pathToHTMLExecutable	= convertPath(theUserPref.getHTMLViewer());
		// pathToGIFExecutable		= convertPath(theUserPref.getGIFViewer());
		// pathToJPEGExecutable	= convertPath(theUserPref.getJPEGViewer());
		
		pathToHTMLExecutable	= CommonFunctions.formatPath(theUserPref.getViewingApplication(".htm"));
		pathToGIFExecutable		= CommonFunctions.formatPath(theUserPref.getViewingApplication(".gif"));
		pathToJPEGExecutable	= CommonFunctions.formatPath(theUserPref.getViewingApplication(".jpg"));
		
		// Determine what type of file this is and grab the correct application to load it with.
		if ( fileName.endsWith(".htm") || fileName.endsWith(".html") || fileName.endsWith(".shtml") ) //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		{
			// If the application has not been selected yet or it is set to use system default.
			if ( pathToHTMLExecutable.equals(StringConstants.useSystemDefault) || pathToHTMLExecutable.equals(StringConstants.missingResourceError) )
			{
				RegistryService theService = RegistryService.getInstance();
				try {
					pathToHTMLExecutable = theService.getEditor(".HTML");
				} catch ( EnvironmentException ee ) {
					// Cannot find the value for this key in the registry. Throw an IO Exception.
					throw new IOException(Translations.getString("RPWAPPLICATION_43") //$NON-NLS-1$
										  + Translations.getString("RPWAPPLICATION_44")); //$NON-NLS-1$
				}

				// Sometimes there is garbage on the end (such as %1) so we will strip this off.
				int pos;
				if ( pathToHTMLExecutable != null && ((pos = pathToHTMLExecutable.lastIndexOf("%")) != -1) ) //$NON-NLS-1$
				{
					// Keys in the registry are always quoted so we will remove these as well.
					pathToHTMLExecutable = pathToHTMLExecutable.substring(0, pos - 1);
					pathToHTMLExecutable = pathToHTMLExecutable.substring(1, pathToHTMLExecutable.length() - 1);
				}
				else
				{
					pathToHTMLExecutable = pathToHTMLExecutable.substring(1, pathToHTMLExecutable.length() - 1);
				}
			}

			// Check the validity of these associations.
			File htmlEditor = new File(pathToHTMLExecutable);
			if ( !htmlEditor.exists() )
			{
				throw new IOException(Translations.getString("RPWAPPLICATION_45") //$NON-NLS-1$
					+ pathToHTMLExecutable +
					Translations.getString("RPWAPPLICATION_46") + "\n" + Translations.getString("RPWAPPLICATION_47")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}

			// Otherwise we have a valid program association so invoke it on the file.
			Runtime rt = Runtime.getRuntime();
			rt.exec(pathToHTMLExecutable + " " + fileName); //$NON-NLS-1$
		}
		else if ( fileName.endsWith(".gif") ) //$NON-NLS-1$
		{
			// Get the association for this type of file.
			if ( pathToGIFExecutable.equals(StringConstants.useSystemDefault) || pathToGIFExecutable.equals(StringConstants.missingResourceError) )
			{
				RegistryService theService = RegistryService.getInstance();
				try {
					pathToGIFExecutable = theService.getEditor(".GIF");
				} catch ( EnvironmentException ee ) {
					// Cannot find the value for this key in the registry. Throw an IO Exception.
					throw new IOException(Translations.getString("RPWAPPLICATION_48") //$NON-NLS-1$
										  + Translations.getString("RPWAPPLICATION_49")); //$NON-NLS-1$
				}

				// Need to strip the quotes off the value.
				pathToGIFExecutable = pathToGIFExecutable.substring(1, pathToGIFExecutable.length() - 1);
			}

			// Check the validity of these associations.
			File gifEditor = new File(pathToGIFExecutable);
			if ( !gifEditor.exists() )
			{
				throw new IOException(Translations.getString("RPWAPPLICATION_50") //$NON-NLS-1$
					+ pathToGIFExecutable +
					Translations.getString("RPWAPPLICATION_51") + "\n" + Translations.getString("RPWAPPLICATION_52")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}

			// Otherwise we have a valid program association so invoke it on the file.
			Runtime rt = Runtime.getRuntime();
			rt.exec(pathToGIFExecutable + " " + "\"" + fileName + "\""); //$NON-NLS-1$
		}
		else if ( fileName.endsWith(".jpeg") || fileName.endsWith(".jpg") ) //$NON-NLS-1$ //$NON-NLS-2$
		{
			// Get the association for this type of file.
			if ( pathToJPEGExecutable.equals(StringConstants.useSystemDefault) || pathToJPEGExecutable.equals(StringConstants.missingResourceError) )
			{
				RegistryService theService = RegistryService.getInstance();
				try {
					pathToJPEGExecutable = theService.getEditor(".JPEG");
				} catch ( EnvironmentException ee ) {
					// Cannot find the value for this key in the registry. Throw an IOException.
					throw new IOException(Translations.getString("RPWAPPLICATION_53") //$NON-NLS-1$
										  + Translations.getString("RPWAPPLICATION_54")); //$NON-NLS-1$
				}

				// Need to strip the quotes off.
				pathToJPEGExecutable = pathToJPEGExecutable.substring(1, pathToJPEGExecutable.length() - 1);
			}

			// Check the validity of these associations.
			File jpegEditor = new File(pathToJPEGExecutable);
			if ( !jpegEditor.exists() )
			{
				throw new IOException(Translations.getString("RPWAPPLICATION_55") //$NON-NLS-1$
					+ pathToJPEGExecutable +
					Translations.getString("RPWAPPLICATION_56") + "\n" + Translations.getString("RPWAPPLICATION_57")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}

			// Otherwise we have a valid program association so invoke it on the file.
			Runtime rt = Runtime.getRuntime();
			rt.exec(pathToJPEGExecutable + " " + "\"" + fileName + "\""); //$NON-NLS-1$
		}
		else
		{
			// Ask the user for an application to open the file with.
			if ( pathToUnknown == null )
			{
				RPWAlertDlg.showInformationMessage(Translations.getString("RPWAPPLICATION_58"), //$NON-NLS-1$
													Translations.getString("RPWAPPLICATION_59")); //$NON-NLS-1$
				RPWFileChooserDlg theDlg = new RPWFileChooserDlg(RPWFileChooserDlg.FILES_AND_DIRECTORIES, new JFrame());
				boolean theResult = theDlg.display(Translations.getString("RPWAPPLICATION_60"), RPWFileChooserDlg.OPEN_DIALOG); //$NON-NLS-1$

                if ( theResult )
                {
                   pathToUnknown = theDlg.getLibraryPath();
                }
                else
                {
                   return;
                }
			}

			// Now invoke the entered application with the input fileName.
			Runtime rt = Runtime.getRuntime();
			rt.exec(pathToUnknown + " " + "\"" + fileName + "\""); //$NON-NLS-1$
		}
	}

	//----------------------------------------------------------------------------
	// Invoke the browser for HTML files.
	public void invokeBrowser(String fileName)
		throws IOException
	{
		// Check if the constructed filename actually exists in the local filesystem or a valid URL path and if
		// not then return an error.
		if (fileName.toLowerCase().startsWith("http:"))  //$NON-NLS-1$
		{
			try {
				URL theURL = new URL(fileName);
			} catch (MalformedURLException mue) {
				throw new IOException( Translations.getString("RPWAPPLICATION_61") //$NON-NLS-1$
				+ " " //$NON-NLS-1$
				+ fileName
				+ " " //$NON-NLS-1$
				+ Translations.getString("RPWAPPLICATION_62")); //$NON-NLS-1$
			}
		}
		else {
			File myFile = new File(fileName);

			if ( !myFile.exists() )
			{
				throw new IOException( Translations.getString("RPWAPPLICATION_63") //$NON-NLS-1$
					+ " " //$NON-NLS-1$
					+ fileName
					+ " " //$NON-NLS-1$
					+ Translations.getString("RPWAPPLICATION_64")); //$NON-NLS-1$
			}
		}


		// Find the application to load the provided files with.
		if ( ( theUserPref.getViewingApplication(".htm").equals(StringConstants.useSystemDefault)
			|| theUserPref.getViewingApplication(".htm").equals(StringConstants.missingResourceError)) )
		{
			RegistryService theService = RegistryService.getInstance();
			try {
				pathToBrowser = theService.getViewer(".HTML");
			} catch ( EnvironmentException ee ) {
				// Cannot find the value for this key in the registry.
				throw new IOException(Translations.getString("RPWAPPLICATION_65") //$NON-NLS-1$
					+ "\n" + Translations.getString("RPWAPPLICATION_66")); //$NON-NLS-1$ //$NON-NLS-2$
			}

			// Now invoke the entered application with the input fileName.
			Runtime rt = Runtime.getRuntime();
			rt.exec(pathToBrowser + " " + fileName); //$NON-NLS-1$
		}
		else
		{
			String application = theUserPref.getViewingApplication(".htm");
			application = CommonFunctions.formatPath(application);
			
			// Check that the assigned application is valid.
			File appFile = new File(application);
			if ( !appFile.exists() )
			{
				throw new IOException(Translations.getString("RPWAPPLICATION_67") + application + Translations.getString("RPWAPPLICATION_68") //$NON-NLS-1$ //$NON-NLS-2$
										+ "\n" + Translations.getString("RPWAPPLICATION_69")); //$NON-NLS-1$ //$NON-NLS-2$

			}

			// Everything OK then run the application.
			Runtime rt = Runtime.getRuntime();
			rt.exec(application + " " + fileName); //$NON-NLS-1$
		}
	}
}
