package org.apache.lucene.analysis.cn;

import java.util.Hashtable;

import org.apache.lucene.analysis.Token;
import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;

/**
 * Title: ChineseFilter
 * Description: Filter with a stop word table
 *              Rule: No digital is allowed.
 *                    English word/token should larger than 1 character.
 *                    One Chinese character as one Chinese word.
 * TO DO:
 *   1. Add Chinese stop words, such as \ue400
 *   2. Dictionary based Chinese word extraction
 *   3. Intelligent Chinese word extraction
 *
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Yiyi Sun
 * @version 1.0
 *
 */
/**
 * Modified by fchong@rational.com on December 9, 2002, for Rational
 * use.
 * 
 * Copyright 2001 - 2002 Rational Software Corporation. All Rights Reserved.
 * This software contains proprietary and confidential information of
 * Rational and its suppliers. Use, disclosure or reproduction is
 * prohibited without the prior express written consent of Rational.
 */

public final class ChineseFilter extends TokenFilter {

    private Hashtable _stopTable;

    public ChineseFilter( Hashtable stopTable,
                           TokenStream in) 
    {
        input = in;

        _stopTable = stopTable;
    }

    public final Token next() throws java.io.IOException {

        for (Token token = input.next(); token != null; token = input.next()) {
            String text = token.termText();

            if (_stopTable.get(text) == null) {
                switch (Character.getType(text.charAt(0))) {

				// June 3, 2003 - fchong added support for digits for Rational
				case Character.DECIMAL_DIGIT_NUMBER:
				case Character.LETTER_NUMBER:
                case Character.LOWERCASE_LETTER:
                case Character.UPPERCASE_LETTER:

                    // English word/token should larger than 1 character.
                    if (text.length()>1) {
                        return token;
                    }
                    break;
                case Character.OTHER_LETTER:

                    // One Chinese character as one Chinese word.
                    // Chinese word extraction to be added later here.

                    return token;
                }

            }

        }
        return null;
    }

}